using System;
using System.Collections.Generic;
using System.Text;

using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.ServiceAccessLayer.Caching
{
    /// <summary>
    /// Diese Klasse stellt Dienst fr eine Cacheverwaltung fr die Ergebnisse
    /// von Anfragen der <see cref="IServiceAccessProvider"/> zur Verfgung. 
    /// </summary>
    internal sealed class CacheManager : ICacheExpirationPolicy
    {
        private TimeSpan validitiyDuration = new TimeSpan(0, 3, 0);
        private StaticCache<User> currentUserCache;
        private RequestCache<PerformanceInfoByStartEndCacheRequest, 
            PerformanceInfo[]> performanceInfoByStartEndCache;
        private RequestCache<PerformanceInfoOfMovieCacheRequest, 
            PerformanceInfo[]> performanceInfoOfMovieCache;
        private RequestCache<PerformanceInfoOfRoomCacheRequest, 
            PerformanceInfo[]> performanceInfoOfRoomCache;
        private RequestCache<PerformanceByGuidCacheRequest, 
            Performance> performanceByGuidCache; 
        private RequestCache<FreeSeatsCacheRequest, int> freeSeatsCache;
        private StaticCache<MovieInfo[]> moviesCache;
        private RequestCache<MovieByGuidCacheRequest, Movie> movieByGuidCache;
        private RequestCache<MoviesByStartDateCacheRequest, 
            MovieInfo[]> moviesByStartDateCache;
        private RequestCache<MoviesByUserInterestsCacheRequest, 
            MovieInfo[]> moviesByUserInterestsCache;
        private RequestCache<MoviesByTitleCacheRequest, 
            MovieInfo[]> moviesByTitleCache;
        private RequestCache<UserCacheRequest, User> userCache;
        private StaticCache<UserInfo[]> usersForNewsletterCache;
        private StaticCache<Room[]> roomsCache;
        private RequestCache<UndispatchedTicketsCacheRequest, 
            Ticket[]> undispatchedTicketsCache;
        private RequestCache<UndispatchedTicketsOfUserCacheRequest,
            Ticket[]> undispatchedTicketsOfUserCache;
        private StaticCache<NewsItem[]> newsCache;
        private RequestCache<NewsItemsByCategoriesCacheRequest, 
            NewsItem[]> newsItemsByCategoriesCache;
        private StaticCache<NewsCategory[]> newsCategoryCache;
        private StaticCache<Genre[]> genreCache;
        private StaticCache<Country[]> countryCache;
        private StaticCache<Distributor[]> distributorCache;
        private StaticCache<ParentalGuidance[]> parentalGuidanceCache;
        private StaticCache<Person[]> personCache;

        public CacheManager()
        {
            // Erzeuge alle Caches und setze den policy provider auf uns selbst

            currentUserCache = new StaticCache<User>(this);
            performanceInfoByStartEndCache = 
                new RequestCache<PerformanceInfoByStartEndCacheRequest, 
                    PerformanceInfo[]>(this);
            performanceInfoOfMovieCache = 
                new RequestCache<PerformanceInfoOfMovieCacheRequest, 
                    PerformanceInfo[]>(this);
            performanceInfoOfRoomCache = 
                new RequestCache<PerformanceInfoOfRoomCacheRequest, 
                    PerformanceInfo[]>(this);
            performanceByGuidCache = 
                new RequestCache<PerformanceByGuidCacheRequest, 
                    Performance>(this);
            freeSeatsCache = new RequestCache<FreeSeatsCacheRequest, 
                int>(this);
            moviesCache = new StaticCache<MovieInfo[]>(this);
            movieByGuidCache = new RequestCache<MovieByGuidCacheRequest, 
                Movie>(this);
            moviesByStartDateCache = 
                new RequestCache<MoviesByStartDateCacheRequest, 
                    MovieInfo[]>(this);
            moviesByUserInterestsCache = 
                new RequestCache<MoviesByUserInterestsCacheRequest, 
                    MovieInfo[]>(this);
            moviesByTitleCache = 
                new RequestCache<MoviesByTitleCacheRequest, MovieInfo[]>(this);
            userCache = 
                new RequestCache<UserCacheRequest, User>(this);
            usersForNewsletterCache = new StaticCache<UserInfo[]>(this);
            roomsCache = new StaticCache<Room[]>(this);
            undispatchedTicketsCache = 
                new RequestCache<UndispatchedTicketsCacheRequest, 
                    Ticket[]>(this);
            undispatchedTicketsOfUserCache = 
                new RequestCache<UndispatchedTicketsOfUserCacheRequest, 
                    Ticket[]>(this);
            newsCache = new StaticCache<NewsItem[]>(this);
            newsItemsByCategoriesCache = 
                new RequestCache<NewsItemsByCategoriesCacheRequest, 
                    NewsItem[]>(this);
            newsCategoryCache = new StaticCache<NewsCategory[]>(this);
            genreCache = new StaticCache<Genre[]>(this);
            countryCache = new StaticCache<Country[]>(this);
            distributorCache = new StaticCache<Distributor[]>(this);
            parentalGuidanceCache = new StaticCache<ParentalGuidance[]>(this);
            personCache = new StaticCache<Person[]>(this);
        }

        /// <summary>
        /// Ruft die Dauer der Gltigkeit von Cache Eintrgen ab oder legt einen
        /// neuen Wert fest.
        /// </summary>
        public TimeSpan ValidityDuration
        {
            get { return validitiyDuration; }
            set { validitiyDuration = value; }
        }

        /// <summary>
        /// Implementiert 
        /// <see cref="ICacheExpirationPolicy.IsExpired(DateTime)"/>. Pft, ob
        /// die Gltigkeit von einem zu dem gegebenen Teitpunkt erzeugter 
        /// Cache Eintrag abgelaufen ist. Das Ergebnis hngt von dem Wert von
        /// von <see cref="ValidityDuration"/> ab.
        /// </summary>
        /// <param name="created">Der Zeitpunkt an dem der Cach Eintrag erzeugt
        /// wurde.
        /// </param>
        public bool IsExpired(DateTime created)
        {         
            // Alle Eintrge im Cache sind solange gltig, bis der Zeitpunkt 
            // der Erzeugung plus Gltigkeit kleiner der jetzigen Uhrzeit ist.

            return DateTime.Now > created + validitiyDuration;
        }

        public void Clear()
        {
            // Leere alle Caches

            currentUserCache.Clear();
            performanceInfoByStartEndCache.Clear();
            performanceInfoOfMovieCache.Clear();
            performanceInfoOfRoomCache.Clear();
            performanceByGuidCache.Clear();
            freeSeatsCache.Clear();
            moviesCache.Clear();
            movieByGuidCache.Clear();
            moviesByStartDateCache.Clear();
            moviesByUserInterestsCache.Clear();
            moviesByTitleCache.Clear();
            userCache.Clear();
            usersForNewsletterCache.Clear();
            roomsCache.Clear();
            undispatchedTicketsCache.Clear();
            undispatchedTicketsOfUserCache.Clear();
            newsCache.Clear();
            newsItemsByCategoriesCache.Clear();
            newsCategoryCache.Clear();
            genreCache.Clear();
            countryCache.Clear();
            distributorCache.Clear();
            parentalGuidanceCache.Clear();
            personCache.Clear();
        }

        public StaticCache<User> CurrentUserCache
        {
            get { return currentUserCache; }
        }

        public RequestCache<PerformanceInfoByStartEndCacheRequest, 
            PerformanceInfo[]> PerformanceInfoByStartEndCache
        {
            get { return performanceInfoByStartEndCache; }
        }

        public RequestCache<PerformanceInfoOfMovieCacheRequest, 
            PerformanceInfo[]> PerformanceInfoOfMovieCache
        {
            get { return performanceInfoOfMovieCache; }
        }

        public RequestCache<PerformanceInfoOfRoomCacheRequest, 
            PerformanceInfo[]> PerformanceInfoOfRoomCache
        {
            get { return performanceInfoOfRoomCache; }
        }

        public RequestCache<PerformanceByGuidCacheRequest, 
            Performance> PerformanceByGuidCache
        {
            get { return performanceByGuidCache; }
        }

        public RequestCache<FreeSeatsCacheRequest, int> FreeSeatsCache
        {
            get { return freeSeatsCache; }
        }

        public StaticCache<MovieInfo[]> MoviesCache
        {
            get { return moviesCache; }
        }

        public RequestCache<MovieByGuidCacheRequest, Movie> MovieByGuidCache
        {
            get { return movieByGuidCache; }
        }

        public RequestCache<MoviesByStartDateCacheRequest, 
            MovieInfo[]> MoviesByStartDateCache
        {
            get { return moviesByStartDateCache; }
        }

        public RequestCache<MoviesByUserInterestsCacheRequest,
            MovieInfo[]> MoviesByUserInterestsCache
        {
            get { return moviesByUserInterestsCache; }
        }

        public RequestCache<MoviesByTitleCacheRequest,
            MovieInfo[]> MoviesByTitleCache
        {
            get { return moviesByTitleCache; }
        }

        public RequestCache<UserCacheRequest, User> UserCache
        {
            get { return userCache; }
        }

        public StaticCache<UserInfo[]> UsersForNewsletterCache
        {
            get { return usersForNewsletterCache; }
        }

        public StaticCache<Room[]> RoomsCache
        {
            get { return roomsCache; }
        }

        public RequestCache<UndispatchedTicketsCacheRequest,
            Ticket[]> UndispatchedTicketsCache
        {
            get { return undispatchedTicketsCache; }
        }

        public RequestCache<UndispatchedTicketsOfUserCacheRequest,
            Ticket[]> UndispatchedTicketsOfUserCache
        {
            get { return undispatchedTicketsOfUserCache; }
        }

        public StaticCache<NewsItem[]> NewsCache
        {
            get { return newsCache; }
        }

        public RequestCache<NewsItemsByCategoriesCacheRequest, 
            NewsItem[]> NewsItemsByCategoriesCache
        {
            get { return newsItemsByCategoriesCache; }
        }

        public StaticCache<NewsCategory[]> NewsCategoryCache
        {
            get { return newsCategoryCache; }
        }

        public StaticCache<Genre[]> GenreCache
        {
            get { return genreCache; }
        }

        public StaticCache<Country[]> CountryCache
        {
            get { return countryCache; }
        }

        public StaticCache<Distributor[]> DistributorCache
        {
            get { return distributorCache; }
        }

        public StaticCache<ParentalGuidance[]> ParentalGuidanceCache
        {
            get { return parentalGuidanceCache; }
        }

        public StaticCache<Person[]> PersonCache
        {
            get { return personCache; }
        }
    }
}
