using System;
using System.Collections.Generic;
using System.Text;

using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.ServiceAccessLayer.Caching
{
    /// <summary>
    /// Dieser Provider dekoriert einen anderen Service Provider und erweitert
    /// ihn um Caching Funktionalitt.
    /// </summary>
    internal sealed class CachingDataProvider : IServiceAccessProvider
    {
        private IServiceAccessProvider dataProvider;
        private CacheManager cacheManager = new CacheManager();

        public CachingDataProvider(IServiceAccessProvider dataProvider)
        {
            if (dataProvider == null)
                throw new ArgumentNullException("dataProvider");

            this.dataProvider = dataProvider;
        }

        public TimeSpan CacheValidityDuration
        {
            get { return cacheManager.ValidityDuration; }
            set { cacheManager.ValidityDuration = value; }
        }

        #region Cache Clearing

        public void ClearCaches()
        {
            // Leere alle Caches.
            cacheManager.Clear();
        }

        private void ClearPerformanceCaches()
        {
            // Leere nur die Caches, die von Vorstellungen abhngen.
            cacheManager.PerformanceByGuidCache.Clear();
            cacheManager.PerformanceInfoByStartEndCache.Clear();
            cacheManager.PerformanceInfoOfMovieCache.Clear();
            cacheManager.PerformanceInfoOfRoomCache.Clear();
        }

        private void ClearMovieCaches()
        {
            // Leere nur die Caches, die von Fimen abhngen.
            cacheManager.MovieByGuidCache.Clear();
            cacheManager.MoviesByUserInterestsCache.Clear();
            cacheManager.MoviesByStartDateCache.Clear();
            cacheManager.MoviesByTitleCache.Clear();
            cacheManager.MoviesCache.Clear();
        }

        private void ClearUserCaches()
        {
            // Leere nur die Caches, die von Benutzern abhngen.
            cacheManager.UserCache.Clear();
            cacheManager.UsersForNewsletterCache.Clear();
            cacheManager.MoviesByUserInterestsCache.Clear();
        }

        private void ClearRoomCaches()
        {
            // Leere nur die Caches, die von Kinosalen abhngen.
            cacheManager.RoomsCache.Clear();
        }

        private void ClearTicketCaches()
        {
            // Leere nur die Caches, die von Tickets abhngen.
            cacheManager.UndispatchedTicketsCache.Clear();
            cacheManager.UndispatchedTicketsOfUserCache.Clear();
            cacheManager.FreeSeatsCache.Clear();
        }

        private void ClearNewsItemCaches()
        {
            // Leere nur die Caches, die von News Eintrgen abhngen.
            cacheManager.NewsCache.Clear();
            cacheManager.NewsItemsByCategoriesCache.Clear();
        }

        private void ClearNewsCategoryCaches()
        {
            // Leere nur die Caches, die von News Kategorien abhngen.
            cacheManager.NewsCategoryCache.Clear();
        }

        private void ClearGenreCategoryCaches()
        {
            // Leere nur die Caches, die von Genres abhngen.
            cacheManager.GenreCache.Clear();
        }

        private void ClearCountryCaches()
        {
            // Leere nur die Caches, die von Lndern abhngen.
            cacheManager.CountryCache.Clear();
        }

        private void ClearDistributorCaches()
        {
            // Leere nur die Caches, die von Verleihern abhngen.
            cacheManager.DistributorCache.Clear();
        }

        private void ClearParentalGuidanceCaches()
        {
            // Leere nur die Caches, die von Altersfreigaben abhngen.
            cacheManager.ParentalGuidanceCache.Clear();
        }

        private void ClearPersonCaches()
        {
            // Leere nur die Caches, die von Personen abhngen.
            cacheManager.PersonCache.Clear();
        }

        #endregion

        #region Session

        public void ConnectToSession(string sessionId)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.ConnectToSession(sessionId);
        }

        public string SessionId
        {
            get
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                return dataProvider.SessionId;
            }
        }

        public User LogOn(string userName, string password)
        {
            // Vor der Anmeldung sollten alle Caches geleert werden.
            ClearCaches();

            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            return dataProvider.LogOn(userName, password);
        }

        public void LogOff()
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.LogOff();

            // Nach dem Abmelden sollten alle Caches geleert, da sie
            // nicht mehr bentigt werden.
            ClearCaches();
        }

        public void ChangePassword(string currentPassword, string newPassword)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.ChangePassword(currentPassword, newPassword);
        }

        public void ResetPassword(string userName)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.ResetPassword(userName);
        }

        public User GetCurrentUser()
        {
            User currentUser;

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.CurrentUserCache.Lookup(out currentUser))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                currentUser = dataProvider.GetCurrentUser();
                cacheManager.CurrentUserCache.Store(currentUser);
            }

            return currentUser;
        }

        #endregion

        #region Performance

        public PerformanceInfo[] GetPerformances(DateTime startDate,
            DateTime endDate)
        {
            PerformanceInfo[] result;
            PerformanceInfoByStartEndCacheRequest cacheRequest =
                new PerformanceInfoByStartEndCacheRequest(startDate, endDate);

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.PerformanceInfoByStartEndCache.Lookup(
                cacheRequest, out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetPerformances(startDate, endDate);
                cacheManager.PerformanceInfoByStartEndCache.Store(cacheRequest,
                    result);
            }

            return result;
        }

        public PerformanceInfo[] GetPerformancesOfMovie(Guid movieId)
        {
            PerformanceInfo[] result;
            PerformanceInfoOfMovieCacheRequest cacheRequest =
                new PerformanceInfoOfMovieCacheRequest(movieId);

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.PerformanceInfoOfMovieCache.Lookup(cacheRequest,
                out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetPerformancesOfMovie(movieId);
                cacheManager.PerformanceInfoOfMovieCache.Store(cacheRequest,
                    result);
            }

            return result;
        }

        public PerformanceInfo[] GetPerformancesOfRoom(Guid roomId,
            DateTime startDate, DateTime endDate)
        {
            PerformanceInfo[] result;
            PerformanceInfoOfRoomCacheRequest cacheRequest =
                new PerformanceInfoOfRoomCacheRequest(roomId, startDate,
                    endDate);

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.PerformanceInfoOfRoomCache.Lookup(cacheRequest,
                out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetPerformancesOfRoom(roomId, startDate,
                    endDate);
                cacheManager.PerformanceInfoOfRoomCache.Store(cacheRequest,
                    result);
            }

            return result;
        }

        public Performance GetPerformance(Guid performanceId)
        {
            Performance result;
            PerformanceByGuidCacheRequest cacheRequest =
                new PerformanceByGuidCacheRequest(performanceId);

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.PerformanceByGuidCache.Lookup(cacheRequest,
                out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetPerformance(performanceId);
                cacheManager.PerformanceByGuidCache.Store(cacheRequest, result);
            }

            return result;
        }

        public void CreatePerformance(Performance performance)
        {
            if (performance == null)
                throw new ArgumentNullException("performance");

            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.CreatePerformance(performance);

            // Nun alle abhngien Caches leeren.
            ClearPerformanceCaches();
        }

        public void UpdatePerformance(Performance performance)
        {
            if (performance == null)
                throw new ArgumentNullException("performance");

            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.UpdatePerformance(performance);

            // Nun alle abhngien Caches leeren.
            ClearPerformanceCaches();
        }

        public void DeletePerformance(Guid performanceId)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.DeletePerformance(performanceId);

            // Nun alle abhngien Caches leeren.
            ClearPerformanceCaches();
        }

        public int GetFreeSeats(Guid performanceId)
        {
            int result;
            FreeSeatsCacheRequest cacheRequest = new FreeSeatsCacheRequest(
                performanceId);

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.FreeSeatsCache.Lookup(cacheRequest, out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetFreeSeats(performanceId);
                cacheManager.FreeSeatsCache.Store(cacheRequest, result);
            }

            return result;
        }

        #endregion

        #region Movie

        public MovieInfo[] GetMovies()
        {
            MovieInfo[] result;

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.MoviesCache.Lookup(out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetMovies();
                cacheManager.MoviesCache.Store(result);
            }

            return result;
        }

        public MovieInfo[] GetMovies(DateTime startDate)
        {
            MovieInfo[] result;
            MoviesByStartDateCacheRequest cacheRequest =
                new MoviesByStartDateCacheRequest(startDate);

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.MoviesByStartDateCache.Lookup(cacheRequest,
                out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetMovies(startDate);
                cacheManager.MoviesByStartDateCache.Store(cacheRequest, result);
            }

            return result;
        }

        public MovieInfo[] GetMoviesByUserInterests(Guid userId)
        {
            MovieInfo[] result;
            MoviesByUserInterestsCacheRequest cacheRequest =
                new MoviesByUserInterestsCacheRequest(userId);

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.MoviesByUserInterestsCache.Lookup(
                cacheRequest, out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetMoviesByUserInterests(userId);
                cacheManager.MoviesByUserInterestsCache.Store(cacheRequest,
                    result);
            }

            return result;
        }

        public MovieInfo[] GetMoviesByTitle(string movieName)
        {
            if (String.IsNullOrEmpty(movieName))
                throw new ArgumentException(Resource.ArgumentNullOrEmptyString,
                    "movieTitle");

            MovieInfo[] result;
            MoviesByTitleCacheRequest cacheRequest =
                new MoviesByTitleCacheRequest(movieName);

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.MoviesByTitleCache.Lookup(cacheRequest,
                out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetMoviesByTitle(movieName);
                cacheManager.MoviesByTitleCache.Store(cacheRequest, result);
            }

            return result;
        }

        public Movie GetMovie(Guid movieId)
        {
            Movie result;
            MovieByGuidCacheRequest cacheRequest =
                new MovieByGuidCacheRequest(movieId);

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.MovieByGuidCache.Lookup(cacheRequest, out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetMovie(movieId);
                cacheManager.MovieByGuidCache.Store(cacheRequest, result);
            }

            return result;
        }

        public void RateMovie(Guid movieId, int rating)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.RateMovie(movieId, rating);

            // Nun alle abhngien Caches leeren.
            ClearMovieCaches();
        }

        public void CreateMovie(Movie movie)
        {
            if (movie == null)
                throw new ArgumentNullException("movie");

            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.CreateMovie(movie);

            // Nun alle abhngien Caches leeren.
            ClearMovieCaches();
        }

        public void UpdateMovie(Movie movie)
        {
            if (movie == null)
                throw new ArgumentNullException("movie");

            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.UpdateMovie(movie);

            // Nun alle abhngien Caches leeren.
            ClearMovieCaches();
        }

        public void DeleteMovie(Guid movieId)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.DeleteMovie(movieId);

            // Nun alle abhngien Caches leeren.
            ClearMovieCaches();
        }

        #endregion

        #region User

        public User GetUser(Guid userId)
        {
            User result;
            UserCacheRequest cacheRequest =
                new UserCacheRequest(userId);

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.UserCache.Lookup(cacheRequest, out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetUser(userId);
                cacheManager.UserCache.Store(cacheRequest, result);
            }

            return result;
        }

        public UserInfo[] GetUsersForNewsletter()
        {
            UserInfo[] result;

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.UsersForNewsletterCache.Lookup(out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetUsersForNewsletter();
                cacheManager.UsersForNewsletterCache.Store(result);
            }

            return result;
        }

        public void CreateUser(User user, string password)
        {
            if (user == null)
                throw new ArgumentNullException("user");

            if (password == null)
                throw new ArgumentNullException("password");

            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.CreateUser(user, password);

            // Nun alle abhngien Caches leeren.
            ClearUserCaches();
        }

        public void UpdateUser(User user)
        {
            if (user == null)
                throw new ArgumentNullException("user");

            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.UpdateUser(user);

            // Nun alle abhngien Caches leeren.
            ClearUserCaches();
        }

        public void DeleteUser(Guid userId)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.DeleteUser(userId);

            // Nun alle abhngien Caches leeren.
            ClearUserCaches();
        }

        #endregion

        #region Room

        public Room[] GetRooms()
        {
            Room[] result;

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.RoomsCache.Lookup(out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetRooms();
                cacheManager.RoomsCache.Store(result);
            }

            return result;
        }

        public void CreateRoom(Room room)
        {
            if (room == null)
                throw new ArgumentNullException("room");

            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.CreateRoom(room);

            // Nun alle abhngien Caches leeren.
            ClearRoomCaches();
        }

        public void UpdateRoom(Room room)
        {
            if (room == null)
                throw new ArgumentNullException("room");

            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.UpdateRoom(room);

            // Nun alle abhngien Caches leeren.
            ClearRoomCaches();
        }

        public void DeleteRoom(Guid roomId)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.DeleteRoom(roomId);

            // Nun alle abhngien Caches leeren.
            ClearRoomCaches();
        }

        #endregion

        #region Ticket

        public string BuyTicket(Guid performanceId, int quantity)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            string result = dataProvider.BuyTicket(performanceId, quantity);
            ClearTicketCaches();

            return result;
        }

        public void SellTicket(Guid performanceId, int quantity)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.SellTicket(performanceId, quantity);

            // Nun alle abhngien Caches leeren.
            ClearTicketCaches();
        }

        public Ticket[] GetUndispatchedTickets(Guid userId)
        {
            Ticket[] result;
            UndispatchedTicketsOfUserCacheRequest cacheRequest =
                new UndispatchedTicketsOfUserCacheRequest(userId);

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.UndispatchedTicketsOfUserCache.Lookup(
                cacheRequest, out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetUndispatchedTickets(userId);
                cacheManager.UndispatchedTicketsOfUserCache.Store(cacheRequest,
                    result);
            }

            return result;
        }

        public Ticket[] GetUndispatchedTickets(string ticketCode)
        {
            if (String.IsNullOrEmpty(ticketCode))
                throw new ArgumentException(Resource.ArgumentNullOrEmptyString,
                    "ticketCode");

            Ticket[] result;
            UndispatchedTicketsCacheRequest cacheRequest =
                new UndispatchedTicketsCacheRequest(ticketCode);

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.UndispatchedTicketsCache.Lookup(cacheRequest,
                out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetUndispatchedTickets(ticketCode);
                cacheManager.UndispatchedTicketsCache.Store(cacheRequest,
                    result);
            }

            return result;
        }

        public void MarkTicketAsDispatched(Ticket ticket)
        {
            if (ticket == null)
                throw new ArgumentNullException("ticket");

            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.MarkTicketAsDispatched(ticket);

            // Nun alle abhngien Caches leeren.
            ClearTicketCaches();
        }

        #endregion

        #region News

        public NewsItem[] GetNewsItems()
        {
            NewsItem[] result;

            if (!cacheManager.NewsCache.Lookup(out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetNewsItems();
                cacheManager.NewsCache.Store(result);
            }

            return result;
        }

        public NewsItem[] GetNewsItemsByCategories(int[] numbers)
        {
            if (numbers == null)
                throw new ArgumentNullException("numbers");

            if (numbers.Length == 0)
                throw new ArgumentException(Resource.ArgumentEmptyArray,
                    "number");

            NewsItem[] result;
            NewsItemsByCategoriesCacheRequest cacheRequest =
                new NewsItemsByCategoriesCacheRequest(numbers);

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.NewsItemsByCategoriesCache.Lookup(cacheRequest,
                out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetNewsItemsByCategories(numbers);
                cacheManager.NewsItemsByCategoriesCache.Store(cacheRequest,
                    result);
            }

            return result;
        }

        public void CreateNewsItem(NewsItem newsItem)
        {
            if (newsItem == null)
                throw new ArgumentNullException("newsItem");

            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.CreateNewsItem(newsItem);

            // Nun alle abhngien Caches leeren.
            ClearNewsItemCaches();
        }

        public void UpdateNewsItem(NewsItem newsItem)
        {
            if (newsItem == null)
                throw new ArgumentNullException("newsItem");

            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.UpdateNewsItem(newsItem);

            // Nun alle abhngien Caches leeren.
            ClearNewsItemCaches();
        }

        public void DeleteNewsItem(Guid newsItemId)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.DeleteNewsItem(newsItemId);

            // Nun alle abhngien Caches leeren.
            ClearNewsItemCaches();
        }

        #endregion

        #region News Category

        public NewsCategory[] GetNewsCategories()
        {
            NewsCategory[] result;

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.NewsCategoryCache.Lookup(out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetNewsCategories();
                cacheManager.NewsCategoryCache.Store(result);
            }

            return result;
        }

        public void CreateNewsCategory(NewsCategory category)
        {
            if (category == null)
                throw new ArgumentNullException("category");

            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.CreateNewsCategory(category);

            // Nun alle abhngien Caches leeren.
            ClearNewsCategoryCaches();
        }

        public void UpdateNewsCategory(NewsCategory category)
        {
            if (category == null)
                throw new ArgumentNullException("category");

            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.UpdateNewsCategory(category);

            // Nun alle abhngien Caches leeren.
            ClearNewsCategoryCaches();
        }

        public void DeleteNewsCategory(Guid categoryId)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.DeleteNewsCategory(categoryId);

            // Nun alle abhngien Caches leeren.
            ClearNewsCategoryCaches();
        }

        #endregion

        #region Genre

        public Genre[] GetGenres()
        {
            Genre[] result;

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.GenreCache.Lookup(out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetGenres();
                cacheManager.GenreCache.Store(result);
            }

            return result;
        }

        public void CreateGenre(Genre genre)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.CreateGenre(genre);

            // Nun alle abhngien Caches leeren.
            ClearGenreCategoryCaches();
        }

        public void UpdateGenre(Genre genre)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.UpdateGenre(genre);

            // Nun alle abhngien Caches leeren.
            ClearGenreCategoryCaches();
        }

        public void DeleteGenre(Guid genreId)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.DeleteGenre(genreId);

            // Nun alle abhngien Caches leeren.
            ClearGenreCategoryCaches();
        }

        #endregion

        #region Country

        public Country[] GetCountries()
        {
            Country[] result;

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.CountryCache.Lookup(out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetCountries();
                cacheManager.CountryCache.Store(result);
            }

            return result;
        }

        public void CreateCountry(Country country)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.CreateCountry(country);

            // Nun alle abhngien Caches leeren.
            ClearCountryCaches();
        }

        public void UpdateCountry(Country country)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.UpdateCountry(country);

            // Nun alle abhngien Caches leeren.
            ClearCountryCaches();
        }

        public void DeleteCountry(Guid countryId)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.DeleteCountry(countryId);

            // Nun alle abhngien Caches leeren.
            ClearCountryCaches();
        }

        #endregion

        #region Distributor

        public Distributor[] GetDistributors()
        {
            Distributor[] result;

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.DistributorCache.Lookup(out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetDistributors();
                cacheManager.DistributorCache.Store(result);
            }

            return result;
        }

        public void CreateDistributor(Distributor distributor)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.CreateDistributor(distributor);

            // Nun alle abhngien Caches leeren.
            ClearDistributorCaches();
        }

        public void UpdateDistributor(Distributor distributor)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.UpdateDistributor(distributor);

            // Nun alle abhngien Caches leeren.
            ClearDistributorCaches();
        }

        public void DeleteDistributor(Guid distributorId)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.DeleteDistributor(distributorId);

            // Nun alle abhngien Caches leeren.
            ClearDistributorCaches();
        }

        #endregion

        #region Parental Guidance

        public ParentalGuidance[] GetParentalGuidances()
        {
            ParentalGuidance[] result;

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.ParentalGuidanceCache.Lookup(out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetParentalGuidances();
                cacheManager.ParentalGuidanceCache.Store(result);
            }

            return result;
        }

        public void CreateParentalGuidance(ParentalGuidance parentalGuidance)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.CreateParentalGuidance(parentalGuidance);

            // Nun alle abhngien Caches leeren.
            ClearParentalGuidanceCaches();
        }

        public void UpdateParentalGuidance(ParentalGuidance parentalGuidance)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.UpdateParentalGuidance(parentalGuidance);

            // Nun alle abhngien Caches leeren.
            ClearParentalGuidanceCaches();
        }

        public void DeleteParentalGuidance(Guid parentalGuidanceId)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.DeleteParentalGuidance(parentalGuidanceId);

            // Nun alle abhngien Caches leeren.
            ClearParentalGuidanceCaches();
        }

        #endregion

        #region Person

        public Person[] GetPersons()
        {
            Person[] result;

            // Bevor der Aufruf delegiert wird, wird zunchst geprft, ob die
            // Anfrage nicht schon im Cache ist.
            if (!cacheManager.PersonCache.Lookup(out result))
            {
                // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
                // Daten Provider delegiert.
                result = dataProvider.GetPersons();
                cacheManager.PersonCache.Store(result);
            }

            return result;
        }

        public void CreatePerson(Person person)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.CreatePerson(person);

            // Nun alle abhngien Caches leeren.
            ClearPersonCaches();
        }

        public void UpdatePerson(Person person)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.UpdatePerson(person);

            // Nun alle abhngien Caches leeren.
            ClearPersonCaches();
        }

        public void DeletePerson(Guid personId)
        {
            // Dieser Aufruf wird direkt an den dekorierten, eigentlichen
            // Daten Provider delegiert.
            dataProvider.DeletePerson(personId);

            // Nun alle abhngien Caches leeren.
            ClearPersonCaches();
        }

        #endregion
    }
}