using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace AdventureWorks.Cinema.ServiceAccessLayer.Caching
{
    /// <summary>
    /// Reprsentiert einen Cache fr parametrisierte Anfragen.
    /// </summary>
    /// <typeparam name="TRequest">Spezifiziert die Klasse der Anfrage (muss von
    /// <see cref="Request"/> abgeleitet sein).</typeparam>
    /// <typeparam name="TResponse">Spezifiziert den Datentyp des Ergebnisses
    /// der Anfrage.</typeparam>
    internal class RequestCache<TRequest, TResponse>
        where TRequest : Request
    {
        /// <summary>
        /// Wir fr die Zuordnung vom Zeitpunkt der Erzeugung des Cache 
        /// Eintrages verwendet.
        /// </summary>
        private class Entry
        {
            public DateTime Created;
            public TResponse Response;
        }

        private ICacheExpirationPolicy expirationPolicy;
        private Dictionary<TRequest, Entry> entries = new Dictionary<TRequest, 
            Entry>();

        public RequestCache(ICacheExpirationPolicy cachePolicyProvider)
        {
            if (cachePolicyProvider == null)
                throw new ArgumentNullException("expirationPolicy");

            this.expirationPolicy = cachePolicyProvider;
        }

        /// <summary>
        /// Entfernt alle gespeicherten Anfragen und deren zugeordnete 
        /// Antworten.
        /// </summary>
        public void Clear()
        {
            entries.Clear();
        }

        /// <summary>
        /// Fgt eine Antwort zu einer Anfrage hinzu. Falls zu der Anfrage
        /// bereits eine Antwort gespeichert wurde, wird diese durch die neue
        /// Antwort ersetzt.
        /// </summary>
        /// <param name="request">Die Anfrage, fr die die Antwort gespeichert
        /// werden soll.</param>.
        /// <param name="response">Die Antwort der Anfrage.</param>.
        public void Store(TRequest request, TResponse response)
        {
            // Der Cache Eintrag wird mit dem aktuellen Datum und Uhrzeit
            // markiert, damit die Gltigkeitsdauer fr Eintrag kontrolliert
            // werden kann.

            Entry entry = new Entry();
            entry.Created = DateTime.Now;
            entry.Response = response;
            entries[request] = entry;
        }

        /// <summary>
        /// Prft, ob fr die gegebene Anfrage eine Antwort im Cache liegt und
        /// gibt diese zurck.
        /// </summary>
        /// <param name="request">Die Anfage, fr die eine Antwort angefordert
        /// wrd.</param>
        /// <param name="response">Falls eine Antwort fr die Anfrage im Cache
        /// liegt, wird <paramref name="response"/> der Wert der Antwort
        /// zugewiesen.</param>
        /// <remarks>
        /// Wenn der Cache Eintrag abgelofen ist (das wird mit einem Aufruf
        /// von <see cref="ICacheExpirationPolicy.IsExpired"/> kontrolliert),
        /// wird der Cache Eintrag entfernt und diese Methode verhlt sich, als
        /// wre keine Antwort im Cache gewesen.
        /// </remarks>
        /// <returns>Wenn eine Antwort zu der Anfrage im Cache vorhanden ist,
        /// ist der Rckgabewert <see langword="true"/>, andernfalls ist er
        /// <see langword="false."/>.
        /// </returns>
        public bool Lookup(TRequest request, out TResponse response)
        {
            // Stell sicher, dass wir dem Output Parameter einen gltigen
            // Wert zugewiesen haben.
            response = default(TResponse);

            // Versuche eine Antwort fr die Anfrage zu finden. Es wird das
            // Anfrageobjekt selbst als Schlssel zum Indizieren des 
            // Wrterbuchs verwendet.
            Entry entry;
            if (!entries.TryGetValue(request, out entry))
                return false;

            // Es wurde eine Antwort gefunden. Nun muss noch geprft werden,
            // ob die Antwort nicht schon abgelaufen ist.
            if (expirationPolicy.IsExpired(entry.Created))
            {
                // Der Cache Policy Provider hat entschieden, dass der Cache
                // Eintrag abgelaufen ist. Daher wird Antwort aus dem Cache
                // gelscht und eine negative Antwort erteilt.
                entries.Remove(request);
                return false;
            }

            // Es wurde eine nicht abgelaufene Antwort zu der Anfrage gefunden. 
            response = entry.Response;
            return true;
        }
    }
}
