using System;
using System.Collections.Generic;
using System.Text;

namespace AdventureWorks.Cinema.ServiceAccessLayer.Caching
{
    /// <summary>
    /// Reprsentiert einen nicht-parametrisierten Cache. Das bedeutet, dieser
    /// Cache hlt tatschlich nur eine Antwort.
    /// </summary>
    /// <typeparam name="TResponse">Spezifiert den Datentyp der Antwort.
    /// </typeparam>
    internal class StaticCache<TResponse>
    {
        private ICacheExpirationPolicy cachePolicyProvider;
        private bool valid;
        private TResponse response;
        private DateTime created;

        public StaticCache(ICacheExpirationPolicy cachePolicyProvider)
        {
            if (cachePolicyProvider == null)
                throw new ArgumentNullException("expirationPolicy");

            this.cachePolicyProvider = cachePolicyProvider;
        }

        /// <summary>
        /// Leert den Cache.
        /// </summary>
        public void Clear()
        {
            valid = false;
        }

        /// <summary>
        /// Speichert die Anwort im Cache. Falls bereits eine Antwort 
        /// gespeichert wurde, wird die alte Antwort durch die neue ersetzt.
        /// </summary>
        /// <param name="response">Die zu speichernde Antwort.</param>
        public void Store(TResponse response)
        {
            // Zur Kontrolle der Cache Gltigkeit muss neben der eigentlichen
            // Antwort auch der Zeitpunkt des Speicherns gespeichert werden.
            // Diese Information wird vom Cache Policy Provider verwendet, um
            // zu ermitteln, ob der Eintrag noch immer gltig ist.
            created = DateTime.Now;
            this.response = response;

            // Da der Antwortdatentyp ein generischer Datentyp ist, sind auch 
            // Werttypen als Antworten zulssig. Das bedeutet, NULL kann nicht 
            // als Indikator fr eine nicht vorhandene Antwort verwendet werden.
            // Stattdessen wir einfache das Flag valid verwendet.
            //
            // Hier muss auf es offensichtlich auf TRUE gesetzt werden.
            valid = true;
        }

        /// <summary>
        /// Prft, ob eine Antwort im Cache ist und gibt diese zurck.
        /// </summary>
        /// <param name="response">Wenn eine Antwort im Cache vorhanden ist,
        /// wird <paramref name="response"/> auf dessen Wert gesetz.</param>
        /// <remarks>
        /// Wenn der Cache Eintrag abgelofen ist (das wird mit einem Aufruf
        /// von <see cref="ICacheExpirationPolicy.IsExpired"/> kontrolliert),
        /// wird der Cache Eintrag entfernt und diese Methode verhlt sich, als
        /// wre keine Antwort im Cache gewesen.
        /// </remarks>
        /// <returns>Wenn eine Antwort zu der Anfrage im Cache vorhanden ist,
        /// ist der Rckgabewert <see langword="true"/>, andernfalls ist er
        /// <see langword="false."/>.
        /// </returns>
        public bool Lookup(out TResponse response)
        {
            if (valid && !cachePolicyProvider.IsExpired(created))
            {
                // Es ist ein Cache Eintrag vorhanden und der Cache Policy
                // Provider hat ihn fr gltig befunden.
                response = this.response;
                return true;
            }
            else
            {
                // Entweder es ist keine Antwort im Cache vorhanden oder
                // der Cache Policy Provider hat ihn fr ungltig befunden.
                // Der Cache Eintrag wird auf den Standardwert gesetzt (was
                // Objekte u.U. fr die Garbage Collection verfgbar macht)
                // und geben einen Wert zurck, der anzeigt, dass der Cache
                // keine Antwort enthlt.
                this.response = default(TResponse);
                response = default(TResponse);
                return false;
            }
        }
    }
}
