using System;
using System.Collections.Generic;
using System.Text;

using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.ServiceAccessLayer.SampleData
{
    /// <summary>
    /// Diese Klasse stellt einfache Methoden zur Verfgung um Zeilen des
    /// typisierten <see cref="System.Data.DataSet"/> 
    /// <see cref="CinemaDataSet"/> in Instanzen von Data Transfer Objects (DTO)
    /// zu konvertieren.
    /// </summary>
    internal static class CinemaDataSetHelper
    {
        internal const int PERSON_TYPE_ACTOR = 0;
        internal const int PERSON_TYPE_PRODUCER = 1;
        internal const int PERSON_TYPE_DIRECTOR = 2;

        /// <summary>
        /// Konvertiert eine Zeile in ein Data Transfer Object (DTO).
        /// </summary>
        /// <param name="row">Die zu konvertierende Zeile.</param>
        public static Distributor RowToDistributor(
            CinemaDataSet.DistributorRow row)
        {
            Distributor result = new Distributor();
            result.Id = row.Id;
            result.Name = row.Name;

            return result;
        }

        /// <summary>
        /// Konvertiert eine Zeile in ein Data Transfer Object (DTO).
        /// </summary>
        /// <param name="row">Die zu konvertierende Zeile.</param>
        public static Country RowToCountry(CinemaDataSet.CountryRow row)
        {
            Country result = new Country();
            result.Id = row.Id;
            result.Name = row.Name;

            return result;
        }

        /// <summary>
        /// Konvertiert eine Zeile in ein Data Transfer Object (DTO).
        /// </summary>
        /// <param name="row">Die zu konvertierende Zeile.</param>
        public static ParentalGuidance RowToParentalGuidance(
            CinemaDataSet.ParentalGuidanceRow row)
        {
            ParentalGuidance result = new ParentalGuidance();
            result.Id = row.Id;
            result.Name = row.Name;

            return result;
        }

        /// <summary>
        /// Konvertiert eine Zeile in ein Data Transfer Object (DTO).
        /// </summary>
        /// <param name="row">Die zu konvertierende Zeile.</param>
        public static Person RowToPerson(CinemaDataSet.PersonRow row)
        {
            Person result = new Person();
            result.Id = row.Id;
            result.Name = row.Name;

            return result;
        }

        /// <summary>
        /// Konvertiert eine Zeile in ein Data Transfer Object (DTO).
        /// </summary>
        /// <param name="row">Die zu konvertierende Zeile.</param>
        public static NewsCategory RowToNewsCategory(
            CinemaDataSet.NewsCategoryRow row)
        {
            NewsCategory result = new NewsCategory();
            result.Id = row.Id;
            result.Name = row.Name;

            return result;
        }

        /// <summary>
        /// Konvertiert eine Zeile in ein Data Transfer Object (DTO).
        /// </summary>
        /// <param name="row">Die zu konvertierende Zeile.</param>
        public static Genre RowToGenre(CinemaDataSet.GenreRow row)
        {
            Genre result = new Genre();
            result.Id = row.Id;
            result.Name = row.Name;

            return result;
        }

        /// <summary>
        /// Konvertiert eine Zeile in ein Data Transfer Object (DTO).
        /// </summary>
        /// <param name="row">Die zu konvertierende Zeile.</param>
        public static Movie RowToMovie(CinemaDataSet.MovieRow row)
        {
            Movie result = new Movie();
            result.Id = row.Id;
            result.Title = row.Title;
            result.OriginalTitle = row.OriginalTitle;
            result.Country = RowToCountry(row.CountryRow);
            result.Year = row.Year;
            result.StartDate = row.StartDate;
            result.Description = row.Description;
            result.Picture = null;
            result.Length = row.Length;
            result.ParentalGuidance = RowToParentalGuidance(
                row.ParentalGuidanceRow);
            result.Distributor = RowToDistributor(row.DistributorRow);

            List<Person> actors = new List<Person>();
            List<Person> directors = new List<Person>();
            List<Person> producers = new List<Person>();

            // Konvertiert jede Personenzeile in ein Personenobjekt.
            //
            // Das DataSet macht keine Unterscheidung zwischen Schauspielern,
            // Produzenten oder Regisseure. Alle werden in der gleichen Tabelle
            // gespeichert. Die Information in welche Gruppe eine Person fllt
            // ist in der Spalte ActorType hinterlegt.

            foreach (CinemaDataSet.Movie_PersonsRow moviePersonRow in 
                row.GetMovie_PersonsRows())
            {
                // Je nachdem in welche Gruppe die Person fllt wird ein anderes
                // Ziel an target zugewiesen.

                List<Person> target;
                switch (moviePersonRow.ActorType)
                {
                    case PERSON_TYPE_ACTOR:
                        target = actors;
                        break;

                    case PERSON_TYPE_PRODUCER:
                        target = producers;
                        break;

                    case PERSON_TYPE_DIRECTOR:
                        target = directors;
                        break;

                    default:
                        continue;
                }

                // Konvertiere die Personenzeile in ein Personenobjekt.
                Person person = RowToPerson(moviePersonRow.PersonRow);
                target.Add(person);
            }

            // Weise dem Filmobjekt alle Personengruppen zu.
            result.Actors = actors.ToArray();
            result.Directors = directors.ToArray();
            result.Producers = producers.ToArray();

            // Konvertiere alle Genrezeilen in Genre Objekte.
            List<Genre> genres = new List<Genre>();
            foreach (CinemaDataSet.Movie_GenresRow movieGenresRow in 
                row.GetMovie_GenresRows())
            {
                Genre genre = RowToGenre(movieGenresRow.GenreRow);
                genres.Add(genre);
            }

            // Weise dem Filmobjekt die Genreobjekte zu.
            result.Genres = genres.ToArray();

            return result;
        }

        /// <summary>
        /// Konvertiert eine Zeile in ein Data Transfer Object (DTO).
        /// </summary>
        /// <param name="row">Die zu konvertierende Zeile.</param>
        public static Performance RowToPerformance(
            CinemaDataSet.PerformanceRow row)
        {
            Performance result = new Performance();
            result.Id = row.Id;
            result.Movie = RowToMovie(row.MovieRow);
            result.Date = row.Date;
            result.Room = RowToRoom(row.RoomRow);
            result.Price = row.Price;
            return result;
        }

        /// <summary>
        /// Konvertiert eine Zeile in ein Data Transfer Object (DTO).
        /// </summary>
        /// <param name="row">Die zu konvertierende Zeile.</param>
        public static Room RowToRoom(CinemaDataSet.RoomRow row)
        {
            Room result = new Room();
            result.Id = row.Id;
            result.Name = row.Name;
            result.ScreenDiagonal = row.ScreenDiagonal;
            result.SeatCount = row.SeatCount;
            result.Inactive = row.Inactive;
            return result;
        }

        /// <summary>
        /// Konvertiert eine Zeile in ein Data Transfer Object (DTO).
        /// </summary>
        /// <param name="row">Die zu konvertierende Zeile.</param>
        public static User RowToUser(CinemaDataSet.UserRow row)
        {
            User result = new User();
            result.Id = row.Id;
            result.UserAccount = row.UserAccount;
            result.LastName = row.LastName;
            result.FirstNames = row.FirstName;
            result.Street = row.Street;
            result.ZipCode = row.ZipCode;
            result.City = row.City;
            result.Email = row.EMail;
            result.Newsletter = row.Newsletter;
            result.Inactive = row.Inactive;

            // Konvertiere alle Genrezeilen in Genreobjekte.
            List<Genre> genres = new List<Genre>();
            foreach (CinemaDataSet.Users_GenresRow userGenresRow in 
                row.GetUsers_GenresRows())
            {
                Genre genre = RowToGenre(userGenresRow.GenreRow);
                genres.Add(genre);
            }

            // Weise dem Benutzerobjekt die Genreobjekte zu.
            result.Genres = genres.ToArray();

            return result;
        }

        /// <summary>
        /// Konvertiert eine Zeile in ein Data Transfer Object (DTO).
        /// </summary>
        /// <param name="row">Die zu konvertierende Zeile.</param>
        public static UserInfo RowToUserInfo(CinemaDataSet.UserRow row)
        {
            UserInfo result = new UserInfo();
            result.Id = row.Id;
            result.UserAccount = row.UserAccount;
            result.Email = row.EMail;

            return result;
        }

        /// <summary>
        /// Konvertiert eine Zeile in ein Data Transfer Object (DTO).
        /// </summary>
        /// <param name="row">Die zu konvertierende Zeile.</param>
        public static NewsItem RowToNewsItem(CinemaDataSet.NewsItemRow row)
        {
            NewsItem result = new NewsItem();
            result.Id = row.Id;
            result.Subject = row.Subject;
            result.Text = row.Text;
            result.StartDate = row.StartDate;
            result.EndDate = row.EndDate;

            // Konvertiere alle Newskategoriezeilen in Newskategorieobjekte.
            List<NewsCategory> categories = new List<NewsCategory>();
            foreach (
                CinemaDataSet.NewItem_NewsCategoriesRow newsItemCategoriesRow 
                in row.GetNewItem_NewsCategoriesRows())
            {
                NewsCategory category = RowToNewsCategory(
                    newsItemCategoriesRow.NewsCategoryRow);
                categories.Add(category);
            }

            // Weise dem Newseintrag alle Newskategorieobjekte zu.
            result.Categories = categories.ToArray();
            return result;
        }

        /// <summary>
        /// Konvertiert eine Zeile in ein Data Transfer Object (DTO).
        /// </summary>
        /// <param name="row">Die zu konvertierende Zeile.</param>
        public static PerformanceInfo RowToPerformanceInfo(
            CinemaDataSet.PerformanceRow row)
        {
            Performance performance = RowToPerformance(row);

            PerformanceInfo result = new PerformanceInfo();
            result.Id = performance.Id;
            result.MovieInfo = new MovieInfo();
            result.MovieInfo.Id = performance.Movie.Id;
            result.MovieInfo.Length = performance.Movie.Length;
            result.MovieInfo.ParentalGuidance = 
                performance.Movie.ParentalGuidance;
            result.MovieInfo.Title = performance.Movie.Title;
            result.RoomName = performance.Room.Name;
            result.Date = performance.Date;
            return result;
        }

        /// <summary>
        /// Konvertiert eine Zeile in ein Data Transfer Object (DTO).
        /// </summary>
        /// <param name="row">Die zu konvertierende Zeile.</param>
        public static MovieInfo RowToMovieInfo(CinemaDataSet.MovieRow row)
        {
            Movie movie = RowToMovie(row);

            MovieInfo result = new MovieInfo();
            result.Id = movie.Id;
            result.Length = movie.Length;
            result.ParentalGuidance = movie.ParentalGuidance;
            result.Title = movie.Title;
            return result;
        }

        /// <summary>
        /// Konvertiert eine Zeile in ein Data Transfer Object (DTO).
        /// </summary>
        /// <param name="row">Die zu konvertierende Zeile.</param>
        internal static Ticket RowToTicket(CinemaDataSet.TicketsRow row)
        {
            Ticket result = new Ticket();
            result.Id = row.Id;
            result.IsDispatched = row.IsDispatched;
            result.IsPaid = row.IsPaid;
            result.Performance = RowToPerformance(row.PerformanceRow);
            result.Quantity = row.Quantity;
            result.TicketCode = row.TicketCode;
            result.User = RowToUser(row.UserRow);

            return result;
        }
    }
}