using System;
using System.Collections.Generic;
using System.Data;
using System.Globalization;
using System.IO;
using System.Text;

using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.ServiceAccessLayer.SampleData
{
    /// <summary>
    /// Diese Klasse reprsentiert einen Servie Provider, der keine erzeugenden,
    /// aktualisierenden oder lschenden Operationen durchfhrt (aber auch nicht
    /// verbietet). Dieser Service Provider wird von Clients benutzt, um einfach
    /// nur Daten aus einer XML Datei anzuzeigen.
    /// </summary>
    internal sealed class SampleDataProvider : IServiceAccessProvider
    {
        private User user;
        private CinemaDataSet dataSet;

        public SampleDataProvider(string fileName)
        {
            dataSet = new CinemaDataSet();
            try
            {
                dataSet.ReadXml(fileName);
            }
            catch (FileNotFoundException)
            {
                // Die Datei mit den Demodaten konnte nicht gefunden werden.

                // HINWEISE: FileNotFoundException.FileName kann leider nicht
                //           verwendet werden, da diese Eigenschaft auf dem
                //           Compact Framework nicht zur Verfgung steht. Um
                //           dennoch einen voll qualifiezerten Dateinamen zu
                //           erhalten (fileName knnten einen relativen Pfad
                //           enthalten) wird Path.GetFullPath() verwendet, da
                //           diese auch auf dem Compact Framework verfgbar
                //           ist.

                string fullyQualifiedPath = Path.GetFullPath(fileName);

                // Wirf eine Ausnahme mit einer besseren Fehlermeldung.
                //
                // HINWEIS: Die FileNotFoundException Ausnahme ist nicht
                //          verpackt, da ASP.NET Anwendungen standardmig die
                //          innerste Ausnahme auf der Fehlerseite anzeigen. In
                //          diesem Fall wrden ASP.NET Anwendungen leider nicht
                //          von der besseren Fehlermeldung profitieren.

                string msg = String.Format(CultureInfo.CurrentCulture,
                    Resource.SampleDataFileNotFound, fullyQualifiedPath);

                throw new ServiceAccessException(msg);
            }
        }

        #region Session

        public void ConnectToSession(string sessionId)
        {
            // HINWEIS: Diese Provider verwendet die User.Id als Session ID.

            if (String.IsNullOrEmpty(sessionId))
                throw new ArgumentException(Resource.ArgumentNullOrEmptyString);

            // Finde einen Benutzer mit der ID, die mit der gegebenen Session 
            // ID bereinstimmt.
            DataRow[] rows = dataSet.User.Select(String.Format(
                CultureInfo.InvariantCulture, "Id = '{0}'", sessionId));

            // Nimm an, dass die Session ID ungltig ist, wenn wir nicht exakt
            // eine Zeile gefunden haben.
            if (rows == null || rows.Length != 1)
                throw new ServiceAccessException(Resource.InvalidSessionId);

            // Konvertere die Benutzerzeile in ein Benutzerobjekt und gib es
            // und speichere es als aktuell angemeldeten Benutzer.
            CinemaDataSet.UserRow row = (CinemaDataSet.UserRow)rows[0];
            this.user = CinemaDataSetHelper.RowToUser(row);
        }

        public string SessionId
        {
            get
            {
                // HINWEIS: Diese Provider verwendet die User.Id als Session ID.

                // Wenn keine Benutzer angemeldet ist, gibt es auch keine 
                // Session ID. In diesem Fall geben wir den Leerstring zurck.
                if (user == null)
                    return String.Empty;

                // Andernfalls ist die Session ID die ID des aktuell
                // angemeldeten Benutzers.
                return user.Id.ToString();
            }
        }

        public User LogOn(string userName, string password)
        {
            // Finde alle Benutzerzeilen die dem gegebenen Benutzernamen und 
            // Kennwort entsprechen.
            DataRow[] rows = dataSet.User.Select(String.Format(
                CultureInfo.InvariantCulture,
                "UserAccount = '{0}' AND Password = '{1}'", userName,
                password));

            // Nimm an, dass, wenn der wir nicht exakt eine Zeile erhalten
            // haben der Benutzername oder das Kennwort falsch ist.
            if (rows == null || rows.Length != 1)
                throw new ServiceAccessException(
                    Resource.InvalidUserNameOrPassword);

            // Konvertiere die gefundene Benutzerzeile in ein Benutzerobjekt
            // und speichere es als den aktuell angemeldeten Benutzer.
            CinemaDataSet.UserRow row = (CinemaDataSet.UserRow)rows[0];
            user = CinemaDataSetHelper.RowToUser(row);
            return user;
        }

        public void LogOff()
        {
            // Die Session kann einfach dadurch beendet werden, dass wir den
            // aktuell angemeldeten Benutzer auf NULL setzen.
            user = null;
        }

        public void ChangePassword(string currentPassword, string newPassword)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void ResetPassword(string userName)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public User GetCurrentUser()
        {
            return user;
        }

        #endregion

        #region Performance

        public PerformanceInfo[] GetPerformances(DateTime startDate, 
            DateTime endDate)
        {
            // Suche alle Zeilen in der Vorstellungstabelle die den gegebenen
            // Kriterien entsprechen.
            DataRow[] rows = dataSet.Performance.Select(
                String.Format(
                    CultureInfo.InvariantCulture, 
                    "#{0}# <= Date AND Date <= #{1}#", 
                    startDate.ToString(CultureInfo.InvariantCulture), 
                    endDate.ToString(CultureInfo.InvariantCulture)));

            // Konvertiere all Vorstellungszeilen in Vorstellungsobjekte.
            List<PerformanceInfo> result = new List<PerformanceInfo>();

            foreach (CinemaDataSet.PerformanceRow row in rows)
            {
                PerformanceInfo performance = 
                    CinemaDataSetHelper.RowToPerformanceInfo(row);
                result.Add(performance);
            }

            return result.ToArray();
        }

        public Performance GetPerformance(Guid performanceId)
        {
            // Suche alle Zeilen in der Vorstellungstabelle die den gegebenen
            // Kriterien entsprechen.
            DataRow[] rows = dataSet.Performance.Select(
                String.Format(CultureInfo.InvariantCulture, 
                    "Id = '{0}'", performanceId));

            // Nimm an, dass die ID falsch ist, wenn wir nicht exakt eine Zeile
            // erhalten. In diesem Fall liefern wir einfach NULL, um anzuzeigen
            // das keine Zeile mit der geforderten ID existiert.
            if (rows == null || rows.Length != 1)
                return null;

            // Andernfalls konvertieren wir die Zeile in ein Data Transfer
            // Object.
            return CinemaDataSetHelper.RowToPerformance(
                (CinemaDataSet.PerformanceRow) rows[0]);
        }

        public PerformanceInfo[] GetPerformancesOfMovie(Guid movieId)
        {
            // Suche alle Zeilen in der Vorstellungstabelle die den gegebenen
            // Kriterien entsprechen.
            DataRow[] rows = dataSet.Performance.Select(
                String.Format(CultureInfo.InvariantCulture, 
                    "MovieId = '{0}'", movieId));

            // Konvertiere alle Zeilen in Data Transfer Objekte
            List<PerformanceInfo> result = new List<PerformanceInfo>();
            foreach (CinemaDataSet.PerformanceRow row in rows)
            {
                PerformanceInfo performance = 
                    CinemaDataSetHelper.RowToPerformanceInfo(row);
                result.Add(performance);
            }
            return result.ToArray();
        }

        public PerformanceInfo[] GetPerformancesOfRoom(Guid roomId, 
            DateTime startDate, DateTime endDate)
        {
            // Suche alle Zeilen in der Vorstellungstabelle die den gegebenen
            // Kriterien entsprechen.
            DataRow[] rows = dataSet.Performance.Select(
                String.Format(CultureInfo.InvariantCulture, 
                    "RoomId = '{0}' AND #{1}# <= Date AND Date <= #{2}#", 
                    roomId, 
                    startDate.ToString(CultureInfo.InvariantCulture), 
                        endDate.ToString(CultureInfo.InvariantCulture)));

            // Konvertiere alle Zeilen in Data Transfer Objekte
            List<PerformanceInfo> result = new List<PerformanceInfo>();
            foreach (CinemaDataSet.PerformanceRow row in rows)
            {
                PerformanceInfo performance =
                    CinemaDataSetHelper.RowToPerformanceInfo(row);
                result.Add(performance);
            }
            return result.ToArray();
        }

        public void CreatePerformance(Performance performance)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void UpdatePerformance(Performance performance)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void DeletePerformance(Guid performanceId)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public int GetFreeSeats(Guid performanceId)
        {
            Performance p = GetPerformance(performanceId);

            // Berechne die Summe aller Sitze aller Tickets, die mit der
            // gegebenen Vorstellung assoziiert sind.
            string expr = "SUM(Quantity)";
            string filter = String.Format(CultureInfo.InvariantCulture, 
                "PerformanceId = '{0}'", performanceId);
            object result = dataSet.Tickets.Compute(expr, filter);

            // NULL Werte mssen speziell gehandhabt werden.
            int seatsInUse;
            if(result == null || result == DBNull.Value)
                seatsInUse = 0;
            else
                seatsInUse = Convert.ToInt32(result, 
                    CultureInfo.InvariantCulture);

            // Berechne die Anzahl der freien Sitze.
            return p.Room.SeatCount - seatsInUse;
        }

        #endregion

        #region Movie

        public MovieInfo[] GetMovies()
        {
            // Konvertiere alle Zeilen in Data Transfer Objekte
            List<MovieInfo> result = new List<MovieInfo>();
            foreach (CinemaDataSet.MovieRow row in dataSet.Movie.Rows)
            {
                MovieInfo m = CinemaDataSetHelper.RowToMovieInfo(row);
                result.Add(m);
            }
            return result.ToArray();
        }

        public MovieInfo[] GetMovies(DateTime startDate)
        {
            // Suche alle Zeilen in der Filmtabelle die den gegebenen
            // Kriterien entsprechen.
            DataRow[] rows = dataSet.Movie.Select(String.Format(
                CultureInfo.InvariantCulture, "StartDate >= #{0}#", startDate));

            // Konvertiere alle Zeilen in Data Transfer Objekte
            List<MovieInfo> result = new List<MovieInfo>();
            foreach (CinemaDataSet.MovieRow row in rows)
            {
                MovieInfo m = CinemaDataSetHelper.RowToMovieInfo(row);
                result.Add(m);
            }
            return result.ToArray();
        }

        public MovieInfo[] GetMoviesByUserInterests(Guid userId)
        {
            // Suche alle Zeilen in der Benutzertabelle die der gegebenen ID
            // entsprechen.
            DataRow[] rows = dataSet.User.Select(
                String.Format(CultureInfo.InvariantCulture, "Id = '{0}'",
                userId));

            // Nimm an, dass die ID falsch ist, wenn wir nicht exakt eine Zeile
            // erhalten. In diesem Fall liefern wir einfach NULL, um anzuzeigen
            // das keine Zeile mit der geforderten ID existiert.
            if (rows == null || rows.Length != 1)
                return new MovieInfo[0];

            CinemaDataSet.UserRow userRow = (CinemaDataSet.UserRow)rows[0];

            // Erzeuge eine Liste von Filmen fr die sich der gegebende
            // Benutzer interessiert. Da mehrere Genres zu demselben Film fhren
            // knnen, benutzen wir ein Wrterbuch mit der Film ID als Schlssel
            // um sicherzustellen das jeder Film nur maximal einmal enthalten
            // ist.
            Dictionary<Guid, MovieInfo> movieInfos =
                new Dictionary<Guid, MovieInfo>();

            foreach (CinemaDataSet.Users_GenresRow ugrow in 
                userRow.GetUsers_GenresRows())
            {
                foreach (CinemaDataSet.Movie_GenresRow mgRow in 
                    ugrow.GenreRow.GetMovie_GenresRows())
                {
                    if (!movieInfos.ContainsKey(mgRow.MovieId))
                    {
                        MovieInfo movieInfo = 
                            CinemaDataSetHelper.RowToMovieInfo(mgRow.MovieRow);
                        movieInfos.Add(movieInfo.Id, movieInfo);
                    }
                }
            }

            MovieInfo[] result = new MovieInfo[movieInfos.Values.Count];
            movieInfos.Values.CopyTo(result, 0);
            return result;
        }

        public MovieInfo[] GetMoviesByTitle(string movieName)
        {
            // Suche alle Filme mit dem gegebenem Namen.
            DataRow[] rows = dataSet.Movie.Select(String.Format(
                CultureInfo.InvariantCulture, "Name LIKE '{0}'", movieName));

            // Konvertiere alle Zeilen in Data Transfer Objekte
            List<MovieInfo> result = new List<MovieInfo>();
            foreach (CinemaDataSet.MovieRow row in rows)
            {
                MovieInfo m = CinemaDataSetHelper.RowToMovieInfo(row);
                result.Add(m);
            }
            return result.ToArray();
        }

        public Movie GetMovie(Guid movieId)
        {
            // Suche alle Filme mit der gegebenen ID.
            DataRow[] rows = dataSet.Movie.Select(String.Format(
                CultureInfo.InvariantCulture, "Id = '{0}'", movieId));

            // Nimm an, dass die ID falsch ist, wenn wir nicht exakt eine Zeile
            // erhalten. In diesem Fall liefern wir einfach NULL, um anzuzeigen
            // das keine Zeile mit der geforderten ID existiert.
            if (rows == null || rows.Length != 1)
                return null;

            // Andernfalls konvertieren wir die Zeile in ein Data Transfer
            // Object.
            return CinemaDataSetHelper.RowToMovie(
                (CinemaDataSet.MovieRow)rows[0]);
        }

        public void CreateMovie(Movie movie)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void UpdateMovie(Movie movie)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void DeleteMovie(Guid movieId)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void RateMovie(Guid movieId, int rating)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        #endregion

        #region User

        public User GetUser(Guid userId)
        {
            // Suche alle Zeilen in der Benutzertabelle die der gegebenen ID
            // entsprechen.
            DataRow[] rows = dataSet.User.Select(
                String.Format(CultureInfo.InvariantCulture, "Id = '{0}'", 
                userId));

            // Nimm an, dass die ID falsch ist, wenn wir nicht exakt eine Zeile
            // erhalten. In diesem Fall liefern wir einfach NULL, um anzuzeigen
            // das keine Zeile mit der geforderten ID existiert.
            if (rows == null || rows.Length != 1)
                return null;

            // Konvertiere die Zeile ein Data Transfer Object.
            return CinemaDataSetHelper.RowToUser(
                (CinemaDataSet.UserRow)rows[0]);
        }

        public UserInfo[] GetUsersForNewsletter()
        {
            // Suche alle Zeilen in der Benutzertabelle die einen Newsletter
            // abboniert haben.
            DataRow[] rows = dataSet.User.Select("Newsletter");

            // Konvertiere alle Zeilen in Data Transfer Objekte
            List<UserInfo> result = new List<UserInfo>();
            foreach (CinemaDataSet.UserRow row in rows)
            {
                UserInfo user = CinemaDataSetHelper.RowToUserInfo(row);
                result.Add(user);
            }
            return result.ToArray();
        }

        public void CreateUser(User user, string password)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void UpdateUser(User user)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void DeleteUser(Guid userId)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        #endregion

        #region Room

        public Room[] GetRooms()
        {
            // Konvertiere alle Zeilen in Data Transfer Objekte
            List<Room> result = new List<Room>();
            foreach (CinemaDataSet.RoomRow row in dataSet.Room.Rows)
            {
                Room r = CinemaDataSetHelper.RowToRoom(row);
                result.Add(r);
            }
            return result.ToArray();
        }

        public void CreateRoom(Room room)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void UpdateRoom(Room room)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void DeleteRoom(Guid roomId)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        #endregion

        #region Ticket

        public string BuyTicket(Guid performanceId, int quantity)
        {
            // Von diesem Service Provider nicht implementiert.
            return String.Empty;
        }

        public void SellTicket(Guid performanceId, int quantity)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public Ticket[] GetUndispatchedTickets(Guid userId)
        {
            // Suche alle Tickets, die von dem gegebenen Benutzer bestellt 
            // worden sind, aber nich nicht abgeholt worden sind.
            DataRow[] rows = dataSet.Tickets.Select(String.Format(
                CultureInfo.InvariantCulture,
                "UserId = '{0}' AND NOT IsDispatched", userId));

            // Konvertiere alle Zeilen in Data Transfer Objekte
            List<Ticket> result = new List<Ticket>();
            foreach (CinemaDataSet.TicketsRow row in rows)
            {
                Ticket t = CinemaDataSetHelper.RowToTicket(row);
                result.Add(t);
            }
            return result.ToArray();
        }

        public Ticket[] GetUndispatchedTickets(string ticketCode)
        {
            // Suche alle Zeilen die dem Ticket-Code entsprechen.
            DataRow[] rows = dataSet.Tickets.Select(String.Format(
                CultureInfo.InvariantCulture,
                "TicketCode = '{0}' AND NOT IsDispatched", ticketCode));

            // Konvertiere alle Zeilen in Data Transfer Objekte
            List<Ticket> result = new List<Ticket>();
            foreach (CinemaDataSet.TicketsRow row in rows)
            {
                Ticket t = CinemaDataSetHelper.RowToTicket(row);
                result.Add(t);
            }
            return result.ToArray();
        }

        public void MarkTicketAsDispatched(Ticket ticket)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        #endregion

        #region News

        public NewsItem[] GetNewsItems()
        {
            // Konvertiere alle Zeilen in Data Transfer Objekte
            List<NewsItem> result = new List<NewsItem>();
            foreach (CinemaDataSet.NewsItemRow row in dataSet.NewsItem.Rows)
            {
                NewsItem n = CinemaDataSetHelper.RowToNewsItem(row);
                result.Add(n);
            }
            return result.ToArray();
         }

        public NewsItem[] GetNewsItemsByCategories(int[] numbers)
        {
            // Suche alle Newskategorien die den gegebenen Nummern
            // entsprechen.
            List<NewsItem> result = new List<NewsItem>();

            foreach (NewsItem item in GetNewsItems())
            {
                foreach (int number in numbers)
                {
                    bool categoryFound = false;

                    foreach (NewsCategory category in item.Categories)
                    {
                        if (category.Number == number)
                        {
                            categoryFound = true;
                            break;
                        }
                    }

                    if (categoryFound)
                    {
                        result.Add(item);
                        break;
                    }
                }
            }

            return result.ToArray();
        }

        public void CreateNewsItem(NewsItem newsItem)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void UpdateNewsItem(NewsItem newsItem)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void DeleteNewsItem(Guid newsItemId)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        #endregion

        #region News Categories

        public NewsCategory[] GetNewsCategories()
        {
            // Konvertiere alle Zeilen in Data Transfer Objekte
            List<NewsCategory> result = new List<NewsCategory>();
            foreach (CinemaDataSet.NewsCategoryRow row in 
                dataSet.NewsCategory.Rows)
            {
                NewsCategory c = CinemaDataSetHelper.RowToNewsCategory(row);
                result.Add(c);
            }
            return result.ToArray();
        }

        public void CreateNewsCategory(NewsCategory category)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void UpdateNewsCategory(NewsCategory category)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void DeleteNewsCategory(Guid categoryId)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        #endregion

        #region Genre

        public Genre[] GetGenres()
        {
            // Konvertiere alle Zeilen in Data Transfer Objekte
            List<Genre> result = new List<Genre>();
            foreach (CinemaDataSet.GenreRow row in
                dataSet.Genre.Rows)
            {
                Genre g = CinemaDataSetHelper.RowToGenre(row);
                result.Add(g);
            }
            return result.ToArray();
        }

        public void CreateGenre(Genre genre)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void UpdateGenre(Genre genre)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void DeleteGenre(Guid genreId)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        #endregion

        #region Country

        public Country[] GetCountries()
        {
            // Konvertiere alle Zeilen in Data Transfer Objekte
            List<Country> result = new List<Country>();
            foreach (CinemaDataSet.CountryRow row in
                dataSet.Country.Rows)
            {
                Country c = CinemaDataSetHelper.RowToCountry(row);
                result.Add(c);
            }
            return result.ToArray();
        }

        public void CreateCountry(Country country)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void UpdateCountry(Country country)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void DeleteCountry(Guid countryId)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        #endregion

        #region Distributor

        public Distributor[] GetDistributors()
        {
            // Konvertiere alle Zeilen in Data Transfer Objekte
            List<Distributor> result = new List<Distributor>();
            foreach (CinemaDataSet.DistributorRow row in
                dataSet.Distributor.Rows)
            {
                Distributor d = CinemaDataSetHelper.RowToDistributor(row);
                result.Add(d);
            }
            return result.ToArray();
        }

        public void CreateDistributor(Distributor distributor)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void UpdateDistributor(Distributor distributor)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void DeleteDistributor(Guid distributorId)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        #endregion

        #region Parental Guidance

        public ParentalGuidance[] GetParentalGuidances()
        {
            // Konvertiere alle Zeilen in Data Transfer Objekte
            List<ParentalGuidance> result = new List<ParentalGuidance>();
            foreach (CinemaDataSet.ParentalGuidanceRow row in
                dataSet.ParentalGuidance.Rows)
            {
                ParentalGuidance p = CinemaDataSetHelper.RowToParentalGuidance(
                    row);
                result.Add(p);
            }
            return result.ToArray();
        }

        public void CreateParentalGuidance(ParentalGuidance parentalGuidance)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void UpdateParentalGuidance(ParentalGuidance parentalGuidance)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void DeleteParentalGuidance(Guid parentalGuidanceId)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        #endregion

        #region Person

        public Person[] GetPersons()
        {
            // Konvertiere alle Zeilen in Data Transfer Objekte
            List<Person> result = new List<Person>();
            foreach (CinemaDataSet.PersonRow row in
                dataSet.Person.Rows)
            {
                Person p = CinemaDataSetHelper.RowToPerson(row);
                result.Add(p);
            }
            return result.ToArray();
        }

        public void CreatePerson(Person person)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void UpdatePerson(Person person)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        public void DeletePerson(Guid personId)
        {
            // Von diesem Service Provider nicht implementiert.
        }

        #endregion
    }
}