using System;
using System.Collections.Generic;
using System.Globalization;
using System.Text;

using AdventureWorks.Cinema.ServiceAccessLayer.Caching;
using AdventureWorks.Cinema.ServiceAccessLayer.SampleData;
using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.ServiceAccessLayer
{
    /// <summary>
    /// Die Service Access API wird von den Clients benutzt, um Methoden des 
    /// Web Service mit Hilfe komfortabler Service Methoden aufzurufen.
    /// </summary>
    public sealed class ServiceAccess
    {
        private Uri webServiceUrl;
        private string sampleDataFileName;
        private bool isSampleMode;
        private bool isCachingEnabled;
        private CachingDataProvider cachingDataProvider;
        private WebServiceDataProvider webServiceDataProvider;
        private SampleDataProvider sampleDataProvider;

        /// <summary>
        /// Der interne Name der Rolle 'Kunde'
        /// </summary>
        public const string CustomerRoleName = "Customer";

        /// <summary>
        /// Der interne Name der Rolle 'Mitarbeiter'
        /// </summary>
        public const string EmployeeRoleName = "Employee";

        /// <summary>
        /// Erzeugt eine Instanz der Klasse <see cref="ServiceAccess"/>, die
        /// noch nicht an eine Session gebunden ist, d.h 
        /// <see cref="IsAuthenticated"/> ist <see langword="false"/>.
        /// </summary>
        public ServiceAccess()
        {
            webServiceUrl = new Uri(Settings.WebServiceUrl);
            sampleDataFileName = Settings.SampleDataFileName;
            isSampleMode = Settings.IsSampleMode;
            isCachingEnabled = Settings.IsCachingEnabled;
        }

        /// <summary>
        /// Erzeugt ein Instanz der Klasse <see cref="ServiceAccess"/>, die
        /// an die angegebene Session gebunden ist, d.h. 
        /// <see cref="IsAuthenticated"/> ist <see langword="true"/>.
        /// </summary>
        /// <remarks>
        /// Die angegebene Session ID wird erst validiert, wenn eine Methode
        /// aufgerufen wird, die mit dem Web Service kommuniziert, z.B.
        /// <see cref="GetPerformances(DateTime,DateTime)"/> oder
        /// <see cref="ChangePassword"/>.
        /// </remarks>
        /// <param name="sessionId">Die Session ID der Session mit der diese
        /// Instanz verbunden werden soll.</param>
        /// <exception cref="ArgumentException"><paramref name="sessionId"/> 
        /// ist <see langword="null"/> oder <see cref="String.Empty"/>.
        /// </exception>
        public ServiceAccess(string sessionId)
            : this()
        {
            if (String.IsNullOrEmpty(sessionId))
                throw new ArgumentException(Resource.ArgumentNullOrEmptyString,
                    "sessionId");

            CurrentProvider.ConnectToSession(sessionId);
        }

        /// <summary>
        /// Ruft die URL des Cinema Web Servers ab oder legt diese fest.
        /// </summary>
        /// <remarks>
        /// Eine nderung dieser Einstellung hat keine Auswirkung auf eine
        /// bereits vorhanden Session. Die Session muss zuerst beendet werden.
        /// </remarks>
        public Uri WebServiceUrl
        {
            get { return webServiceUrl; }
            set { webServiceUrl = value; }
        }

        /// <summary>
        /// Ruft den Dateipfad der XML Datei mit Demodaten ab oder legt 
        /// diese fest.
        /// </summary>
        /// <remarks>
        /// Eine nderung dieser Einstellung hat keine Auswirkung auf eine
        /// bereits vorhanden Session. Die Session muss zuerst beendet werden.
        /// </remarks>
        /// <exception cref="ArgumentException">
        /// <para><paramref name="value"/> ist <see cref="String.Empty"/></para>
        /// <para>- oder -</para>
        /// <para><paramref name="value"/> ist <see langword="null"/>.</para>
        /// </exception>
        public string SampleDataFileName
        {
            get { return sampleDataFileName; }
            set
            {
                if (String.IsNullOrEmpty(value))
                    throw new ArgumentException(
                        Resource.ArgumentNullOrEmptyString);

                sampleDataFileName = value;
            }
        }

        /// <summary>
        /// Ruft basierend auf den Einstellungen von 
        /// <see cref="isCachingEnabled"/> und <see cref="isSampleMode"/> den
        /// aktuell zu verwendenden Service Provider ab.
        /// </summary>
        private IServiceAccessProvider CurrentProvider
        {
            get 
            {
                IServiceAccessProvider result;

                if (!isSampleMode)
                {
                    // Setze den Demodaten Provider zurck.
                    sampleDataProvider = null;

                    // "Realen" Daten Provider zurckgeben.
                    if (webServiceDataProvider == null)
                        webServiceDataProvider = new WebServiceDataProvider(
                            webServiceUrl);

                    result = webServiceDataProvider;
                }
                else
                {   
                    // Gib den Demodaten Provider zurck. Dieser liefert
                    // die Daten, die in der Datei sampleDataFileName 
                    // gespeichert ist.
                    if (sampleDataProvider == null)
                        sampleDataProvider =
                            new SampleDataProvider(sampleDataFileName);

                    result = sampleDataProvider;
                }

                if (!isCachingEnabled)
                {
                    // Setzen den Caching Daten Provider zurck.
                    cachingDataProvider = null;
                }
                else
                {
                    // Wenn Caching aktiviert ist, wird er aktulle Service
                    // Provider in einem Caching Daten Provider verpackt.
                    if (cachingDataProvider == null)
                        cachingDataProvider = new CachingDataProvider(result);

                    result = cachingDataProvider;
                }

                return result;
            }
        }

        #region Caching

        /// <summary>
        /// Ruft die Dauer der Gltigkeit von Cache Eintrgen ab oder legt 
        /// diese fest.
        /// </summary>
        public TimeSpan CacheValidityDuration
        {
            get { return cachingDataProvider.CacheValidityDuration; }
            set { cachingDataProvider.CacheValidityDuration = value; }
        }

        /// <summary>
        /// Entfernt alle Cache Eintrge.
        /// </summary>
        public void ClearCaches()
        {
            if (cachingDataProvider != null)
                cachingDataProvider.ClearCaches();
        }

        /// <summary>
        /// Ruft einen Wert ab, der festlegt, ob Caching aktiviert ist oder 
        /// legt diesen fest.
        /// </summary>
        public bool IsCachingEnabled
        {
            get { 
                return isCachingEnabled;
            }
            set 
            { 
                isCachingEnabled = value;

                if (!value)
                    cachingDataProvider = null;
            }
        }

        #endregion

        #region Session

        /// <summary>
        /// Ruft einen Wert ab, der festlegt, ob der Demo-Modus aktiviert ist
        /// oder legt diesen fest.
        /// </summary>
        public bool IsSampleMode
        {
            get { 
                return isSampleMode;
            }
            set
            {
                isSampleMode = value;
                cachingDataProvider = null;
            }
        }

        /// <summary>
        /// Ruft die ID der aktiven Session ab.
        /// </summary>
        /// <returns>Wenn diese Instanz nicht an eine Session gebunden ist, d.h.
        /// entweder <see cref="LogOn"/> wurde noch nicht aufgerufen oder die
        /// Session wurde durch Aufruf von <see cref="LogOff"/> beendet, wird
        /// <see cref="String.Empty"/> zurckgegeben.</returns>
        public string SessionId
        {
            get
            {
                string result = CurrentProvider.SessionId;

                if (result == null)
                    return String.Empty;

                return result;
            }
        }

        /// <summary>
        /// Ruft einen Wert ab, der angibt, ob diese Instanz an eine Session
        /// gebunden ist.
        /// </summary>
        /// <returns>Wenn diese Instanz nicht an eine Session gebunden ist, d.h.
        /// entweder <see cref="LogOn"/> wurde noch nicht aufgerufen oder die
        /// Session wurde durch Aufruf von <see cref="LogOff"/> beendet, wird
        /// <see langword="false"/> zurckgegeben. Andernfalls wird 
        /// <see langword="true"/> zurckgegeben.</returns>
        public bool IsAuthenticated
        {
            get { 
                return !String.IsNullOrEmpty(CurrentProvider.SessionId); 
            }
        }

        /// <summary>
        /// Erzeugt mit den gegebenen Anmeldedaten eine Session.
        /// </summary>
        /// <param name="userName">Der Benutzerkennung des Benutzers.</param>
        /// <param name="password">Das Kennwort des Benutzers.</param>
        /// <exception cref="ArgumentException">
        /// <para><paramref name="userName"/> ist 
        /// <see langword="null"/> oder <see cref="String.Empty"/>.</para>
        /// <para>- oder -</para>
        /// <para><paramref name="password"/> ist 
        /// <see langword="null"/> oder <see cref="String.Empty"/>.</para>
        /// </exception>
        /// <returns>Der <see cref="User"/> der den angemeldeten Benutzer 
        /// reprsentiert.
        /// </returns>
        public User LogOn(string userName, string password)
        {
            if (String.IsNullOrEmpty(userName))
                throw new ArgumentException(Resource.ArgumentNullOrEmptyString,
                    "userName");

            if (String.IsNullOrEmpty(password))
                throw new ArgumentException(Resource.ArgumentNullOrEmptyString,
                    "password");

            User result = CurrentProvider.LogOn(userName, password);
            return result;
        }

        /// <summary>
        /// Beendet die Session des aktuell angemeldeteten Benutzers auf dem
        /// Server.
        /// </summary>
        public void LogOff()
        {
            CurrentProvider.LogOff();
        }

        /// <summary>
        /// ndert das Kennwort des aktuell angemeldeten Benutzers. Fr diese
        /// Operation wird zustzlich auch das aktuelle Kennwort bentigt. 
        /// Dadurch kann verhindert werden, dass jemand das Kennwort eines
        /// anderen ndern kann, nachdem dieser kurz die Arbeitsstation 
        /// verlassen hat.
        /// </summary>
        /// <param name="currentPassword">Das aktuelle Kennwort</param>
        /// <param name="newPassword">Das neue Kennwort</param>
        /// <exception cref="ArgumentException">
        /// <para><paramref name="currentPassword"/> ist 
        /// <see langword="null"/> oder <see cref="String.Empty"/>.</para>
        /// <para>- oder -</para>
        /// <para><paramref name="newPassword"/> ist
        /// <see langword="null"/> oder <see cref="String.Empty"/>.</para>
        /// </exception>
        public void ChangePassword(string currentPassword, 
            string newPassword)
        {
            if (String.IsNullOrEmpty(currentPassword))
                throw new ArgumentException(Resource.ArgumentNullOrEmptyString, 
                    "currentPassword");

            if (String.IsNullOrEmpty(newPassword))
                throw new ArgumentException(Resource.ArgumentNullOrEmptyString, 
                    "newPassword");

            CurrentProvider.ChangePassword(currentPassword, newPassword);
        }

        /// <summary>
        /// Setzt das Kennwort fr den gegebenen Benutzer zurck. Fr diesen
        /// Benutzer wird ein neues Kennwort generiert, dass diesem dann per
        /// E-Mail zugeschickt wird.
        /// </summary>
        /// <param name="userName">Die Benutzerkennung des Benutzers, fr den 
        /// das Kennwort zurckgesetzt werden kann.</param>
        /// <exception cref="ArgumentNullException"><paramref name="user"/> ist 
        /// <see langword="null"/> oder <see cref="String.Empty"/>.
        /// </exception>
        public void ResetPassword(string userName)
        {
            if (String.IsNullOrEmpty(userName))
                throw new ArgumentException(Resource.ArgumentNullOrEmptyString, 
                    "userName");

            CurrentProvider.ResetPassword(userName);
        }

        /// <summary>
        /// Ruft den aktuell angemeldeten Benutzer ab.
        /// </summary>
        /// <return>Gibt den aktuell angemeldeten Benutzer zurck. Wenn kein
        /// Benutzer angemeldet ist, wird <see langword="null"/> zurckgegeben.
        ///</return>
        public User GetCurrentUser()
        {
            return CurrentProvider.GetCurrentUser();
        }

        #endregion

        #region Performance

        /// <summary>
        /// Gibt eine Liste mit allen Vorstellungen zurck, die in dem 
        /// gegebenen Zeitraum stattfinden.
        /// </summary>
        /// <param name="startDate">Die untere Grenze des Zeitraumes.</param>
        /// <param name="endDate">Die obere Grenze des Zeitraumes</param>
        /// <returns>Gibt eine List von Veranstallungsinformationen zurck.
        /// </returns>
        /// <exception cref="ArgumentException"><paramref name="endDate"/> ist
        /// vor <paramref name="startDate"/>.
        /// </exception>
        public PerformanceInfo[] GetPerformances(DateTime startDate, 
            DateTime endDate)
        {
            if (endDate < startDate)
                throw new ArgumentException(
                    Resource.EndDateMustBeBeforeStartDate, "endDate");

            return CurrentProvider.GetPerformances(startDate, endDate);
        }

        /// <summary>
        /// Gibt eine List aller Vorstellungen von einem Film zurck.
        /// </summary>
        /// <param name="movie">Der Film fr den Vorstellungen gesucht werden 
        /// sollen.</param>
        /// <returns>Gibt eine List von Veranstallungsinformationen zurck.
        /// </returns>
        /// <exception cref="ArgumentNullException"><paramref name="movie"/>
        /// ist <see langword="null"/>.</exception>
        public PerformanceInfo[] GetPerformances(Movie movie)
        {
            if (movie == null)
                throw new ArgumentNullException("movie");

            return CurrentProvider.GetPerformancesOfMovie(movie.Id);
        }

        /// <summary>
        /// Gibt eine List aller Vorstellungen von einem Film zurck.
        /// </summary>
        /// <param name="movieInfo">Der Film fr den Vorstellungen gesucht
        /// werden sollen.</param>
        /// <returns>Gibt eine List von Veranstallungsinformationen zurck.
        /// </returns>
        /// <exception cref="ArgumentNullException"><paramref name="movieInfo"/> 
        /// ist <see langword="null"/>.</exception>
        public PerformanceInfo[] GetPerformances(MovieInfo movieInfo)
        {
            if (movieInfo == null)
                throw new ArgumentNullException("movieInfo");

            return CurrentProvider.GetPerformancesOfMovie(movieInfo.Id);
        }

        /// <summary>
        /// Gibt eine List aller Vorstellungen zurck, die in enem bestimmtem
        /// Raum in einem bestimmten Zeitraum stattfinden.
        /// </summary>
        /// <param name="room">Der Raum zu dem Vorstellungen gesucht werden
        /// sollen.</param>
        /// <param name="startDate">Die untere Grenze des Zeitraumes.</param>
        /// <param name="endDate">Die obere Grenze des Zeitraumes</param>
        /// <returns>Gibt eine List von Veranstallungsinformationen zurck.
        /// </returns>
        /// <exception cref="ArgumentNullException"><paramref name="room"/> ist
        /// <see langword="null"/>.</exception>
        /// <exception cref="ArgumentException"><paramref name="endDate"/> ist
        /// vor <paramref name="startDate"/>.
        /// </exception>
        public PerformanceInfo[] GetPerformances(Room room, 
            DateTime startDate, DateTime endDate)
        {
            if (room == null)
                throw new ArgumentNullException("room");

            if (endDate < startDate)
                throw new ArgumentException(
                    Resource.EndDateMustBeBeforeStartDate, "endDate");

            return CurrentProvider.GetPerformancesOfRoom(room.Id, startDate, 
                endDate);
        }

        /// <summary>
        /// Gibt eine volles Vorstellungsobjekt fr die 
        /// Vorstellungsinformation.
        /// </summary>
        /// <param name="performanceInfo">Die Vorstellungsinformation fr das
        /// ein volles Vorstellungsobjekt geladen werden soll.</param>
        /// <returns>Gibt eine volles Vorstellungsobjekt zurck.</returns>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="performanceInfo"/> ist <see langword="null"/>.
        /// </exception>
        public Performance GetPerformance(PerformanceInfo performanceInfo)
        {
            if (performanceInfo == null)
                throw new ArgumentNullException("performanceInfo");

            return CurrentProvider.GetPerformance(performanceInfo.Id);
        }

        /// <summary>
        /// Erzeugt eine Vorstellung mit den gegebenen Daten.
        /// </summary>
        /// <param name="performance">Das Vorstellungsobjekt mit den Daten.
        /// </param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="performance"/> ist <see langword="null"/>.
        /// </exception>
        public void CreatePerformance(Performance performance)
        {
            if (performance == null)
                throw new ArgumentNullException("performance");

            CurrentProvider.CreatePerformance(performance);
        }

        /// <summary>
        /// Aktualisiert eine Vorstellung mit den gegebenen Daten.
        /// </summary>
        /// <param name="performance">Das Vorstellungsobjekt mit den Daten.
        /// </param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="performance"/> ist <see langword="null"/>.
        /// </exception>
        public void UpdatePerformance(Performance performance)
        {
            if (performance == null)
                throw new ArgumentNullException("performance");

            CurrentProvider.UpdatePerformance(performance);
        }

        /// <summary>
        /// Lscht eine Vorstellung.
        /// </summary>
        /// <remarks>
        /// Eine Vorstellung kann nur gelscht werden, wenn keine Tickts 
        /// existieren. Falls bereits Tickets exisitieren wird eine Ausnahme
        /// geworfen.
        /// </remarks>
        /// <param name="performance">Die zu lschende Vorstellung.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="performance"/> ist <see langword="null"/>.
        /// </exception>
        public void DeletePerformance(Performance performance)
        {
            if (performance == null)
                throw new ArgumentNullException("performance");

            CurrentProvider.DeletePerformance(performance.Id);
        }

        /// <summary>
        /// Lscht eine Vorstellung.
        /// </summary>
        /// <remarks>
        /// Eine Vorstellung kann nur gelscht werden, wenn keine Tickts 
        /// existieren. Falls bereits Tickets exisitieren wird eine Ausnahme
        /// geworfen.
        /// </remarks>
        /// <param name="performanceInfo">Die zu lschende Vorstellung.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="performanceInfo"/> ist <see langword="null"/>.
        /// </exception>
        public void DeletePerformance(PerformanceInfo performanceInfo)
        {
            if (performanceInfo == null)
                throw new ArgumentNullException("performanceInfo");

            CurrentProvider.DeletePerformance(performanceInfo.Id);
        }

        /// <summary>
        /// Gibt die Anzahl der freien Pltze fr eine Vorstellung zurck.
        /// </summary>
        /// <param name="performance">Die Vorstellung fr die Anzahl der freien
        /// Pltze ermittelt werden soll.</param>
        /// <returns>Die Anzahl der freien Pltze.</returns>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="performance"/> ist <see langword="null"/>.
        /// </exception>
        public int GetFreeSeats(Performance performance)
        {
            if (performance == null)
                throw new ArgumentNullException("performance");

            return CurrentProvider.GetFreeSeats(performance.Id);
        }

        /// <summary>
        /// Gibt die Anzahl der freien Pltze fr eine Vorstellung zurck.
        /// </summary>
        /// <param name="performanceInfo">TDie Vorstellung fr die Anzahl der freien
        /// Pltze ermittelt werden soll.</param>
        /// <returns>Die Anzahl der freien Pltze.</returns>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="performanceInfo"/> ist <see langword="null"/>.
        /// </exception>
        public int GetFreeSeats(PerformanceInfo performanceInfo)
        {
            if (performanceInfo == null)
                throw new ArgumentNullException("performanceInfo");

            return CurrentProvider.GetFreeSeats(performanceInfo.Id);
        }

        #endregion

        #region Movie

        /// <summary>
        /// Liefert eine Liste aller Filme in der Datenbank.
        /// </summary>
        /// <returns>Eine List von Filmen.</returns>
        public MovieInfo[] GetMovies()
        {
            return CurrentProvider.GetMovies();
        }

        /// <summary>
        /// Liefert eine Liste aller Filme, die an oder nach einem bestimmten
        /// Datum anlaufen.
        /// </summary>
        /// <returns>Eine Liste von Filmen.</returns>
        public MovieInfo[] GetMovies(DateTime startDate)
        {
            return CurrentProvider.GetMovies(startDate);
        }

        /// <summary>
        /// Liefert eine Liste aller Filme, die innerhalb der Interessen des
        /// aktuell angemeldeten Benutzers liegen.
        /// </summary>
        /// <param name="user">Der Benutzer auf dessen Profil die Auswahl der
        /// Filme erfolgen soll.</param>
        /// <returns>Eine Liste von Filmen.</returns>
        /// <exception cref="ArgumentNullException"><paramref name="user"/> ist
        /// <see langword="null"/>.</exception>
        public MovieInfo[] GetMoviesByUserInterests(User user)
        {
            if (user == null)
                throw new ArgumentNullException("user");

            return CurrentProvider.GetMoviesByUserInterests(user.Id);
        }

        /// <summary>
        /// Liefert eine Liste aller Filme, die innerhalb der Interessen des
        /// aktuell angemeldeten Benutzers liegen.
        /// </summary>
        /// <param name="userInfo">Der Benutzer auf dessen Profil die Auswahl der
        /// Filme erfolgen soll.</param>
        /// <returns>Eine Liste von Filmen.</returns>
        /// <exception cref="ArgumentNullException"><paramref name="user"/> ist
        /// <see langword="null"/>.</exception>
        public MovieInfo[] GetMoviesByUserInterests(UserInfo userInfo)
        {
            if (userInfo == null)
                throw new ArgumentNullException("userInfo");

            return CurrentProvider.GetMoviesByUserInterests(userInfo.Id);
        }

        /// <summary>
        /// Liefert eine Liste aller Filme, die den gegebenen Text entweder in
        /// <see cref="Movie.Title"/> oder in
        /// <see cref="Movie.OriginalTitle"/> enthalten.
        /// </summary>
        /// <param name="movieTitle">Der zu suchende Titel</param>
        /// <exception cref="ArgumentException"><paramref name="movieTitle"/> 
        /// ist <see langword="null"/> or <see cref="String.Empty"/>.
        /// </exception>
        public MovieInfo[] GetMoviesByTitle(string movieTitle)
        {
            if (String.IsNullOrEmpty(movieTitle))
                throw new ArgumentException(Resource.ArgumentNullOrEmptyString, 
                    "movieTitle");

            return CurrentProvider.GetMoviesByTitle(movieTitle);
        }

        /// <summary>
        /// Bewertet eine Film mit eine gegebenen Bewertung. Die Bewertung
        /// ist verknpft mit dem aktuell angemeldeten Benutzer.
        /// </summary>
        /// <remarks>
        /// Ein Film kann pro Benutzer nur einmal bewertet werden.
        /// </remarks>
        /// <param name="movie">Der zu bewertende Film</param>
        /// <param name="rating">Die Bewertung des Benutzers. Der Wert muss 
        /// zwischen <c>1</c> und <c>10</c> sein.</param>
        /// <exception cref="ArgumentNullException"><paramref name="movie"/> ist
        /// <see langword="null"/>.</exception>
        /// <exception cref="ArgumentException"><paramref name="rating"/> ist 
        /// nicht zwischen <c>1</c> und <c>10</c>.
        /// </exception>
        public void RateMovie(Movie movie, int rating)
        {
            if (movie == null)
                throw new ArgumentNullException("movie");

            if (rating < 1 || rating > 10)
                throw new ArgumentOutOfRangeException("rating",
                    Resource.RatingMustBeBetweenOneAndTen);

            CurrentProvider.RateMovie(movie.Id, rating);
        }

        /// <summary>
        /// Bewertet eine Film mit eine gegebenen Bewertung. Die Bewertung
        /// ist verknpft mit dem aktuell angemeldeten Benutzer.
        /// on user.
        /// </summary>
        /// <remarks>
        /// Ein Film kann pro Benutzer nur einmal bewertet werden.
        /// </remarks>
        /// <param name="movieInfo">Der zu bewertende Film</param>
        /// <param name="rating">Die Bewertung des Benutzers. Der Wert muss 
        /// zwischen <c>1</c> und <c>10</c> sein.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="movieInfo"/> ist
        /// <see langword="null"/>.</exception>
        /// <exception cref="ArgumentException"><paramref name="rating"/> ist 
        /// nicht zwischen <c>1</c> und <c>10</c>.
        /// </exception>
        public void RateMovie(MovieInfo movieInfo, int rating)
        {
            if (movieInfo == null)
                throw new ArgumentNullException("movieInfo");

            if (rating < 1 || rating > 10)
                throw new ArgumentOutOfRangeException("rating",
                    Resource.RatingMustBeBetweenOneAndTen);

            CurrentProvider.RateMovie(movieInfo.Id, rating);
        }

        /// <summary>
        /// Gibt ein volles Filmobjekt fr die Filminformation.
        /// </summary>
        /// <param name="movieInfo">Die Filminformation fr die ein volles 
        /// Filmobjekt geliefert werden soll.</param>
        /// <returns>Ein volles Filmobjekt.</returns>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="movieInfo"/> ist <see langword="null"/>.</exception>
        public Movie GetMovie(MovieInfo movieInfo)
        {
            if (movieInfo == null)
                throw new ArgumentNullException("movieInfo");

            return CurrentProvider.GetMovie(movieInfo.Id);
        }

        /// <summary>
        /// Erzeugt einen Film mit den gegebenen Daten.
        /// </summary>
        /// <param name="movie">Die Daten fr den zu erzeugenden Film.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="movie"/> ist <see langword="null"/>.</exception>
        public void CreateMovie(Movie movie)
        {
            if (movie == null)
                throw new ArgumentNullException("movie");

            CurrentProvider.CreateMovie(movie);
        }

        /// <summary>
        /// Aktualisiert einen Film mit den gegebenen Daten.
        /// </summary>
        /// <param name="movie">Die Daten fr den zu aktualisierenden Film.
        /// </param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="movie"/> ist <see langword="null"/>.</exception>
        public void UpdateMovie(Movie movie)
        {
            if (movie == null)
                throw new ArgumentNullException("movie");

            CurrentProvider.UpdateMovie(movie);
        }

        /// <summary>
        /// Lscht einen Film.
        /// </summary>
        /// <remarks>
        /// Der Film kann nur gelscht werden, wenn weder Bewertungen noch 
        /// Vorstellungen existieren. Andernfalls wird ein Ausnahme ausgelst.
        /// </remarks>
        /// <param name="movie">Der zu lschenden Film.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="movie"/> ist <see langword="null"/>.</exception>
        public void DeleteMovie(Movie movie)
        {
            if (movie == null)
                throw new ArgumentNullException("movie");

            CurrentProvider.DeleteMovie(movie.Id);
        }

        /// <summary>
        /// Lscht einen Film.
        /// </summary>
        /// <remarks>
        /// Der Film kann nur gelscht werden, wenn weder Bewertungen noch 
        /// Vorstellungen existieren. Andernfalls wird eine Ausnahme ausgelst.
        /// </remarks>
        /// <param name="movieInfo">Der zu lschenden Film.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="movieInfo"/> ist <see langword="null"/>.</exception>
        public void DeleteMovie(MovieInfo movieInfo)
        {
            if (movieInfo == null)
                throw new ArgumentNullException("movieInfo");

            CurrentProvider.DeleteMovie(movieInfo.Id);
        }

        #endregion

        #region User

        /// <summary>
        /// Gibt ein volles Benutzerobjekt fr die Benutzerinformation.
        /// </summary>
        /// <param name="userInfo">Die Benutzerinformation fr das ein volles
        /// Benutzerobjekt geliefert werden soll.</param>
        /// <returns>Ein volles Benutzerobjekt.</returns>
        public User GetUser(UserInfo userInfo)
        {
            if (userInfo == null)
                throw new ArgumentNullException("userInfo");

            return CurrentProvider.GetUser(userInfo.Id);
        }

        /// <summary>
        /// Liefert eine Liste aller Benutzer, die einen Newsletter abonniert 
        /// haben.
        /// </summary>
        /// <returns>Eine Liste von Benutzern.</returns>
        public UserInfo[] GetUsersForNewsletter()
        {
            return CurrentProvider.GetUsersForNewsletter();
        }

        /// <summary>
        /// Registriert einen neuen Benutzer mit den gegebenen Daten und dem
        /// gegebenen Kennwort.
        /// </summary>
        /// <param name="user">Die Daten des zu erzeugenden Benutzers.</param>
        /// <param name="password">Das Klartext Kennwort des zu erzeugenden
        /// Benutzers.</param>
        /// <exception cref="ArgumentNullException"><para>
        /// <paramref name="user"/> ist <see langword="null"/>.</para>
        /// <para>- oder - </para>
        /// <para><paramref name="password"/> ist 
        /// <see langword="null"/>.</para>
        /// </exception>
        public void CreateUser(User user, string password)
        {
            if (user == null)
                throw new ArgumentNullException("user");

            if (password == null)
                throw new ArgumentNullException("password");

            CurrentProvider.CreateUser(user, password);
        }

        /// <summary>
        /// Aktualisiert einen Benutzer mit den gegebenen Daten.
        /// </summary>
        /// <param name="user">Die Daten des zu aktualsierenden Benutzers.
        /// </param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="user"/> ist <see langword="null"/>.</exception>
        public void UpdateUser(User user)
        {
            if (user == null)
                throw new ArgumentNullException("user");

            CurrentProvider.UpdateUser(user);
        }

        /// <summary>
        /// Lscht einen Benutzer.
        /// </summary>
        /// <remarks>
        /// Benutzer knnen physikalisch nicht gelscht werden, wenn Bewertungen
        /// oder Tickets mit dem Benutzer verknpft sind. In diesem Fall werden
        /// alle persnlichen Daten auf <see langword="null"/> gesetzt.
        /// </remarks>
        /// <param name="user">Der zu lschende Benutzer.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="user"/> ist <see langword="null"/>.</exception>
        public void DeleteUser(User user)
        {
            if (user == null)
                throw new ArgumentNullException("user");

            CurrentProvider.DeleteUser(user.Id);
        }

        /// <summary>
        /// Lscht einen Benutzer.
        /// </summary>
        /// <remarks>
        /// Benutzer knnen physikalisch nicht gelscht werden, wenn Bewertungen
        /// oder Tickets mit dem Benutzer verknpft sind. In diesem Fall werden
        /// alle persnlichen Daten auf <see langword="null"/> gesetzt.
        /// </remarks>
        /// <param name="userInfo">Der zu lschende Benutzer.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="userInfo"/> ist <see langword="null"/>.</exception>
        public void DeleteUser(UserInfo userInfo)
        {
            if (userInfo == null)
                throw new ArgumentNullException("userInfo");

            CurrentProvider.DeleteUser(userInfo.Id);
        }

        #endregion

        #region Room

        /// <summary>
        /// Liefert eine Liste aller Kinosale.
        /// </summary>
        /// <returns>Liefert eine Liste von Kinosalen.</returns>
        public Room[] GetRooms()
        {
            return CurrentProvider.GetRooms();
        }

        /// <summary>
        /// Erzeugt einen Kinosaal mit den gegebenen Daten.
        /// </summary>
        /// <param name="room">Die Daten des zu erzeugenden Kinosaals.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="room"/> ist <see langword="null"/>.</exception>
        public void CreateRoom(Room room)
        {
            if (room == null)
                throw new ArgumentNullException("room");

            CurrentProvider.CreateRoom(room);
        }

        /// <summary>
        /// Aktualisiert einen Kinosaal.
        /// </summary>
        /// <param name="room">Die Daten des zu aktualisierenden Kinosaals.
        /// </param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="room"/> ist <see langword="null"/>.</exception>
        public void UpdateRoom(Room room)
        {
            if (room == null)
                throw new ArgumentNullException("room");

            CurrentProvider.UpdateRoom(room);
        }

        /// <summary>
        /// Lscht einen Kinosaal.
        /// </summary>
        /// <remarks>        
        /// Ein Kinosaal kann nur gelscht werden, wenn keine zuknftigen 
        /// Vorstellungen existieren. Andernfalls wird eine Ausnahme geworfen.
        /// Zustzlich wird ein Kinosaal nur physikalisch gelscht, wenn keine
        /// Vorstellungen existieren. Andernfalls wir der Kinosaal nur 
        /// deaktviert.
        /// </remarks>
        /// <param name="room">Der zu lschende Kinosaal.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="room"/> ist <see langword="null"/>.</exception>
        public void DeleteRoom(Room room)
        {
            if (room == null)
                throw new ArgumentNullException("room");

            CurrentProvider.DeleteRoom(room.Id);
        }

        #endregion

        #region Ticket

        /// <summary>
        /// Erstellt ein Ticket fr den aktuell angemeldeten Benutzer und die
        /// gegebenene Vorstellung und Anzahl Pltze. Die Kosten werden dem
        /// Benutzer in Rechnung gestellt.
        /// </summary>
        /// <param name="performance">Die Vorstellung fr die ein Ticket gekauft
        /// werden soll.</param>
        /// <param name="quantity">Die Anzahl der Pltze.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="performance"/> ist <see langword="null"/>.
        /// </exception>
        /// <exception cref="ArgumentOutOfRangeException">
        /// <paramref name="quantity"/> ist nicht grer null.</exception>
        /// <returns>Den <see cref="Ticket.TicketCode"/> des Tickets.
        /// </returns>
        public string BuyTicket(Performance performance, int quantity)
        {
            if (performance == null)
                throw new ArgumentNullException("performance");

            if (quantity <= 0)
                throw new ArgumentOutOfRangeException("quantity",
                    Resource.QuantityMustBeGreaterThanZero);

            return CurrentProvider.BuyTicket(performance.Id, quantity);
        }

        /// <summary>
        /// Erstellt ein Ticket fr den aktuell angemeldeten Benutzer und die
        /// gegebenene Vorstellung und Anzahl Pltze. Die Kosten werden dem
        /// Benutzer in Rechnung gestellt.
        /// </summary>
        /// <param name="performanceInfo">Die Vorstellung fr die ein Ticket 
        /// gekauft werden soll.</param>
        /// <param name="quantity">Die Anzahl der Pltze.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="performanceInfo"/> ist <see langword="null"/>.
        /// </exception>
        /// <exception cref="ArgumentOutOfRangeException">
        /// <paramref name="quantity"/> ist nicht grer null.</exception>
        /// <returns>Den <see cref="Ticket.TicketCode"/> des Tickets.
        /// </returns>
        public string BuyTicket(PerformanceInfo performanceInfo, 
            int quantity)
        {
            if (performanceInfo == null)
                throw new ArgumentNullException("performanceInfo");

            if (quantity <= 0)
                throw new ArgumentOutOfRangeException("quantity",
                    Resource.QuantityMustBeGreaterThanZero);

            return CurrentProvider.BuyTicket(performanceInfo.Id, quantity);
        }

        /// <summary>
        /// Verkauft ein Ticket anonym. Diese Methode wird vom Kassenpersonal
        /// aufgerufen. Die Kosten werden niemanden in Rechnung gestellt, da
        /// das Kassenpersonal vor Ort kassiert.
        /// </summary>
        /// <param name="performance">Die Vorstellung fr die ein Ticket gekauft
        /// werden soll.</param>
        /// <param name="quantity">Die Anzahl der Pltze.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="performance"/> ist <see langword="null"/>.
        /// </exception>
        /// <exception cref="ArgumentOutOfRangeException">
        /// <paramref name="quantity"/> ist nicht grer null.</exception>
        public void SellTicket(Performance performance, int quantity)
        {
            if (performance == null)
                throw new ArgumentNullException("performance");

            if (quantity <= 0)
                throw new ArgumentOutOfRangeException("quantity",
                    Resource.QuantityMustBeGreaterThanZero);

            CurrentProvider.SellTicket(performance.Id, quantity);
        }

        /// <summary>
        /// Verkauft ein Ticket anonym. Diese Methode wird vom Kassenpersonal
        /// aufgerufen. Die Kosten werden niemanden in Rechnung gestellt, da
        /// das Kassenpersonal vor Ort kassiert.
        /// </summary>
        /// <param name="performanceInfo">Die Vorstellung fr die ein Ticket 
        /// gekauft werden soll.</param>
        /// <param name="quantity">Die Anzahl der Pltze.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="performanceInfo"/> ist <see langword="null"/>.
        /// </exception>
        /// <exception cref="ArgumentOutOfRangeException">
        /// <paramref name="quantity"/> ist nicht grer null.</exception>
        public void SellTicket(PerformanceInfo performanceInfo, int quantity)
        {
            if (performanceInfo == null)
                throw new ArgumentNullException("performanceInfo");

            if (quantity <= 0)
                throw new ArgumentOutOfRangeException("quantity",
                    Resource.QuantityMustBeGreaterThanZero);

            CurrentProvider.SellTicket(performanceInfo.Id, quantity);
        }

        /// <summary>
        /// Liefert eine Liste aller noch nicht abgeholten Tickets fr den 
        /// gegebenen Benutzer.
        /// </summary>
        /// <param name="user">Der Benutzer, dessen noch nicht abholten Tickets
        /// geliefert werden soll.</param>
        /// <returns>Eine List von Tickts.</returns>
        /// <exception cref="ArgumentException">
        /// <paramref name="user"/> ist <see langword="null"/>.
        /// </exception>
        public Ticket[] GetUndispatchedTickets(User user)
        {
            if (user == null)
                throw new ArgumentNullException("user");

            return CurrentProvider.GetUndispatchedTickets(user.Id);
        }

        /// <summary>
        /// Liefert eine Liste aller noch nicht abgeholten Tickets fr den 
        /// gegebenen Benutzer.
        /// </summary>
        /// <param name="userInfo">Der Benutzer, dessen noch nicht abholten 
        /// Tickets geliefert werden soll.</param>
        /// <returns>Eine List von Tickts.</returns>
        /// <exception cref="ArgumentException">
        /// <paramref name="userInfo"/> ist <see langword="null"/>.
        /// </exception>
        public Ticket[] GetUndispatchedTickets(UserInfo userInfo)
        {
            if (userInfo == null)
                throw new ArgumentNullException("userInfo");

            return CurrentProvider.GetUndispatchedTickets(userInfo.Id);
        }

        /// <summary>
        /// Liefert eine Liste aller Tickets mit dem gegebenen Ticketcode. Die
        /// Ticketcodes sind nur pro Benutzer eindeutig. Fr eine Unterscheidung
        /// muss vor Ort der Name des Benutzers verglichen werden.
        /// </summary>
        /// <param name="ticketCode">Der Ticketcode</param>
        /// <returns>Eine Liste von Tickts.</returns>
        /// <exception cref="ArgumentException">
        /// <paramref name="ticketCode"/> ist <see langword="null"/> or 
        /// <see cref="String.Empty"/>.
        /// </exception>
        public Ticket[] GetUndispatchedTickets(string ticketCode)
        {
            if (String.IsNullOrEmpty(ticketCode))
                throw new ArgumentException(Resource.ArgumentNullOrEmptyString, 
                    "ticketCode");

            return CurrentProvider.GetUndispatchedTickets(ticketCode);
        }

        /// <summary>
        /// Markiert das gegebene Ticket als abgeholt. Das bedeutet, es ist 
        /// nicht lnger verfgbar und wird von den folgenden Methoden nicht
        /// mehr zurckgegeben:
        /// <see cref="GetUndispatchedTickets(string)"/>,
        /// <see cref="GetUndispatchedTickets(User)"/> und
        /// <see cref="GetUndispatchedTickets(UserInfo)"/>
        /// </summary>
        /// <param name="ticket">Das als abgeholt zu markierendes Ticket.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="ticket"/> ist <see langword="null"/>
        /// </exception>.
        public void MarkTicketAsDispatched(Ticket ticket)
        {
            if (ticket == null)
                throw new ArgumentNullException("ticket");

            CurrentProvider.MarkTicketAsDispatched(ticket);
        }

        #endregion

        #region News

        /// <summary>
        /// Liefert eine List aller Newseintrge.
        /// </summary>
        /// <returns>Eine Liste mit allen Newseintrgen.</returns>
        public NewsItem[] GetNewsItems()
        {
            return CurrentProvider.GetNewsItems();
        }

        /// <summary>
        /// Liefert eine Liste aller Newseintrgen die in einer einer oder 
        /// mehreren der gegebenenen Kategorien sind.
        /// </summary>
        /// <param name="numbers">Eine List von Newskategorien, reprsentiert
        /// durch die Kategorienummer (<see cref="NewsCategory.Number"/>)
        /// </param>
        /// <returns>Eine List mit News.</returns>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="numbers"/> ist <see langword="null"/>.</exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="numbers"/> ist leer.</exception>
        public NewsItem[] GetNewsItemsByCategories(int[] numbers)
        {
            if (numbers == null)
                throw new ArgumentNullException("numbers");

            if (numbers.Length == 0)
                throw new ArgumentException(Resource.ArgumentEmptyArray, 
                    "numbers");

            return CurrentProvider.GetNewsItemsByCategories(numbers);
        }

        /// <summary>
        /// Erzeugt einen Newseintrag mit den gegebenen Daten.
        /// </summary>
        /// <param name="newsItem">Die Daten des zu erzeugenden Newseintrags.
        /// </param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="newsItem"/> ist <see langword="null"/>.</exception>
        public void CreateNewsItem(NewsItem newsItem)
        {
            if (newsItem == null)
                throw new ArgumentNullException("newsItem");

            CurrentProvider.CreateNewsItem(newsItem);
        }

        /// <summary>
        /// Aktualisiert einen Newseintrag mit den gegebenen Daten.
        /// </summary>
        /// <param name="newsItem">Die Daten des zu aktualisierenden 
        /// Newseintrags.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="newsItem"/> ist <see langword="null"/>.</exception>
        public void UpdateNewsItem(NewsItem newsItem)
        {
            if (newsItem == null)
                throw new ArgumentNullException("newsItem");

            CurrentProvider.UpdateNewsItem(newsItem);
        }

        /// <summary>
        /// Lscht einen Newseintrag.
        /// </summary>
        /// <param name="newsItem">Der zu lschende Newseintrag.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="newsItem"/> ist <see langword="null"/>.</exception>
        public void DeleteNewsItem(NewsItem newsItem)
        {
            if (newsItem == null)
                throw new ArgumentNullException("newsItem");

            CurrentProvider.DeleteNewsItem(newsItem.Id);
        }

        #endregion

        #region News Category

        /// <summary>
        /// Liefert eine Liste aller Newskategorien.
        /// </summary>
        /// <returns>Eine Liste aller Newskategorien.</returns>
        public NewsCategory[] GetNewsCategories()
        {
            return CurrentProvider.GetNewsCategories();
        }

        /// <summary>
        /// Erzeugt eine Newskategorie mit den gegebenen Daten.
        /// </summary>
        /// <param name="category">Die Daten des zu erzeugenden Newseintrags.
        /// </param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="category"/> ist <see langword="null"/>.</exception>
        public void CreateNewsCategory(NewsCategory category)
        {
            if (category == null)
                throw new ArgumentNullException("category");

            CurrentProvider.CreateNewsCategory(category);
        }

        /// <summary>
        /// Aktualisiert eine Newskategorie mit gegebenen Daten.
        /// </summary>
        /// <param name="category">Die Daten des zu aktualisierenden 
        /// Newseintrags.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="category"/> ist <see langword="null"/>.</exception>
        public void UpdateNewsCategory(NewsCategory category)
        {
            if (category == null)
                throw new ArgumentNullException("category");

            CurrentProvider.UpdateNewsCategory(category);
        }

        /// <summary>
        /// Lscht eine Newskategorie.
        /// </summary>
        /// <param name="category">Die zu lschende Newskategorie.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="category"/> ist <see langword="null"/>.</exception>
        public void DeleteNewsCategory(NewsCategory category)
        {
            if (category == null)
                throw new ArgumentNullException("category");

            CurrentProvider.DeleteNewsCategory(category.Id);
        }

        #endregion

        #region Genre

        /// <summary>
        /// Liefert eine Liste aller Genres.
        /// </summary>
        /// <returns>Eine Liste aller Genres.</returns>
        public Genre[] GetGenres()
        {
            return CurrentProvider.GetGenres();
        }

        /// <summary>
        /// Erzeugt ein Genre mit den gegebenen Daten.
        /// </summary>
        /// <param name="genre">Die Daten des zu erzeugenden 
        /// Genres.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="genre"/> ist <see langword="null"/>.
        /// </exception>
        public void CreateGenre(Genre genre)
        {
            if (genre == null)
                throw new ArgumentNullException("genre");

            CurrentProvider.CreateGenre(genre);
        }

        /// <summary>
        /// Aktualisiert ein Genre mit gegebenen Daten.
        /// </summary>
        /// <param name="genre">Die Daten des zu aktualisierenden 
        /// Genres.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="genre"/> ist <see langword="null"/>.
        /// </exception>
        public void UpdateGenre(Genre genre)
        {
            if (genre == null)
                throw new ArgumentNullException("genre");

            CurrentProvider.UpdateGenre(genre);
        }

        /// <summary>
        /// Lscht ein Genre.
        /// </summary>
        /// <param name="genre">Das zu lschende Genre.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="genre"/> ist <see langword="null"/>.
        /// </exception>
        public void DeleteGenre(Genre genre)
        {
            if (genre == null)
                throw new ArgumentNullException("genre");

            CurrentProvider.DeleteGenre(genre.Id);
        }

        #endregion

        #region Country

        /// <summary>
        /// Liefert eine Liste aller Lnder.
        /// </summary>
        /// <returns>Eine Liste aller Lnder.</returns>
        public Country[] GetCountries()
        {
            return CurrentProvider.GetCountries();
        }

        /// <summary>
        /// Erzeugt ein Land mit den gegebenen Daten.
        /// </summary>
        /// <param name="country">Die Daten des zu erzeugenden 
        /// Landes.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="country"/> ist <see langword="null"/>.
        /// </exception>
        public void CreateCountry(Country country)
        {
            if (country == null)
                throw new ArgumentNullException("country");

            CurrentProvider.CreateCountry(country);
        }

        /// <summary>
        /// Aktualisiert ein Land mit gegebenen Daten.
        /// </summary>
        /// <param name="country">Die Daten des zu aktualisierenden 
        /// Landes.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="country"/> ist <see langword="null"/>.
        /// </exception>
        public void UpdateCountry(Country country)
        {
            if (country == null)
                throw new ArgumentNullException("country");

            CurrentProvider.UpdateCountry(country);
        }

        /// <summary>
        /// Lscht ein Land.
        /// </summary>
        /// <param name="country">Das zu lschende Land.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="country"/> ist <see langword="null"/>.
        /// </exception>
        public void DeleteCountry(Country country)
        {
            if (country == null)
                throw new ArgumentNullException("country");

            CurrentProvider.DeleteCountry(country.Id);
        }

        #endregion

        #region Distributor

        /// <summary>
        /// Liefert eine Liste aller Verleiher.
        /// </summary>
        /// <returns>Eine Liste aller Verleiher.</returns>
        public Distributor[] GetDistributors()
        {
            return CurrentProvider.GetDistributors();
        }

        /// <summary>
        /// Erzeugt einen Verleih mit den gegebenen Daten.
        /// </summary>
        /// <param name="distributor">Die Daten des zu erzeugenden 
        /// Verleihs.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="distributor"/> ist <see langword="null"/>.
        /// </exception>
        public void CreateDistributor(Distributor distributor)
        {
            if (distributor == null)
                throw new ArgumentNullException("distributor");

            CurrentProvider.CreateDistributor(distributor);
        }

        /// <summary>
        /// Aktualisiert einen Verleih mit gegebenen Daten.
        /// </summary>
        /// <param name="distributor">Die Daten der zu aktualisierenden 
        /// Alterfreigabe.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="distributor"/> ist <see langword="null"/>.
        /// </exception>
        public void UpdateDistributor(Distributor distributor)
        {
            if (distributor == null)
                throw new ArgumentNullException("distributor");

            CurrentProvider.UpdateDistributor(distributor);
        }

        /// <summary>
        /// Lscht einen Verleih.
        /// </summary>
        /// <param name="distributor">Der zu lschende Verleih.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="distributor"/> ist <see langword="null"/>.
        /// </exception>
        public void DeleteDistributor(Distributor distributor)
        {
            if (distributor == null)
                throw new ArgumentNullException("distributor");

            CurrentProvider.DeleteDistributor(distributor.Id);
        }

        #endregion

        #region Parental Guidance

        /// <summary>
        /// Liefert eine Liste aller Alterfreigaben.
        /// </summary>
        /// <returns>Eine Liste aller Alterfreigaben.</returns>
        public ParentalGuidance[] GetParentalGuidances()
        {
            return CurrentProvider.GetParentalGuidances();
        }

        /// <summary>
        /// Erzeugt eine Alterfreigabe mit den gegebenen Daten.
        /// </summary>
        /// <param name="parentalGuidance">Die Daten der zu erzeugenden 
        /// Alterfreigabe.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="parentalGuidance"/> ist <see langword="null"/>.
        /// </exception>
        public void CreateParentalGuidance(ParentalGuidance parentalGuidance)
        {
            if (parentalGuidance == null)
                throw new ArgumentNullException("parentalGuidance");

            CurrentProvider.CreateParentalGuidance(parentalGuidance);
        }

        /// <summary>
        /// Aktualisiert eine Alterfreigabe mit gegebenen Daten.
        /// </summary>
        /// <param name="parentalGuidance">Die Daten der zu aktualisierenden 
        /// Alterfreigabe.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="parentalGuidance"/> ist <see langword="null"/>.
        /// </exception>
        public void UpdateParentalGuidance(ParentalGuidance parentalGuidance)
        {
            if (parentalGuidance == null)
                throw new ArgumentNullException("parentalGuidance");

            CurrentProvider.UpdateParentalGuidance(parentalGuidance);
        }

        /// <summary>
        /// Lscht eine Alterfreigabe.
        /// </summary>
        /// <param name="parentalGuidance">Die zu lschende Alterfreigabe.
        /// </param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="parentalGuidance"/> ist <see langword="null"/>.
        /// </exception>
        public void DeleteParentalGuidance(ParentalGuidance parentalGuidance)
        {
            if (parentalGuidance == null)
                throw new ArgumentNullException("parentalGuidance");

            CurrentProvider.DeleteParentalGuidance(parentalGuidance.Id);
        }

        #endregion

        #region Person

        /// <summary>
        /// Liefert eine Liste aller Personen, d.h. Schauspieler, Produzenten
        /// oder Regisseure.
        /// </summary>
        /// <returns>Eine Liste aller Personen.</returns>
        public Person[] GetPersons()
        {
            return CurrentProvider.GetPersons();
        }

        /// <summary>
        /// Erzeugt eine Person mit den gegebenen Daten.
        /// </summary>
        /// <param name="person">Die Daten der zu erzeugenden Person.
        /// </param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="person"/> ist <see langword="null"/>.</exception>
        public void CreatePerson(Person person)
        {
            if (person == null)
                throw new ArgumentNullException("person");

            CurrentProvider.CreatePerson(person);
        }

        /// <summary>
        /// Aktualisiert eine Person mit gegebenen Daten.
        /// </summary>
        /// <param name="person">Die Daten der zu aktualisierenden 
        /// Person.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="person"/> ist <see langword="null"/>.</exception>
        public void UpdatePerson(Person person)
        {
            if (person == null)
                throw new ArgumentNullException("person");

            CurrentProvider.UpdatePerson(person);
        }

        /// <summary>
        /// Lscht eine Person.
        /// </summary>
        /// <param name="person">Die zu lschende Person.</param>
        /// <exception cref="ArgumentNullException">
        /// <paramref name="person"/> ist <see langword="null"/>.</exception>
        public void DeletePerson(Person person)
        {
            if (person == null)
                throw new ArgumentNullException("person");

            CurrentProvider.DeletePerson(person.Id);
        }

        #endregion
    }
}
