using System;
using System.Collections.Generic;
using System.Globalization;
using System.Net;
using System.Text;
using System.Web.Services.Protocols;
using System.Xml;

using AdventureWorks.Cinema.ServiceAccessLayer.Caching;

namespace AdventureWorks.Cinema.ServiceAccessLayer.WebService
{
    /// <summary>
    /// Dieser Service Provider stellt den "echten" Service Provider dar, der
    /// mit dem Web Service des Kinos kommuniziert um lesende, erzeugende,
    /// schreibende 
    /// </summary>
    internal sealed class WebServiceDataProvider : IServiceAccessProvider
    {
        private CinemaServiceImplementation webService;

        public WebServiceDataProvider(Uri webServiceUrl)
        {
            // Erzeuge den Web Service Proxy mit der gegebenen URL.
            webService = new CinemaServiceImplementation();
            webService.Url = webServiceUrl.ToString();
        }

        // HINWEIS: Nahezu alle Methoden folgen unten stehendem Muster:
        //
        // {
        //    << Argument prfen >>
        //
        //    << Erzeuge Web Service Anfrage-Objekt >>
        //
        //    try
        //    {
        //       << Web Service aufrufen >>
        //
        //       return << result >>;
        //    }
        //    catch (Exception ex)
        //    {
        //       Exception wrappedException = WrapException(ex);
        //
        //       if (wrappedException != null)
        //           throw wrappedException;
        //
        //       throw;
        //    }
        // }
        //
        // Da dieses Muster sehr simpel und transparent ist, wurden alle 
        // Methoden, diesem Muster folgen, nicht weiter kommentiert.
        //
        // Fr Details wie Ausnahmen behandelt werden siehe WrapException().

        #region Exception Handling

        private static Exception WrapException(Exception exception)
        {
            // Zunchst wird geprft, ob es sich bei exception um eine Instanz
            // von WebException handelt. Dies ist dann der Fall, wenn etwas
            // grundlegendes schief gegangen ist, z.B. wenn keine Verbindung 
            // zum Server aufgabaut werden konnte etc.
            WebException webException = exception as WebException;

            if (webException != null)
            {
                return new ServiceAccessException(
                    String.Format(CultureInfo.CurrentUICulture,
                        Resource.WebExceptionOccurred, exception.Message),
                    exception);
            }

            // Prfe, ob es sich um eine SoapException handelt. SoapExceptions
            // werden vom Web Service zurckgegeben. Das knnen interne Server
            // Fehler (Bugs, Datenbankfehler ect.) oder Verletzungen von
            // Geschftsregeln (Falsche Benutzername oder Kennwort etc.) sein.
            SoapException soapException = exception as SoapException;

            if (soapException != null)
            {
                // Der Web Service liefert detailierte Information ber den 
                // Fehler in SoapException.Detail als XML Knoten.
                //
                // Prfe, ob dieser vorhanden ist.
                if (soapException.Detail != null)
                {
                    // Innerhalb dieses Knotens liegt der Text der original
                    // Aushname in dem Knoten "Message".
                    //
                    // HINWEIS: Wir htten auch Exception.Message verwenden
                    //          knnen, aber dieser Text enthlt auch ein Teile
                    //          wie "SoapException. Der Server war nicht in der
                    //          Lage die Anfrage zu bearbeiten --> ...". Dieser
                    //          Text ist daher nicht besonders 
                    //          benutzerfreundlich.

                    XmlNode messageNode = soapException.Detail.SelectSingleNode(
                        "Message");

                    if (messageNode != null)
                        return new ServiceAccessException(messageNode.InnerText,
                            exception);
                }

                // Die SoapException enthlt leider keine benutzerfreundliche
                // Fehlermeldung. Wir verpacken die SoapException in einer
                // ServiceAccessException und verwenden SoapException.Message
                // als Message.
                return new ServiceAccessException(exception.Message,
                    exception);
            }

            // Es handelt sich weder um eine WebException noch um eine 
            // SoapException, daher verpacken und behandeln wir die Ausnahme
            // nicht.
            //
            // Der Aufrufer ruf uns wie folgt auf:
            //
            // try
            // {
            //    ...
            // }
            // catch (Exception ex)
            // {
            //     Exception wrappedException = WrapException(ex);
            //
            //     if (wrappedException != null)
            //         throw wrappedException;
            //
            //     throw;
            // }
            //
            // Damit wird sichergestellt, dass, falls die Ausnahme nicht
            // verpackt wird, der Stack Trace nicht verndert wird (weil
            // "throw;" den Stack Trace unverndert lsst, "throw ex;"
            // allerdings einen neuen Stack Trace, abhngig vom Call Stack
            // zuweist).
            //
            // Daher geben wir NULL zurck um anzuzeigen, dass wir die Ausnahme
            // nicht verpackt haben.

            return null;
        }

        #endregion

        #region Session

        public void ConnectToSession(string sessionId)
        {
            if (String.IsNullOrEmpty(sessionId))
                throw new ArgumentException(Resource.ArgumentNullOrEmptyString);

            // Versuche die Session ID in eine Guid zu konvertieren.
            Guid sessionIdGuid;
            try
            {
                sessionIdGuid = new Guid(sessionId);
            }
            catch (FormatException)
            {
                // Offensichtlich handelt es sich nicht um eine gltige Guid 
                // und daher auch nicht um eine gltige Session ID.
                throw new ServiceAccessException(Resource.InvalidSessionId);
            }

            // Von jetzt an wird die Session ID bei jedem Web Servcice Aufruf
            // an selbigen bergeben.
            //
            // Die Session ID wird ber einen sog. Custom Soap Header
            // bertragen, den wir hier auch gleich initialisieren.
            webService.SessionHeaderValue = new SessionHeader();
            webService.SessionHeaderValue.SessionId = sessionIdGuid;
        }

        public string SessionId
        {
            get
            {
                if (webService.SessionHeaderValue == null)
                    return String.Empty;

                return webService.SessionHeaderValue.SessionId.ToString();
            }
        }

        public User LogOn(string userName, string password)
        {
            LogOnRequestMessage request = new LogOnRequestMessage();
            request.UserName = userName;
            request.Password = password;

            try
            {
                LogOnResult result = webService.LogOn(request);
                return result.User;
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void LogOff()
        {
            try
            {
                webService.LogOff();
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void ChangePassword(string currentPassword, string newPassword)
        {
            ChangePasswordRequestMessage request =
                new ChangePasswordRequestMessage();
            request.CurrentPassword = currentPassword;
            request.NewPassword = newPassword;

            try
            {
                webService.ChangePassword(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void ResetPassword(string userName)
        {
            try
            {
                ResetPasswordRequestMessage request
                    = new ResetPasswordRequestMessage();
                request.UserName = userName;

                webService.ResetPassword(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public User GetCurrentUser()
        {
            try
            {
                GetCurrentUserResult response = webService.GetCurrentUser();
                return response.CurrentUser;
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        #endregion

        #region Performance

        public PerformanceInfo[] GetPerformances(DateTime startDate,
            DateTime endDate)
        {
            if (endDate < startDate)
                throw new ArgumentException(
                    Resource.EndDateMustBeBeforeStartDate, "endDate");

            GetPerformancesRequestMessage request = 
                new GetPerformancesRequestMessage();
            request.StartDate = startDate;
            request.EndDate = endDate;

            try
            {
                PerformanceInfo[] result = webService.GetPerformances(request);
                return result;
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public PerformanceInfo[] GetPerformancesOfMovie(Guid movieId)
        {
            GetPerformancesOfMovieRequestMessage request =
                new GetPerformancesOfMovieRequestMessage();
            request.MovieId = movieId;

            try
            {
                return webService.GetPerformancesOfMovie(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public PerformanceInfo[] GetPerformancesOfRoom(Guid roomId,
            DateTime startDate, DateTime endDate)
        {
            GetPerformancesOfRoomRequestMessage request =
                new GetPerformancesOfRoomRequestMessage();
            request.RoomId = roomId;
            request.StartDate = startDate;
            request.EndDate = endDate;

            try
            {
                return webService.GetPerformancesOfRoom(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public Performance GetPerformance(Guid performanceId)
        {
            GetPerformanceRequestMessage request =
                new GetPerformanceRequestMessage();
            request.PerformanceId = performanceId;

            try
            {
                GetPerformanceResult response =
                    webService.GetPerformance(request);

                return response.Performance;
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void CreatePerformance(Performance performance)
        {
            if (performance == null)
                throw new ArgumentNullException("performance");

            CreatePerformanceRequestMessage request =
                new CreatePerformanceRequestMessage();
            request.Performance = performance;

            try
            {
                webService.CreatePerformance(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void UpdatePerformance(Performance performance)
        {
            if (performance == null)
                throw new ArgumentNullException("performance");

            UpdatePerformanceRequestMessage request =
                new UpdatePerformanceRequestMessage();
            request.Performance = performance;

            try
            {
                webService.UpdatePerformance(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void DeletePerformance(Guid performanceId)
        {
            DeletePerformanceRequestMessage request =
                new DeletePerformanceRequestMessage();
            request.PerformanceId = performanceId;

            try
            {
                webService.DeletePerformance(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public int GetFreeSeats(Guid performanceId)
        {
            GetFreeSeatsRequestMessage request =
                new GetFreeSeatsRequestMessage();
            request.PerformanceId = performanceId;

            try
            {
                GetFreeSeatsResult response = webService.GetFreeSeats(request);
                int result = response.FreeSeats;
                return result;
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        #endregion

        #region Movie

        public MovieInfo[] GetMovies()
        {
            try
            {
                MovieInfo[] result = webService.GetMovies();
                return result;
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public MovieInfo[] GetMovies(DateTime startDate)
        {
            try
            {
                MovieInfo[] result = webService.GetMovies();
                return result;
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public MovieInfo[] GetMoviesByUserInterests(Guid userId)
        {
            GetMoviesByUserInterestsRequestMessage request =
                new GetMoviesByUserInterestsRequestMessage();
            request.UserId = userId;

            try
            {
                MovieInfo[] result = webService.GetMoviesByUserInterests(
                    request);

                return result;
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public MovieInfo[] GetMoviesByTitle(string movieTitle)
        {
            if (String.IsNullOrEmpty(movieTitle))
                throw new ArgumentException(Resource.ArgumentNullOrEmptyString,
                    "movieTitle");

            GetMoviesByNameRequestMessage request =
                new GetMoviesByNameRequestMessage();
            request.MovieName = movieTitle;

            try
            {
                return webService.GetMoviesByName(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public Movie GetMovie(Guid movieId)
        {
            GetMovieRequestMessage request = new GetMovieRequestMessage();
            request.MovieId = movieId;

            try
            {
                GetMovieResult response = webService.GetMovie(request);
                return response.Movie;
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void RateMovie(Guid movieId, int rating)
        {
            RateMovieRequestMessage request = new RateMovieRequestMessage();
            request.MovieId = movieId;
            request.Rating = rating;

            try
            {
                webService.RateMovie(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void CreateMovie(Movie movie)
        {
            if (movie == null)
                throw new ArgumentNullException("movie");

            CreateMovieRequestMessage request =
                new CreateMovieRequestMessage();
            request.Movie = movie;

            try
            {
                webService.CreateMovie(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void UpdateMovie(Movie movie)
        {
            if (movie == null)
                throw new ArgumentNullException("movie");

            UpdateMovieRequestMessage request =
                new UpdateMovieRequestMessage();
            request.Movie = movie;

            try
            {
                webService.UpdateMovie(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void DeleteMovie(Guid movieId)
        {
            DeleteMovieRequestMessage request =
                new DeleteMovieRequestMessage();
            request.MovieId = movieId;

            try
            {
                webService.DeleteMovie(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        #endregion

        #region User

        public User GetUser(Guid userId)
        {
            GetUserDataRequestMessage request =
                new GetUserDataRequestMessage();
            request.UserId = userId;

            try
            {
                GetUserDataResult response = webService.GetUserData(
                    request);
                return response.UserData;
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public UserInfo[] GetUsersForNewsletter()
        {
            try
            {
                UserInfo[] result = webService.GetUsersForNewsletter();
                return result;
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void CreateUser(User user, string password)
        {
            if (user == null)
                throw new ArgumentNullException("user");

            if (password == null)
                throw new ArgumentNullException("password");

            CreateUserRequestMessage request =
                new CreateUserRequestMessage();
            request.User = user;
            request.Password = password;

            try
            {
                webService.CreateUser(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void UpdateUser(User user)
        {
            if (user == null)
                throw new ArgumentNullException("user");

            UpdateUserRequestMessage request =
                new UpdateUserRequestMessage();
            request.User = user;

            try
            {
                webService.UpdateUser(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void DeleteUser(Guid userId)
        {
            DeleteUserRequestMessage request =
                new DeleteUserRequestMessage();
            request.UserId = userId;

            try
            {
                webService.DeleteUser(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        #endregion

        #region Room

        public Room[] GetRooms()
        {
            try
            {
                Room[] result = webService.GetRooms();
                return result;
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void CreateRoom(Room room)
        {
            if (room == null)
                throw new ArgumentNullException("room");

            CreateRoomRequestMessage request = new CreateRoomRequestMessage();
            request.Room = room;

            try
            {
                webService.CreateRoom(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void UpdateRoom(Room room)
        {
            if (room == null)
                throw new ArgumentNullException("room");

            UpdateRoomRequestMessage request = new UpdateRoomRequestMessage();
            request.Room = room;

            try
            {
                webService.UpdateRoom(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void DeleteRoom(Guid roomId)
        {
            DeleteRoomRequestMessage request = new DeleteRoomRequestMessage();
            request.RoomId = roomId;

            try
            {
                webService.DeleteRoom(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        #endregion

        #region Ticket

        public string BuyTicket(Guid performanceId, int quantity)
        {
            if (quantity <= 0)
                throw new ArgumentOutOfRangeException("quantity",
                    Resource.QuantityMustBeGreaterThanZero);

            BuyTicketsRequestMessage request = new BuyTicketsRequestMessage();
            request.PerformanceId = performanceId;
            request.Quantity = quantity;

            try
            {
                BuyTicketsResult response = webService.BuyTickets(request);
                return response.TicketCode;
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void SellTicket(Guid performanceId, int quantity)
        {
            if (quantity <= 0)
                throw new ArgumentOutOfRangeException("quantity",
                    Resource.QuantityMustBeGreaterThanZero);

            SellTicketsRequestMessage request = new SellTicketsRequestMessage();
            request.PerformanceId = performanceId;
            request.Quantity = quantity;

            try
            {
                webService.SellTickets(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public Ticket[] GetUndispatchedTickets(Guid userId)
        {
            GetUndispatchedTicketsByUserIdRequestMessage request =
                new GetUndispatchedTicketsByUserIdRequestMessage();
            request.UserId = userId;

            try
            {
                return webService.GetUndispatchedTicketsByUserId(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public Ticket[] GetUndispatchedTickets(string ticketCode)
        {
            if (String.IsNullOrEmpty(ticketCode))
                throw new ArgumentException(Resource.ArgumentNullOrEmptyString,
                    "ticketCode");

            GetUndispatchedTicketsRequestMessage request =
                new GetUndispatchedTicketsRequestMessage();
            request.TicketCode = ticketCode;

            try
            {
                return webService.GetUndispatchedTickets(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void MarkTicketAsDispatched(Ticket ticket)
        {
            if (ticket == null)
                throw new ArgumentNullException("ticket");

            MarkTicketAsDispatchedRequestMessage request =
                new MarkTicketAsDispatchedRequestMessage();
            request.TicketId = ticket.Id;

            try
            {
                webService.MarkTicketAsDispatched(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        #endregion

        #region News

        public NewsItem[] GetNewsItems()
        {
            try
            {
                NewsItem[] result = webService.GetNewsItems();
                return result;
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void CreateNewsItem(NewsItem newsItem)
        {
            if (newsItem == null)
                throw new ArgumentNullException("newsItem");

            CreateNewsItemRequestMessage request =
                new CreateNewsItemRequestMessage();
            request.NewsItem = newsItem;

            try
            {
                webService.CreateNewsItem(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void UpdateNewsItem(NewsItem newsItem)
        {
            if (newsItem == null)
                throw new ArgumentNullException("newsItem");

            UpdateNewsItemRequestMessage request =
                new UpdateNewsItemRequestMessage();
            request.NewsItem = newsItem;

            try
            {
                webService.UpdateNewsItem(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void DeleteNewsItem(Guid newsItemId)
        {
            DeleteNewsItemRequestMessage request =
                new DeleteNewsItemRequestMessage();
            request.NewsItemId = newsItemId;

            try
            {
                webService.DeleteNewsItem(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public NewsItem[] GetNewsItemsByCategories(int[] numbers)
        {
            if (numbers == null)
                throw new ArgumentNullException("numbers");

            if (numbers.Length == 0)
                throw new ArgumentException(Resource.ArgumentEmptyArray,
                    "number");

            try
            {
                NewsItem[] result = webService.GetNewsItemsByCategoryNumber(
                    numbers);
                return result;
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        #endregion

        #region News Categories

        public NewsCategory[] GetNewsCategories()
        {
            try
            {
                NewsCategory[] result = webService.GetNewsCategories();
                return result;
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void CreateNewsCategory(NewsCategory category)
        {
            if (category == null)
                throw new ArgumentNullException("category");

            CreateNewsCategoryRequestMessage request =
                new CreateNewsCategoryRequestMessage();
            request.NewsCategory = category;

            try
            {
                webService.CreateNewsCategory(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void UpdateNewsCategory(NewsCategory category)
        {
            if (category == null)
                throw new ArgumentNullException("category");

            UpdateNewsCategoryRequestMessage request =
                new UpdateNewsCategoryRequestMessage();
            request.NewsCategory = category;

            try
            {
                webService.UpdateNewsCategory(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void DeleteNewsCategory(Guid categoryId)
        {
            DeleteNewsCategoryRequestMessage request =
                new DeleteNewsCategoryRequestMessage();
            request.NewsCategoryId = categoryId;

            try
            {
                webService.DeleteNewsCategory(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        #endregion

        #region Genre

        public Genre[] GetGenres()
        {
            try
            {
                return webService.GetAllGenres();
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void CreateGenre(Genre genre)
        {
            if (genre == null)
                throw new ArgumentNullException("genre");

            try
            {
                CreateGenreRequestMessage request =
                    new CreateGenreRequestMessage();
                request.Genre = genre;

                webService.CreateGenre(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void UpdateGenre(Genre genre)
        {
            if (genre == null)
                throw new ArgumentNullException("genre");

            try
            {
                UpdateGenreRequestMessage request =
                    new UpdateGenreRequestMessage();
                request.Genre = genre;

                webService.UpdateGenre(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void DeleteGenre(Guid genreId)
        {
            try
            {
                DeleteGenreRequestMessage request =
                    new DeleteGenreRequestMessage();
                request.GenreId = genreId;

                webService.DeleteGenre(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        #endregion

        #region Country

        public Country[] GetCountries()
        {
            try
            {
                return webService.GetAllCountries();
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void CreateCountry(Country country)
        {
            if (country == null)
                throw new ArgumentNullException("country");

            try
            {
                CreateCountryRequestMessage request =
                    new CreateCountryRequestMessage();
                request.Country = country;

                webService.CreateCountry(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void UpdateCountry(Country country)
        {
            if (country == null)
                throw new ArgumentNullException("country");

            try
            {
                UpdateCountryRequestMessage request =
                    new UpdateCountryRequestMessage();
                request.Country = country;

                webService.UpdateCountry(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void DeleteCountry(Guid countryId)
        {
            try
            {
                DeleteCountryRequestMessage request =
                    new DeleteCountryRequestMessage();
                request.CountryId = countryId;

                webService.DeleteCountry(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        #endregion

        #region Distributor

        public Distributor[] GetDistributors()
        {
            try
            {
                return webService.GetAllDistributors();
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void CreateDistributor(Distributor distributor)
        {
            if (distributor == null)
                throw new ArgumentNullException("distributor");

            try
            {
                CreateDistributorRequestMessage request =
                    new CreateDistributorRequestMessage();
                request.Distributor = distributor;

                webService.CreateDistributor(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void UpdateDistributor(Distributor distributor)
        {
            if (distributor == null)
                throw new ArgumentNullException("distributor");

            try
            {
                UpdateDistributorRequestMessage request =
                    new UpdateDistributorRequestMessage();
                request.Distributor = distributor;

                webService.UpdateDistributor(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void DeleteDistributor(Guid distributorId)
        {
            try
            {
                DeleteDistributorRequestMessage request =
                    new DeleteDistributorRequestMessage();
                request.DistributorId = distributorId;

                webService.DeleteDistributor(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        #endregion

        #region Parental Guidance

        public ParentalGuidance[] GetParentalGuidances()
        {
            try
            {
                return webService.GetAllParentalGuidances();
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void CreateParentalGuidance(ParentalGuidance parentalGuidance)
        {
            if (parentalGuidance == null)
                throw new ArgumentNullException("parentalGuidance");

            try
            {
                CreateParentalGuidanceRequestMessage request =
                    new CreateParentalGuidanceRequestMessage();
                request.ParentalGuidance = parentalGuidance;

                webService.CreateParentalGuidance(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void UpdateParentalGuidance(ParentalGuidance parentalGuidance)
        {
            if (parentalGuidance == null)
                throw new ArgumentNullException("parentalGuidance");

            try
            {
                UpdateParentalGuidanceRequestMessage request =
                    new UpdateParentalGuidanceRequestMessage();
                request.ParentalGuidance = parentalGuidance;

                webService.UpdateParentalGuidance(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void DeleteParentalGuidance(Guid parentalGuidanceId)
        {
            try
            {
                DeleteParentalGuidanceRequestMessage request =
                    new DeleteParentalGuidanceRequestMessage();
                request.ParentalGuidanceId = parentalGuidanceId;

                webService.DeleteParentalGuidance(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        #endregion

        #region Person

        public Person[] GetPersons()
        {
            try
            {
                return webService.GetAllPersons();
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void CreatePerson(Person person)
        {
            if (person == null)
                throw new ArgumentNullException("person");

            try
            {
                CreatePersonRequestMessage request =
                    new CreatePersonRequestMessage();
                request.Person = person;

                webService.CreatePerson(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void UpdatePerson(Person person)
        {
            if (person == null)
                throw new ArgumentNullException("person");

            try
            {
                UpdatePersonRequestMessage request =
                    new UpdatePersonRequestMessage();
                request.Person = person;

                webService.UpdatePerson(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        public void DeletePerson(Guid personId)
        {
            try
            {
                DeletePersonRequestMessage request =
                    new DeletePersonRequestMessage();
                request.PersonId = personId;

                webService.DeletePerson(request);
            }
            catch (Exception ex)
            {
                Exception wrappedException = WrapException(ex);

                if (wrappedException != null)
                    throw wrappedException;

                throw;
            }
        }

        #endregion
    }
}