using System;
using System.Collections.Generic;
using System.Text;

#if !PocketPC
using System.Runtime.Serialization;
#endif

using AdventureWorks.Cinema.ServiceAccessLayer;
using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.SharedBusinessLogic
{

    /// <summary>
    /// Hauptklasse der gemeinsamen Geschftslogik.
    /// </summary>
    public partial class CinemaBusiness
    {

        // Deklariere ein ServiceAccess-Objekt.
        private ServiceAccess serviceAccess;

        // Deklariere Geschftsobjekte.
        private CountryBusiness countries;
        private CustomerBusiness customers;
        private DistributorBusiness distributors;
        private EmployeeBusiness employees;
        private GenreBusiness genres;
        private MovieBusiness movies;
        private NewsBusiness news;
        private ParentalGuidanceBusiness parentalGuidances;
        private PerformanceBusiness performances;
        private PersonBusiness persons;
        private RoomBusiness rooms;
        private TicketBusiness tickets;

        /// <summary>
        /// Initializiere eine neue Instanz der Klasse 
        /// AdventureWorks.Cinema.SharedBusinessLogic.CinemaBusiness.
        /// </summary>
        public CinemaBusiness()
        {
            // Erzeuge Instanz der ServiceAccess-Klasse.
            this.serviceAccess = new ServiceAccess();

            // Erzeuge Instanzen der Geschftsklassen.
            this.countries = new CountryBusiness(this);
            this.customers = new CustomerBusiness(this);
            this.distributors = new DistributorBusiness(this);
            this.employees = new EmployeeBusiness(this);
            this.genres = new GenreBusiness(this);
            this.movies = new MovieBusiness(this);
            this.news = new NewsBusiness(this);
            this.parentalGuidances = new ParentalGuidanceBusiness(this);
            this.performances = new PerformanceBusiness(this);
            this.persons = new PersonBusiness(this);
            this.rooms = new RoomBusiness(this);
            this.tickets = new TicketBusiness(this);
        }

        /// <summary>
        /// Initializiere eine Instanz der Klasse 
        /// AdventureWorks.Cinema.SharedBusinessLogic.CinemaBusiness
        /// mit der gegebenen Sitzungs-ID.
        /// </summary>
        /// <param name="sessionId">ID einer Sitzung.</param>
        /// <exception cref="ArgumentNullException">
        /// Die Sitzungs-ID ist NULL oder leer. </exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public CinemaBusiness(string sessionId): this()
        {

            // Prfe den Parameter sessionId.
            if (String.IsNullOrEmpty(sessionId))
                throw new ArgumentNullException("sessionId");

            // Erzeuge Instanz der Klasse ServiceAccess mit der
            // gegebenen Session-ID.
            this.serviceAccess = new ServiceAccess(sessionId);

            // Abrufen des angemeldeten Benutzers dieser Sitzung.
            User currentUser = serviceAccess.GetCurrentUser();

            // Legt den angemeldeten Benutzer als Kunden fest.
            this.customers = new CustomerBusiness(this, currentUser);
        }

        /// <summary>
        /// Ruft den Demo-Modus ab oder legt diesen fest.
        /// </summary>
        /// <remarks>
        /// Diese Eigenschaft verwendet direkt die Eigenschaft IsSampleMode der
        /// aktuellen ServiceAccess-Instanz. Daher braucht der Client keine
        /// Referenz zur ServiceAccess-Instanz.
        /// </remarks>
        public bool IsSampleMode
        {
            get
            {
                return ServiceAccess.IsSampleMode;
            }
            set
            {
                ServiceAccess.IsSampleMode = value;
            }
        }

        #region Properties

        /// <summary>
        /// Ruft den URL zum Cinema-Web-Service ab oder legt diesen fest.
        /// </summary>
        /// <remarks>
        /// Eine nderung dieser Eigenschaft hat keinen Einfluss auf 
        /// eine aktuelle Verbindung.
        /// </remarks>
        public Uri WebServiceAddress
        {
            get
            {
                return ServiceAccess.WebServiceUrl;
            }
            set
            {
                ServiceAccess.WebServiceUrl = value;
            }
        }

        /// <summary>
        /// Ruft die Sitzungs-ID der aktuellen ServiceAccess-Instanz ab.
        /// </summary>
        /// <returns>Wenn die aktuelle ServiceAccess-Instanz an keine Sitzung
        /// gebunden ist, d.h. weder <see cref="CustomerBusiness.LogOn"/> noch
        /// <see cref="EmployeeBusiness.LogOn"/> aufgerufen wurden oder die 
        /// Sitzung durch den Aufruf der entsprechenden LogOff-Methoden 
        /// geschlossen wurde, ist der Rckgabewert <see cref="String.Empty"/>.
        /// </returns>
        public string SessionId
        {
            get
            {
                return ServiceAccess.SessionId;
            }
        }

        /// <summary>
        /// Ruft die ServiceAccess-Instanz ab.
        /// </summary>
        internal ServiceAccess ServiceAccess
        {
            get
            {
                return serviceAccess;
            }
        }

        /// <summary>
        /// Ruft die CountryBusiness-Instanz ab, um Lnder zu verwalten.
        /// </summary>
        public CountryBusiness Countries
        {
            get
            {
                return countries;
            }
        }

        /// <summary>
        /// Ruft die CustomerBusiness-Instanz ab, um Kunden zu verwalten.
        /// </summary>
        public CustomerBusiness Customers
        {
            get
            {
                return customers;
            }
        }

        /// <summary>
        /// Ruft die DistributorBusiness-Instanz ab, um Verleiher zu verwalten.
        /// </summary>
        public DistributorBusiness Distributors
        {
            get
            {
                return distributors;
            }
        }

        /// <summary>
        /// Ruft die EmployeeBusiness-Instanz ab, um Mitarbeiter zu verwalten.
        /// </summary>
        public EmployeeBusiness Employees
        {
            get
            {
                return employees;
            }
        }

        /// <summary>
        /// Ruft die GenreBusiness-Instanz ab, um Genres zu verwalten.
        /// </summary>
        public GenreBusiness Genres
        {
            get
            {
                return genres;
            }
        }

        /// <summary>
        /// Ruft die MovieBusiness-Instanz ab, um Filme zu verwalten.
        /// </summary>
        public MovieBusiness Movies
        {
            get
            {
                return movies;
            }
        }

        /// <summary>
        /// Ruft die NewsBusiness-Instanz ab, um Nachrichten und Kategorien 
        /// zu verwalten.
        /// </summary>
        public NewsBusiness News
        {
            get
            {
                return news;
            }
        }

        /// <summary>
        /// Ruft die ParentalGuidanceBusiness-Instanz ab, um Altersfreigaben zu 
        /// verwalten.
        /// </summary>
        public ParentalGuidanceBusiness ParentalGuidances
        {
            get
            {
                return parentalGuidances;
            }
        }

        /// <summary>
        /// Ruft die PerformanceBusiness-Instanz ab, um Vorstellungen zu 
        /// verwalten.
        /// </summary>
        public PerformanceBusiness Performances
        {
            get
            {
                return performances;
            }
        }

        /// <summary>
        /// Ruft die PersonBusiness-Instanz ab, um Personen zu verwalten.
        /// </summary>
        public PersonBusiness Persons
        {
            get
            {
                return persons;
            }
        }

        /// <summary>
        /// Ruft die RoomBusiness-Instanz ab, um Rume zu verwalten.
        /// </summary>
        public RoomBusiness Rooms
        {
            get
            {
                return rooms;
            }
        }

        /// <summary>
        /// Ruft die TicketBusiness-Instanz ab, um Tickets zu verwalten.
        /// </summary>
        public TicketBusiness Tickets
        {
            get
            {
                return tickets;
            }
        }

        #endregion

    }

    #region Exceptions

    /// <summary>
    /// Die Ausnahme, die ausgelst wird, wenn eine Methode aufgerufen wird, 
    /// aber kein Benutzer angemeldet ist.
    /// </summary>
#if !PocketPC
    [Serializable]
#endif
    public class NoUserLoggedOnException : Exception
    {
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse 
        /// AdventureWorks.Cinema.SharedBusinessLogic.NoUserLoggedOnException.
        /// </summary>
        public NoUserLoggedOnException()
            : base()
        {
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse 
        /// AdventureWorks.Cinema.SharedBusinessLogic.NoUserLoggedOnException 
        /// mit der angegebenen Fehlermeldung.
        /// </summary>
        /// <param name="message">Eine Beschreibung des Fehlers.</param>
        public NoUserLoggedOnException(string message)
            : base(message)
        {
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse 
        /// AdventureWorks.Cinema.SharedBusinessLogic.NoUserLoggedOnException 
        /// mit der angegebenen Fehlermeldung und einer InnerException, die die 
        /// Ausnahme ausgelst hat.
        /// </summary>
        /// <param name="message">Die Beschreibung der Ausnahme.</param>
        /// <param name="innerException">Die Ausnahme, die die aktuelle 
        /// Ausnahme ausgelst hat.</param>
        public NoUserLoggedOnException(string message, Exception innerException)
            : base(message, innerException)
        {
        }

#if !PocketPC
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse 
        /// AdventureWorks.Cinema.SharedBusinessLogic.NoUserLoggedOnException 
        /// mit serialisierten Daten.</summary>
        /// <param name="info"> Die 
        /// System.Runtime.Serialization.SerializationInfo mit den 
        /// serialisierten Objektdaten fr die ausgelste Ausnahme.</param>
        /// <param name="context">Der 
        /// System.Runtime.Serialization.StreamingContext mit 
        /// Kontextinformationen zu Quelle oder Ziel.</param>
        /// <exception cref="System.Runtime.Serialization.SerializationException">
        /// Der Klassenname ist NULL oder System.Exception.HResult ist Null (0).
        /// </exception>
        /// <exception cref="System.ArgumentNullException">
        /// Der Parameter info ist NULL.
        /// </exception>
        protected NoUserLoggedOnException(SerializationInfo info,
            StreamingContext context) : base(info, context)
        {
        }
#endif

    }

    /// <summary>
    /// Die Ausnahme, die ausgelst wird, wenn ein Kunde Tickets kaufen mchte, 
    /// aber die gewnschte Anzahl an Tickets nicht mehr vorhanden ist.
    /// </summary>
#if !PocketPC
    [Serializable]
#endif
    public class NotEnoughFreeSeatsException : Exception
    {
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse 
        /// AdventureWorks.Cinema.SharedBusinessLogic.NotEnoughFreeSeatsException.
        /// </summary>
        public NotEnoughFreeSeatsException()
            : base()
        {
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse 
        /// AdventureWorks.Cinema.SharedBusinessLogic.NotEnoughFreeSeatsException 
        /// mit der angegebenen Fehlermeldung.
        /// </summary>
        /// <param name="message">The message that describes the error.</param>
        public NotEnoughFreeSeatsException(string message)
            : base(message)
        {
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse 
        /// AdventureWorks.Cinema.SharedBusinessLogic.NotEnoughFreeSeatsException 
        /// mit der angegebenen Fehlermeldung und einer InnerException, die die 
        /// Ausnahme ausgelst hat.
        /// </summary>
        /// <param name="message">Die Beschreibung der Ausnahme.</param>
        /// <param name="innerException">Die Ausnahme, die die aktuelle 
        /// Ausnahme ausgelst hat.</param>
        public NotEnoughFreeSeatsException(string message, Exception innerException)
            : base(message, innerException)
        {
        }

#if !PocketPC
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse 
        /// AdventureWorks.Cinema.SharedBusinessLogic.NotEnoughFreeSeatsException 
        /// mit serialisierten Daten.</summary>
        /// <param name="info"> Die 
        /// System.Runtime.Serialization.SerializationInfo mit den 
        /// serialisierten Objektdaten fr die ausgelste Ausnahme.</param>
        /// <param name="context">Der 
        /// System.Runtime.Serialization.StreamingContext mit 
        /// Kontextinformationen zu Quelle oder Ziel.</param>
        /// <exception cref="System.Runtime.Serialization.SerializationException">
        /// Der Klassenname ist NULL oder System.Exception.HResult ist Null (0).
        /// </exception>
        /// <exception cref="System.ArgumentNullException">
        /// Der Parameter info ist NULL.
        /// </exception>
        protected NotEnoughFreeSeatsException(SerializationInfo info,
            StreamingContext context) : base(info, context)
        {
        }
#endif

    }

    /// <summary>
    /// Thrown when an end date is before a start date.
    /// </summary>
#if !PocketPC
    [Serializable]
#endif
    public class EndBeforeStartException : Exception
    {
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse 
        /// AdventureWorks.Cinema.SharedBusinessLogic.EndBeforeStartException.
        /// </summary>
        public EndBeforeStartException()
            : base()
        {
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse 
        /// AdventureWorks.Cinema.SharedBusinessLogic.EndBeforeStartException 
        /// mit der angegebenen Fehlermeldung.
        /// </summary>
        /// <param name="message">The message that describes the error.</param>
        public EndBeforeStartException(string message)
            : base(message)
        {
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse 
        /// AdventureWorks.Cinema.SharedBusinessLogic.EndBeforeStartException 
        /// mit der angegebenen Fehlermeldung und einer InnerException, die die 
        /// Ausnahme ausgelst hat.
        /// </summary>
        /// <param name="message">Die Beschreibung der Ausnahme.</param>
        /// <param name="innerException">Die Ausnahme, die die aktuelle 
        /// Ausnahme ausgelst hat.</param>
        public EndBeforeStartException(string message, Exception innerException)
            : base(message, innerException)
        {
        }

#if !PocketPC
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse 
        /// AdventureWorks.Cinema.SharedBusinessLogic.EndBeforeStartException 
        /// mit serialisierten Daten.</summary>
        /// <param name="info"> Die 
        /// System.Runtime.Serialization.SerializationInfo mit den 
        /// serialisierten Objektdaten fr die ausgelste Ausnahme.</param>
        /// <param name="context">Der 
        /// System.Runtime.Serialization.StreamingContext mit 
        /// Kontextinformationen zu Quelle oder Ziel.</param>
        /// <exception cref="System.Runtime.Serialization.SerializationException">
        /// Der Klassenname ist NULL oder System.Exception.HResult ist Null (0).
        /// </exception>
        /// <exception cref="System.ArgumentNullException">
        /// Der Parameter info ist NULL.
        /// </exception>
        protected EndBeforeStartException(SerializationInfo info,
            StreamingContext context) : base(info, context)
        {
        }
#endif

    }

    #endregion

}
