using System;
using System.Collections.Generic;
using System.Text;

using AdventureWorks.Cinema.ServiceAccessLayer;
using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.SharedBusinessLogic
{
    /// <summary>
    /// Enthlt Methoden zur Verwaltung von Lndern in AdventureWorks Cinema.
    /// </summary>
    public class CountryBusiness
    {

        // Deklariere Variable fr CinemaBusiness.
        private CinemaBusiness cinemaBusiness;

        /// <summary>
        /// Initialisiert eine neue CountryBusiness-Instanz mit der 
        /// angegebenen CinemaBuiness-Instanz.
        /// </summary>
        /// <param name="cinemaBusiness">Eine CinemaBusiness-Instanz, die von 
        /// dieser CountryBusiness-Instanz verwendet werden soll.</param>
        internal CountryBusiness(CinemaBusiness cinemaBusiness)
        {

            // Speichere Referenz zum CinemaBusiness ab.
            this.cinemaBusiness = cinemaBusiness;

        }

        /// <summary>
        /// Ruft alle Lnder vom Server ab.
        /// </summary>
        /// <returns>Liste von Lndern.</returns>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<Country> Countries
        {
            get
            {
                return new List<Country>(
                    cinemaBusiness.ServiceAccess.GetCountries());
            }
        }

        /// <summary>
        /// Erstellt ein neues Land mit dem angegebenen Namen auf dem Server.
        /// </summary>
        /// <param name="name">Der Name des Landes.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// name ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// name ist ein leerer String.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        /// <remarks>Diese Methode folgt dem Factory-Muster.</remarks>
        public Country Create(string name)
        {

            // Erzeuge lokal eine Country-Instanz.
            Country country = new Country();
            country.Id = Guid.NewGuid();
            country.Name = name;

            // Erzeuge ein Land auf dem Server. 
            Create(country);

            return country;

        }

        /// <summary>
        /// Fgt eine existierende Country-Instanz dem Server hinzu.
        /// </summary>
        /// <param name="country">Eine Country-Instanz, die auf dem Server 
        /// hinzugefgt werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// country ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Der Name des Landes ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Create(Country country)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (country == null)
                throw new ArgumentNullException("country");
            if (String.IsNullOrEmpty(country.Name))
                throw new ArgumentException(Resource.NullOrEmptyCountryName, 
                    "country");

            // Erzeuge Land auf dem Server.
            cinemaBusiness.ServiceAccess.CreateCountry(country);
        }

        /// <summary>
        /// ndert ein Land auf dem Server.
        /// </summary>
        /// <param name="country">Eine auf dem Server existierende 
        /// Country-Instanz, die gendert werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// country ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Der Name des Landes ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Update(Country country)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (country == null)
                throw new ArgumentNullException("country");
            if (String.IsNullOrEmpty(country.Name))
                throw new ArgumentException(Resource.NullOrEmptyCountryName,
                    "country");

            // ndere das Land.
            cinemaBusiness.ServiceAccess.UpdateCountry(country);
        }

        /// <summary>
        /// Lscht ein Land vom Server.
        /// </summary>
        /// <param name="country">Ein existierendes Land, das gelscht werden 
        /// soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// country ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Delete(Country country)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (country == null)
                throw new ArgumentNullException("country");

            // Lsche das Land vom Server.
            cinemaBusiness.ServiceAccess.DeleteCountry(country);
        }
    }

}