using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

using AdventureWorks.Cinema.ServiceAccessLayer;
using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.SharedBusinessLogic
{
    /// <summary>
    /// Enthlt Methoden zur Verwaltung von Kunden in AdventureWorks Cinema.
    /// </summary>
    public class CustomerBusiness
    {

        // Deklariere Variable fr CinemaBusiness.
        private CinemaBusiness cinemaBusiness;

        /// <summary>
        /// Initialisiert eine neue CustomerBusiness-Instanz mit der 
        /// angegebenen CinemaBuiness-Instanz.
        /// </summary>
        /// <param name="cinemaBusiness">Eine CinemaBusiness-Instanz, die von 
        /// dieser CountryBusiness-Instanz verwendet werden soll.</param>
        internal CustomerBusiness(CinemaBusiness cinemaBusiness)
        {

            // Speichere Referenz zum CinemaBusiness ab.
            this.cinemaBusiness = cinemaBusiness;

        }

        /// <summary>
        /// Initialisiert eine neue CustomerBusiness-Instanz mit der 
        /// angegebenen CinemaBuiness-Instanz und einem angemeldeten Kunden.
        /// </summary>
        /// <param name="cinemaBusiness">Eine CinemaBusiness-Instanz, die von 
        /// dieser CountryBusiness-Instanz verwendet werden soll.</param>
        /// <param name="loggedOnCustomer">Eine User-Instanz eines aktuell 
        /// angemeldeten Benutzers.</param>
        internal CustomerBusiness(CinemaBusiness cinemaBusiness, 
            User loggedOnCustomer): this(cinemaBusiness)
        {
            // Speichere den aktuell angemeldeten Kunden.
            this.customer = loggedOnCustomer;
        }

        // Deklariere Variable fr einen Kunden.
        private User customer;

        /// <summary>
        /// Gibt den aktuell angemeldeten Kunden zurck.
        /// </summary>
        /// <returns>Der aktuell angemeldete Kunde.</returns>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Kunde angemeldet.</exception>
        public User CurrentCustomer
        {
            get
            {
                // Prfe, ob ein Kunde angemeldet ist.
                if (!IsLoggedOn)
                    throw new NoUserLoggedOnException();

                return this.customer;
            }
        }

        /// <summary>
        /// Ruft alle Kunden vom Server ab, die einen Newsletter abonniert 
        /// haben.</summary>
        /// <returns>Liste von Kunden, die einen Newsletter abonniert haben.
        /// </returns>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<UserInfo> UsersForNewsletter
        {
            get
            {

                // Prfe, ob ein Mitarbeiter angemeldet ist.
                if (!cinemaBusiness.Employees.IsLoggedOn)
                    throw new NoUserLoggedOnException();

                return new List<UserInfo>(
                    cinemaBusiness.ServiceAccess.GetUsersForNewsletter());
            }
        }

        /// <summary>
        /// Ruft eine User-Instanz fr die angegebene UserInfo-Instanz ab.
        /// </summary>
        /// <param name="userInfo">Eine UserInfo-Instanz, fr die eine 
        /// User-Instanz bentigt wird.</param>
        /// <returns>Eine User-Instanz, die der UserInfo-Instant entspricht.
        /// </returns>
        public User Get(UserInfo userInfo)
        {

            // Prfe Parameter.
            if (userInfo == null)
                throw new ArgumentNullException("userInfo");

            // Rufe Benutzer ab.
            return cinemaBusiness.ServiceAccess.GetUser(userInfo);
        }

        /// <summary>
        /// Gibt an, ob aktuell ein Kunde angemeldet ist.
        /// </summary>
        public bool IsLoggedOn
        {
            get
            {
                return this.customer != null;
            }
        }

        /// <summary>
        /// Meldet einen Kunden an.
        /// </summary>
        /// <param name="userName">Der Benutzername des Kunden.</param>
        /// <param name="password">Das Kennwort des Kunden.</param>
        /// <exception cref="ArgumentException">
        /// userName ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void LogOn(string userName, string password)
        {

            // Melde den aktuellen Mitarbeiter ab.
            cinemaBusiness.Employees.LogOff();

            // Melde den aktuellen Kunden ab.
            LogOff();

            // Prfe Parameter.
            if (String.IsNullOrEmpty(userName))
                throw new ArgumentException(Resource.NullOrEmptyUserName,
                    "userName");
            if (String.IsNullOrEmpty(password))
                throw new ArgumentException(Resource.NullOrEmptyPassword, 
                    "password");

            // Melde den Kunden an.
            this.customer = cinemaBusiness.ServiceAccess
                .LogOn(userName, password);
        }

        /// <summary>
        /// Melde den aktuellen Kunden ab.
        /// </summary>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void LogOff()
        {

            // Melde den aktuellen Kunden ab, sofern einer angemeldet ist.
            if (IsLoggedOn)
            {

                cinemaBusiness.ServiceAccess.LogOff();

                // Lsche Referenz zum Kunden.
                this.customer = null;

            }

        }

        /// <summary>
        /// Erstellt einen neuen Kunden auf dem Server.
        /// </summary>
        /// <param name="userName">Benutzername des Kunden.</param>
        /// <param name="password">Kennwort des Kunden.</param>
        /// <exception cref="ArgumentNullException">
        /// userName oder password ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// userName ist leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public User Create(string userName, string password)
        {

            // Erstelle lokal einen Kunden und speichere seine Daten.
            Role role = new Role();
            role.Name = ServiceAccessLayer.ServiceAccess.CustomerRoleName;

            User customer = new User();
            customer.Id = Guid.NewGuid();
            customer.UserAccount = userName;
            customer.Roles = new Role[] { role };

            // Create the customer on the server.
            Create(customer, password);

            return customer;

        }

        /// <summary>
        /// Erstellt einen neuen Kunden mit den angegebenen Daten auf dem 
        /// Server.
        /// </summary>
        /// <param name="firstNames">Die Vornamen des neuen Kunden.</param>
        /// <param name="lastName">Der Nachname des neuen Kunden.</param>
        /// <param name="street">Die Strae des neuen Kunden.</param>
        /// <param name="zipCode">Die Postleitzahl des neuen Kunden.</param>
        /// <param name="city">Die Stadt des neuen Kunden.</param>
        /// <param name="email">Die E-Mail-Adresse des neuen Kunden.</param>
        /// <param name="newsletter">Gibt an, ob der neue Kunde Newsletter 
        /// erhalten mchte.</param>
        /// <param name="userName">Der Benutzername des neuen Kunden.</param>
        /// <param name="password">Das Kennwort des neuen Kunden.</param>
        /// <remarks>Diese Methode folgt dem Factory-Muster.</remarks>
        /// <exception cref="ArgumentNullException">
        /// Benutzername oder Kennwort ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Benutzername ist leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public User Create(string firstNames, string lastName,
            string street, string zipCode, string city, string email,
            bool newsletter, string userName, string password)
        {

            // Erzeuge lokal einen neuen Kunden und speichere seine Daten.
            Role role = new Role();
            role.Name = ServiceAccessLayer.ServiceAccess.CustomerRoleName;

            User customer = new User();
            customer.Id = Guid.NewGuid();
            customer.UserAccount = userName;
            customer.FirstNames = firstNames;
            customer.LastName = lastName;
            customer.Street = street;
            customer.ZipCode = zipCode;
            customer.City = city;
            customer.Email = email;
            customer.Newsletter = newsletter;
            customer.Roles = new Role[] { role };

            // Erzeuge den neuen Kunden auf dem Server.
            Create(customer, password);

            return customer;

        }

        /// <summary>
        /// Fgt eine lokal existierende Customer-Instanz dem Server hinzu.
        /// </summary>
        /// <param name="customer">Eine Customer-Instanz, die dem Server hinzu 
        /// gefgt werden soll.</param>
        /// <param name="password">Das Kennwort des Kunden.</param>
        /// <exception cref="ArgumentNullException">
        /// customer oder password ist NULL.
        /// - oder -
        /// password ist leer.</exception>
        /// <exception cref="ArgumentException">
        /// customer.UserAccount ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Create(User customer, string password)
        {

            // Prfe Parameter.
            if (customer == null)
                throw new ArgumentNullException("customer");
            if (String.IsNullOrEmpty(customer.UserAccount))
                throw new ArgumentException(
                    Resource.NullOrEmptyCustomerUserAccount, "customer");
            if (String.IsNullOrEmpty(password))
                throw new ArgumentNullException("password");

            // Erzeuge einen Kunden auf dem Server.
            cinemaBusiness.ServiceAccess.CreateUser(customer, password);

        }

        /// <summary>
        /// ndere einen vorhandenen Kunden auf dem Server.
        /// </summary>
        /// <param name="customer">Eine Customer-Instanz, die verndert werden 
        /// soll.</param>
        /// <remarks>
        /// Der Kunde, dessen Daten verndert werden, muss aktuell nicht 
        /// angemeldet sein.
        /// </remarks>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter oder Kunde angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// customer ist NULL.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// customer.UserAccount ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Update(User customer)
        {

            // Prfe, ob ein Kunde oder Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn &&
                !IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (customer == null)
                throw new ArgumentNullException("customer");
            if (String.IsNullOrEmpty(customer.UserAccount))
                throw new ArgumentException(
                    Resource.NullOrEmptyCustomerUserAccount, "customer");

            // ndere die Kundendaten auf dem Server.
            cinemaBusiness.ServiceAccess.UpdateUser(customer);
        }

        /// <summary>
        /// Lsche den aktuell angemeldeten Kunden vom Server.
        /// </summary>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter oder Kunde angemeldet.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Delete()
        {

            // Prfe, ob ein Kunde oder Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn &&
                !IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Sichere Customer-Instanz ber die Abmeldung.
            User c = this.customer;

            // Melde den aktuellen Kunden ab.
            LogOff();

            // Lsche den Kunden vom Server.
            cinemaBusiness.ServiceAccess.DeleteUser(c);
        }

        /// <summary>
        /// ndert das aktuelle Kennwort des Kunden.
        /// </summary>
        /// <param name="currentPassword">Das aktuelle Kennwort des Kunden.
        /// </param>
        /// <param name="newPassword">Das neue Kennwort des Kunden.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter oder Kunde angemeldet.</exception>
        /// <exception cref="ArgumentException">
        /// currentPassword oder newPassword ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void ChangePassword(string currentPassword, string newPassword)
        {

            // Prfe, ob ein Kunde oder Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn &&
                !IsLoggedOn)
                throw new NoUserLoggedOnException(
                    Resource.ChangePasswordNoUserLoggedOnException);

            // Prfe Parameter.
            if (String.IsNullOrEmpty(currentPassword))
                throw new ArgumentException(
                    Resource.NullOrEmptyCurrentPassword, "currentPassword");
            if (String.IsNullOrEmpty(newPassword))
                throw new ArgumentException(
                    Resource.NullOrEmptyNewPassword, "newPassword");

            // ndere das Kennwort.
            cinemaBusiness.ServiceAccess.
                ChangePassword(currentPassword, newPassword);
        }

        /// <summary>
        /// Setzt das Kennwort eines bestimmten Benutzers zurck. Diesem 
        /// Benutzer wird sein neues Kennwort an seine E-Mail-Adresse geschickt.
        /// </summary>
        /// <param name="userName">Der Benutzername des Benutzers, dessen 
        /// Kennwort zurckgesetzt werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void ResetPassword(string userName)
        {
            cinemaBusiness.ServiceAccess.ResetPassword(userName);
        }

        /// <summary>
        /// Ruft alle Filme ab, die den Interessen des aktuell angemeldeten 
        /// Kunden entsprechen.
        /// </summary>
        /// <returns>Liste mit MovieInfo-Instanzen.</returns>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Kunde angemeldet.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<MovieInfo> Movies
        {
            get
            {
                // Prfe, ob ein Kunde angemeldet ist.
                if (!IsLoggedOn)
                    throw new NoUserLoggedOnException();

                // Rufe Filme ab.
                return cinemaBusiness.Movies.GetByUserInterests(customer);
            }
        }

        /// <summary>
        /// Rufe die Tickets des aktuell angemeldeten Kunden ab.
        /// </summary>
        /// <returns>Liste mit Tickets.</returns>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Kunde angemeldet.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<Ticket> Tickets
        {
            get
            {

                // Prfe, ob ein Kunde angemeldet ist.
                if (!IsLoggedOn)
                    throw new NoUserLoggedOnException();

                // Rufe Tickets ab.
                return new List<Ticket>(
                    cinemaBusiness.ServiceAccess.GetUndispatchedTickets(
                    customer));
            }
        }

    }

}
