using System;
using System.Collections.Generic;
using System.Text;

using AdventureWorks.Cinema.ServiceAccessLayer;
using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.SharedBusinessLogic
{
    /// <summary>
    /// Enthlt Methoden zur Verwaltung von Verleihern in AdventureWorks Cinema.
    /// </summary>
    public class DistributorBusiness
    {

        // Deklariere Variable fr CinemaBusiness.
        private CinemaBusiness cinemaBusiness;

        /// <summary>
        /// Initialisiert eine neue DistributorBusiness-Instanz mit der 
        /// angegebenen CinemaBuiness-Instanz.
        /// </summary>
        /// <param name="cinemaBusiness">Eine CinemaBusiness-Instanz, die von 
        /// dieser CountryBusiness-Instanz verwendet werden soll.</param>
        internal DistributorBusiness(CinemaBusiness cinemaBusiness)
        {

            // Speichere Referenz zum CinemaBusiness ab.
            this.cinemaBusiness = cinemaBusiness;

        }

        /// <summary>
        /// Ruft alle Verleiher vom Server ab.
        /// </summary>
        /// <returns>Liste mit Verleihern.</returns>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<Distributor> Distributors
        {
            get
            {
                return new List<Distributor>(
                    cinemaBusiness.ServiceAccess.GetDistributors());
            }
        }

        /// <summary>
        /// Erstellt einen neuen Verleiher auf dem Server.
        /// </summary>
        /// <param name="name">Der Name des Verleihers.</param>
        /// <remarks>Diese Methode folgt dem Factory-Muster.</remarks>
        /// <exception cref="ArgumentException">
        /// name ist NULL oder leer.</exception>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public Distributor Create(string name)
        {

            // Erzeuge Verleiher lokal.
            Distributor distributor = new Distributor();
            distributor.Id = Guid.NewGuid();
            distributor.Name = name;

            // Erzeuge Verleiher auf dem Server.
            Create(distributor);

            return distributor;

        }

        /// <summary>
        /// Fgt einen lokal vorhandenen Verleiher dem Server hinzu.
        /// </summary>
        /// <param name="distributor">Distributor-Instanz, die dem Server 
        /// hinzugefgt werden soll.</param>
        /// <exception cref="ArgumentNullException">
        /// distributor ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Die Name-Eigenschaft von distributor ist NULL oder leer.</exception>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Create(Distributor distributor)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (distributor == null)
                throw new ArgumentNullException("distributor");
            if (String.IsNullOrEmpty(distributor.Name))
                throw new ArgumentException(
                    Resource.NullOrEmptyDistributorName, "distributor");

            // Erstelle Verleiher auf dem Server.
            cinemaBusiness.ServiceAccess.CreateDistributor(distributor);
        }

        /// <summary>
        /// ndert einen Verleiher auf dem Server.
        /// </summary>
        /// <param name="distributor">Eine Distributor-Instanz, die auf dem 
        /// Server verndert werden soll.</param>
        /// <exception cref="ArgumentNullException">
        /// distributor ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Die Name-Eigenschaft von distributor ist NULL oder leer.</exception>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Update(Distributor distributor)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (distributor == null)
                throw new ArgumentNullException("distributor");
            if (String.IsNullOrEmpty(distributor.Name))
                throw new ArgumentException(
                    Resource.NullOrEmptyDistributorName, "distributor");

            // ndere Verleiher auf dem Server.
            cinemaBusiness.ServiceAccess.UpdateDistributor(distributor);
        }

        /// <summary>
        /// Lscht einen Verleiher vom Server.
        /// </summary>
        /// <param name="distributor">Distributor-Instanz, die vom Server 
        /// gelscht werden soll.</param>
        /// <exception cref="ArgumentNullException">
        /// distributor ist NULL.</exception>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Delete(Distributor distributor)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (distributor == null)
                throw new ArgumentNullException("distributor");

            // Lsche Verleiher vom Server.
            cinemaBusiness.ServiceAccess.DeleteDistributor(distributor);
        }

    }

}
