using System;
using System.Collections.Generic;
using System.Security;
using System.Text;

using AdventureWorks.Cinema.ServiceAccessLayer;
using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.SharedBusinessLogic
{
    /// <summary>
    /// Enthlt Methoden zur Verwaltung von Mitarbeitern in 
    /// AdventureWorks Cinema.
    /// </summary>
    public class EmployeeBusiness
    {

        // Deklariere Variable fr CinemaBusiness.
        private CinemaBusiness cinemaBusiness;

        /// <summary>
        /// Initialisiert eine neue EmployeeBusiness-Instanz mit der 
        /// angegebenen CinemaBuiness-Instanz.
        /// </summary>
        /// <param name="cinemaBusiness">Eine CinemaBusiness-Instanz, die von 
        /// dieser CountryBusiness-Instanz verwendet werden soll.</param>
        internal EmployeeBusiness(CinemaBusiness cinemaBusiness)
        {

            // Speichere Referenz zum CinemaBusiness ab.
            this.cinemaBusiness = cinemaBusiness;

        }

        // Deklariere Variable fr einen Mitarbeiter.
        private User employee;

        /// <summary>
        /// Prft, ob aktuell ein Mitarbeiter angemeldet ist.
        /// </summary>
        public bool IsLoggedOn
        {
            get
            {
                return this.employee != null;
            }
        }

        /// <summary>
        /// Gibt den aktuell angemeldeten Mitarbeiter zurck.
        /// </summary>
        /// <returns>Der aktuell angemeldete Mitarbeiter.</returns>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        public User CurrentEmployee
        {
            get
            {
                // Prfe, ob ein Mitarbeiter angemeldet ist.
                if (!IsLoggedOn)
                    throw new NoUserLoggedOnException();

                return this.employee;
            }
        }

        /// <summary>
        /// Meldet einen Mitarbeiter an.
        /// </summary>
        /// <param name="userName">Der Benutzername des Mitarbeiters.</param>
        /// <param name="password">Das Kennwort des Mitarbeiters.</param>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        /// <exception cref="SecurityException">Der Benutzer gehrt nicht zur 
        /// Rolle der Mitarbeiter.</exception>
        public void LogOn(string userName, string password)
        {
            // Melde den aktuellen Kunden ab.
            cinemaBusiness.Customers.LogOff();

            // Melde den aktuellen Mitarbeiter ab.
            LogOff();

            // Melde den Mitarbeiter an.
            this.employee = cinemaBusiness.ServiceAccess.LogOn(userName, 
                password);

            // berspringe die Rollenprfung im Demo-Modus.
            if (cinemaBusiness.IsSampleMode)
                return;

            foreach (Role role in employee.Roles)
            {
                if (String.Compare(role.Name, ServiceAccess.EmployeeRoleName, 
                    StringComparison.InvariantCultureIgnoreCase) == 0)
                {
                    // Current user is a member of employee role.
                    return;
                }
            }

            // Wenn dieser Punkte erreicht wurde, hat der aktuell angemeldete 
            // Benutzer keine Mitarbeiterrechte. Es wird eine 
            // Sicherheitsausnahme ausgelst.
            //
            // NOTE: Aus dem Blickwinkel der Sicherheit ist dies nicht unbedingt 
            //       notwendig. Der Web Service selbst stellt sicher, dass
            //       bestimmte Methoden Mitarbeiterrechte voraussetzen.
            //       Diese Sicherheitsausnahme ermglicht diesen Fehler bereits 
            //       zur Anmeldung festzustellen und nicht erst, wenn eine 
            //       solche Methode aufgerufen wird.

            // Zu diesem Zeitpunkt ist der Kunde bereits angemeldet.
            // Mache dies nun rckgngig.
            LogOff();

            throw new SecurityException(Resource.EmployeePrivilegesRequired);
        }

        /// <summary>
        /// Meldet den aktuellen Mitarbeiter ab.
        /// </summary>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void LogOff()
        {

            // Melde den Mitarbeiter ab, sofern einer angemeldet ist.
            if (IsLoggedOn)
            {

                cinemaBusiness.ServiceAccess.LogOff();

                // Lsche Referenz zur Employee-Instanz.
                this.employee = null;

            }

        }

    }

}
