using System;
using System.Collections.Generic;
using System.Text;

using AdventureWorks.Cinema.ServiceAccessLayer;
using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.SharedBusinessLogic
{
    /// <summary>
    /// Enthlt Methoden zur Verwaltung von Genres in AdventureWorks Cinema.
    /// </summary>
    public class GenreBusiness
    {

        // Deklariere Variable fr CinemaBusiness.
        private CinemaBusiness cinemaBusiness;

        /// <summary>
        /// Initialisiert eine neue GenreBusiness-Instanz mit der 
        /// angegebenen CinemaBuiness-Instanz.
        /// </summary>
        /// <param name="cinemaBusiness">Eine CinemaBusiness-Instanz, die von 
        /// dieser CountryBusiness-Instanz verwendet werden soll.</param>
        internal GenreBusiness(CinemaBusiness cinemaBusiness)
        {

            // Speichere Referenz zum CinemaBusiness ab.
            this.cinemaBusiness = cinemaBusiness;

        }

        /// <summary>
        /// Ruft alle Genres vom Server ab.
        /// </summary>
        /// <returns>Liste mit Genres.</returns>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<Genre> Genres
        {
            get
            {
                return new List<Genre>(
                    cinemaBusiness.ServiceAccess.GetGenres());
            }
        }

        /// <summary>
        /// Erstellt ein neues Genre auf dem Server.
        /// </summary>
        /// <param name="name">Der Name des neuen Genre.</param>
        public Genre Create(string name)
        {

            // Erzeuge lokal neue Genre-Instanz.
            Genre genre = new Genre();
            genre.Id = Guid.NewGuid();
            genre.Name = name;

            // Erzeuge neues Genre auf dem Server.
            Create(genre);

            return genre;

        }

        /// <summary>
        /// Fgt ein vorhandenes Genre zum Server hinzu.
        /// </summary>
        /// <param name="genre">Eine Genre-Instanz, die auf dem Server 
        /// hinzugefgt werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter eingeloggt.</exception>
        /// <exception cref="ArgumentNullException">
        /// genre ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Die Name-Eigenschaft von genre ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Create(Genre genre)
        {

            // Prfe, ob ein Mitarbeiter eingeloggt ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (genre == null)
                throw new ArgumentNullException("genre");
            if (String.IsNullOrEmpty(genre.Name))
                throw new ArgumentException(Resource.NullOrEmptyGenreName,
                    "genre");

            // Erzeuge neues Genre auf dem Server.
            cinemaBusiness.ServiceAccess.CreateGenre(genre);
        }

        /// <summary>
        /// ndert ein Genre auf dem Server.
        /// </summary>
        /// <param name="genre">Eine Genre-Instanz, die auf dem Server 
        /// gendert werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter eingeloggt.</exception>
        /// <exception cref="ArgumentNullException">
        /// genre ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Die Name-Eigenschaft von genre ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Update(Genre genre)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (genre == null)
                throw new ArgumentNullException("genre");
            if (String.IsNullOrEmpty(genre.Name))
                throw new ArgumentException(Resource.NullOrEmptyGenreName,
                    "genre");

            // ndere das Genre auf dem Server.
            cinemaBusiness.ServiceAccess.UpdateGenre(genre);
        }

        /// <summary>
        /// Lscht ein Genre vom Server.
        /// </summary>
        /// <param name="genre">Eine Genre-Instanz, die vom Server 
        /// gelscht werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter eingeloggt.</exception>
        /// <exception cref="ArgumentNullException">
        /// genre ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Delete(Genre genre)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (genre == null)
                throw new ArgumentNullException("genre");

            // Lsche das Genre vom Server.
            cinemaBusiness.ServiceAccess.DeleteGenre(genre);
        }

    }

}