using System;
using System.Collections.Generic;
using System.Text;

using AdventureWorks.Cinema.ServiceAccessLayer;
using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.SharedBusinessLogic
{
    /// <summary>
    /// Enthlt Methoden zur Verwaltung von Filmen in AdventureWorks Cinema.
    /// </summary>
    public class MovieBusiness
    {

        // Deklariere Variable fr CinemaBusiness.
        private CinemaBusiness cinemaBusiness;

        /// <summary>
        /// Initialisiert eine neue MovieBusiness-Instanz mit der 
        /// angegebenen CinemaBuiness-Instanz.
        /// </summary>
        /// <param name="cinemaBusiness">Eine CinemaBusiness-Instanz, die von 
        /// dieser CountryBusiness-Instanz verwendet werden soll.</param>
        internal MovieBusiness(CinemaBusiness cinemaBusiness)
        {

            // Speichere Referenz zum CinemaBusiness ab.
            this.cinemaBusiness = cinemaBusiness;

        }

        /// <summary>
        /// Ruft eine Movie-Instanz fr die angegebene MovieInfo-Instanz ab.
        /// </summary>
        /// <param name="movieInfo">Eine MovieInfo-Instanz, fr die eine 
        /// Movie-Instanz zurckgegeben werden soll.</param>
        /// <returns>Eine Movie-Instanz, die der MovieInfo-Instanz 
        /// entspricht.</returns>
        /// <exception cref="ArgumentNullException">
        /// movieInfo ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public Movie Get(MovieInfo movieInfo)
        {

            // Prfe Parameter.
            if (movieInfo == null)
                throw new ArgumentNullException("movieInfo");

            // Rufe Film ab.
            return cinemaBusiness.ServiceAccess.GetMovie(movieInfo);
        }

        /// <summary>
        /// Ruft alle Filme vom Server ab.
        /// </summary>
        /// <returns>Liste mit MovieInfo-Instanzen.</returns>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<MovieInfo> Movies
        {
            get
            {
                return new List<MovieInfo>(
                    cinemaBusiness.ServiceAccess.GetMovies());
            }
        }

        /// <summary>
        /// Ruft alle Filme mit dem angegebenen Titel oder Original-Titel vom 
        /// Server ab.
        /// </summary>
        /// <returns>Liste mit MovieInfo-Instanzen.</returns>
        /// <exception cref="ArgumentNullException">
        /// title ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<MovieInfo> GetByTitle(string title)
        {

            // Prfe Parameter.
            if (String.IsNullOrEmpty(title))
                throw new ArgumentNullException("title");

            // Rufe Filme vom Server ab.
            return new List<MovieInfo>(cinemaBusiness.ServiceAccess.
                GetMoviesByTitle(title));
        }

        /// <summary>
        /// Ruft alle Filme nach einem bestimmten Startdatum vom Server ab.
        /// </summary>
        /// <param name="start">Datum, ab dem die Filme abgerufen werden sollen.
        /// </param>
        /// <returns>Liste mit MovieInfo-Instanzen.</returns>
        /// <exception cref="ArgumentNullException">
        /// Das Startdatum ist Null (0).</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<MovieInfo> GetByDate(DateTime start)
        {

            // Prfe Parameter.
            if (start.Ticks == 0)
                throw new ArgumentNullException("start");

            return new List<MovieInfo>(
                cinemaBusiness.ServiceAccess.GetMovies(start));

        }

        /// <summary>
        /// Ruft alle Filme ab, fr sich der angegebene Benutzer interessiert.
        /// </summary>
        /// <param name="userInfo">Eine UserInfo-Instanz.</param>
        /// <returns>Liste mit MovieInfo-Instanzen.</returns>
        /// <exception cref="ArgumentNullException">
        /// userInfo ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<MovieInfo> GetByUserInterests(UserInfo userInfo)
        {
            // Prfe Parameter.
            if (userInfo == null)
                throw new ArgumentNullException("userInfo");

            // Rufe Filme ab, fr die sich der Benutzer interessiert.
            return new List<MovieInfo>(cinemaBusiness.ServiceAccess.
                GetMoviesByUserInterests(userInfo));
        }

        /// <summary>
        /// Ruft alle Filme ab, fr sich der angegebene Benutzer interessiert.
        /// </summary>
        /// <param name="user">Eine User-Instanz.</param>
        /// <returns>Liste mit MoveInfo-Instanzen.</returns>
        /// <exception cref="ArgumentNullException">
        /// user ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<MovieInfo> GetByUserInterests(User user)
        {
            // Prfe Parameter.
            if(user == null)
                throw new ArgumentNullException("user");

            // Rufe die Filme ab, fr die sich der Benutzer interessiert.
            return new List<MovieInfo>(cinemaBusiness.ServiceAccess.
                GetMoviesByUserInterests(user));
        }

        /// <summary>
        /// Speichert die Bewertung fr einen Film.
        /// </summary>
        /// <param name="movie">Eine MovieInstanz, fr die eine Bewertung 
        /// gespeichert werden soll.</param>
        /// <param name="rating">Eine Bewertung von 1 bis 10.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Kunde oder Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// movie ist NULL.</exception>
        /// <exception cref="ArgumentOutOfRangeException">
        /// Die Bewertung ist auerhalb des Bereichs von 1 bis 10.
        /// </exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Rate(Movie movie, int rating)
        {

            // Prfe, ob ein Kunde oder ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Customers.IsLoggedOn &&
                !cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (movie == null)
                throw new ArgumentNullException("movie");
            if (rating <=0 && rating >= 10)
                throw new ArgumentOutOfRangeException("rating");

            // Speichere die Bewertung auf dem Server.
            cinemaBusiness.ServiceAccess.RateMovie(movie, rating);
        }

        /// <summary>
        /// Erstellt einen neuen Film auf dem Server.
        /// </summary>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentException">
        /// title ist NULL oder leer.
        /// - oder - 
        /// Die Liste genres ist leer.</exception>
        /// <exception cref="ArgumentOutOfRangeException">
        /// Die Filmlnge ist negativ.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public Movie Create(string title, string originalTitle,
            string description, Country country, int year,
            List<Person> producers, List<Person> directors, List<Person> actors,
            Distributor distributor, List<Genre> genres, int length,
            DateTime startDate, ParentalGuidance parentalGuidance,
            byte[] picture)
        {

            // Erzeuge lokal eine Movie-Instanz und speichere die Daten.
            Movie movie = new Movie();
            movie.Id = Guid.NewGuid();
            movie.Title = title;
            movie.OriginalTitle = originalTitle;
            movie.Description = description;
            movie.Country = country;
            movie.Year = year;
            movie.Producers = producers.ToArray();
            movie.Directors = directors.ToArray();
            movie.Actors = actors.ToArray(); ;
            movie.Distributor = distributor;
            movie.Genres = genres.ToArray();
            movie.Length = length;
            movie.StartDate = startDate;
            movie.ParentalGuidance = parentalGuidance;
            movie.Picture = picture;

            // Erstelle den Film auf dem Server.
            Create(movie);

            return movie;

        }
        /// <summary>
        /// Creates a new movie on the server.
        /// </summary>
        /// <param name="movie">Movie to be created.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// movie ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Die Title-Eigenschaft von movie ist NULL oder leer.
        /// - oder - 
        /// Die Genres-Eigenschaft von movie ist leer.</exception>
        /// <exception cref="ArgumentOutOfRangeException">
        /// Die Filmlnge ist negativ.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Create(Movie movie)
        {

            // Prfe, ob ein Mitarbeiter eingeloggt ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (movie == null)
                throw new ArgumentNullException("movie");
            if (String.IsNullOrEmpty(movie.Title))
                throw new ArgumentException(
                    Resource.NullOrEmptyMovieTitle, "movie");
            if (movie.Length <= 0)
                throw new ArgumentOutOfRangeException("movie",
                    Resource.GreaterThanZeroMovieLength);
            if (movie.Genres == null)
                throw new ArgumentException(Resource.NullMovieGenres, "movie");

            // Erstelle einen neuen Film auf dem Server.
            cinemaBusiness.ServiceAccess.CreateMovie(movie);
        }

        /// <summary>
        /// ndert einen vorhandenen Film auf dem Server.
        /// </summary>
        /// <param name="movie">Eine Movie-Instanz, die verndert werden soll.
        /// </param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// movie ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Die Title-Eigenschaft von movie ist NULL oder leer.
        /// - oder - 
        /// Die Genres-Eigenschaft von movie ist leer.</exception>
        /// <exception cref="ArgumentOutOfRangeException">
        /// Die Filmlnge ist negativ.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Update(Movie movie)
        {

            // Prfe, ob ein Mitarbeiter eingeloggt ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException(
                    Resource.UpdateMovieNoUserLoggedOnException);

            // Prfe Parameter.
            if (movie == null)
                throw new ArgumentNullException("movie");
            if (String.IsNullOrEmpty(movie.Title))
                throw new ArgumentException(Resource.NullOrEmptyMovieTitle,
                    "movie");
            if (movie.Length <= 0)
                throw new ArgumentOutOfRangeException("movie",
                    Resource.GreaterThanZeroMovieLength);
            if (movie.Genres == null)
                throw new ArgumentException(Resource.NullMovieGenres, "movie");

            // Speichere die nderungen des Films auf dem Server.
            cinemaBusiness.ServiceAccess.UpdateMovie(movie);
        }

        /// <summary>
        /// Lscht einen Film vom Server.
        /// </summary>
        /// <param name="movieInfo">Eine MovieInfo-Instanz, dessen Film vom 
        /// Server gelscht werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// movieInfo ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Delete(MovieInfo movieInfo)
        {

            // Prfe, ob ein Mitarbeiter eingeloggt ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (movieInfo == null)
                throw new ArgumentNullException("movieInfo");

            // Lsche Film vom Server.
            cinemaBusiness.ServiceAccess.DeleteMovie(movieInfo);
        }

        /// <summary>
        /// Lscht einen Film vom Server.
        /// </summary>
        /// <param name="movie">Eine Movie-Instanz, dessen Film vom 
        /// Server gelscht werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// movie ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Delete(Movie movie)
        {

            // Prfe, ob ein Mitarbeiter eingeloggt ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (movie == null)
                throw new ArgumentNullException("movie");

            // Lsche Film vom Server.
            cinemaBusiness.ServiceAccess.DeleteMovie(movie);
        }

    }

}
