using System;
using System.Collections.Generic;
using System.Text;

using AdventureWorks.Cinema.ServiceAccessLayer;
using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.SharedBusinessLogic
{
    /// <summary>
    /// Enthlt Methoden zur Verwaltung von News in AdventureWorks Cinema.
    /// </summary>
    public class NewsBusiness
    {

        // Deklariere Variable fr CinemaBusiness.
        private CinemaBusiness cinemaBusiness;

        /// <summary>
        /// Initialisiert eine neue NewsBusiness-Instanz mit der 
        /// angegebenen CinemaBuiness-Instanz.
        /// </summary>
        /// <param name="cinemaBusiness">Eine CinemaBusiness-Instanz, die von 
        /// dieser CountryBusiness-Instanz verwendet werden soll.</param>
        internal NewsBusiness(CinemaBusiness cinemaBusiness)
        {

            // Speichere Referenz zum CinemaBusiness ab.
            this.cinemaBusiness = cinemaBusiness;

        }

        /// <summary>
        /// Rufe alle News vom Server ab.
        /// </summary>
        /// <returns>Liste mit NewsItem-Instanzen.</returns>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<NewsItem> NewsItems
        {
            get
            {
                return new List<NewsItem>(
                    cinemaBusiness.ServiceAccess.GetNewsItems());
            }
        }

        /// <summary>
        /// Rufe alle Kategorien vom Server ab.
        /// </summary>
        /// <returns>Liste mit NewsCategory-Instanzen.</returns>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<NewsCategory> Categories
        {
            get
            {
                return new List<NewsCategory>(
                    cinemaBusiness.ServiceAccess.GetNewsCategories());
            }
        }

        /// <summary>
        /// Rufe alle News vom Server ab, die der angegebenen Kategorie 
        /// entsprechen.
        /// </summary>
        /// <param name="numbers">Ein Feld mit Integers, fr die die News 
        /// abgerufen werden.</param>
        /// <returns>Liste mit NewsItem-Instanzen.</returns>
        /// <exception cref="ArgumentNullException">
        /// numbers ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Das Feld numbers ist leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        private List<NewsItem> GetByCategory(int[] numbers)
        {

            // Prfe Parameter.
            if (numbers == null)
                throw new ArgumentNullException("numbers");
            if (numbers.Length == 0)
                throw new ArgumentException("numbers");

            return new List<NewsItem>(
                cinemaBusiness.ServiceAccess.GetNewsItemsByCategories(numbers));

        }

        /// <summary>
        /// Rufe alle News vom Server ab, die der angegebenen Kategorie 
        /// entsprechen.</summary>
        /// <param name="category">Eine Kategorie, fr die die News abgerufen 
        /// werden sollen.</param>
        /// <returns>Liste mit NewsItem-Instanzen.</returns>
        /// <exception cref="ArgumentNullException">
        /// category ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<NewsItem> GetByCategory(NewsCategory category)
        {

            // Check the parameters.
            if (category == null)
                throw new ArgumentNullException("category");

            return GetByCategory(new int[] { category.Number });

        }

        /// <summary>
        /// Rufe alle News vom Server ab, die den angegebenen Kategorien 
        /// entsprechen.</summary>
        /// <param name="categories">Die Kategorien, fr die die News 
        /// abgerufen werden sollen.</param>
        /// <returns>Liste mit NewsItem-Instanzen.</returns>
        /// <exception cref="ArgumentNullException">
        /// categories ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Das Feld categories ist leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<NewsItem> GetByCategory(List<NewsCategory> categories)
        {

            // Prfe Parameter.
            if (categories == null)
                throw new ArgumentNullException("categories");
            if (categories.Count == 0)
                throw new ArgumentException("categories");

            // Konvertiere Kategorien-Liste in ein Integer-Feld.
            int[] numbers = new int[categories.Count];
            for (int i = 0; i < categories.Count; i++)
                numbers[i] = categories[i].Number;

            return GetByCategory(numbers);

        }

        /// <summary>
        /// Creates a new news item with the given data on the server.
        /// Erstellt eine neue Nachricht mit den angegebenen Daten auf dem 
        /// Server.</summary>
        /// <param name="subject">Die berschrift der neuen Nachricht.</param>
        /// <param name="text">Der Text der neuen Nachricht.</param>
        /// <param name="categories">Die Kategorien, zu der die neue Nachricht 
        /// gehrt.</param>
        /// <param name="startDate">Das Startdatum, ab dem die Nachricht gltig 
        /// ist.</param>
        /// <param name="endDate">Das Enddatum, bis zu dem die Nachricht gltig 
        /// ist.</param>
        /// <returns>Eine NewsItem-Instanz, die erstellt wurde.</returns>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentException">
        /// Die Liste categories ist NULL oder leer.
        /// - oder -
        /// Das Start- oder Endatum ist Null (0).</exception>
        /// <exception cref="EndBeforeStartException">
        /// Das Enddatum ist vor dem Startdatum.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        /// <remarks>Diese Methode folgt dem Factory-Muster.</remarks>
        public NewsItem Create(string subject, string text,
            List<NewsCategory> categories, DateTime startDate, DateTime endDate)
        {

            // Erzeuge lokal eine NewsItem-Instanz und speichere die Daten.
            NewsItem newsItem = new NewsItem();
            newsItem.Id = Guid.NewGuid();
            newsItem.Subject = subject;
            newsItem.Text = text;
            newsItem.Categories = categories.ToArray();
            newsItem.StartDate = startDate;
            newsItem.EndDate = endDate;

            // Erstelle eine neue Nachricht auf dem Server.
            Create(newsItem);

            return newsItem;

        }

        /// <summary>
        /// Fgt eine lokal vorhandene Nachricht dem Server hinzu.
        /// </summary>
        /// <param name="newsItem">Eine NewsItem-Instanz, die auf dem Server 
        /// hinzugefgt werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// newsItem ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Die Eigenschaft Categories von newsItem ist NULL oder leer.
        /// - oder -
        /// Das Start- oder Endatum  von newsItem ist Null (0).</exception>
        /// <exception cref="EndBeforeStartException">
        /// Das Enddatum ist vor dem Startdatum.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Create(NewsItem newsItem)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (newsItem == null)
                throw new ArgumentNullException("newsItem");
            if (newsItem.Categories == null)
                throw new ArgumentException(
                    Resource.NullNewsItemCategories, "newsItem");
            if (newsItem.Categories.Length == 0)
                throw new ArgumentException(
                    Resource.EmptyArrayNewsItemCategories, "newsItem");
            if (newsItem.StartDate.Ticks == 0)
                throw new ArgumentException(Resource.ZeroNewsItemStartDate,
                    "newsItem");
            if (newsItem.EndDate.Ticks == 0)
                throw new ArgumentException(Resource.ZeroNewsItemEndDate,
                    "newsItem");
            if (newsItem.EndDate < newsItem.StartDate)
                throw new EndBeforeStartException(
                    Resource.NewsItemEndBeforeStartException);

            // Erstelle Nachricht auf dem Server.
            cinemaBusiness.ServiceAccess.CreateNewsItem(newsItem);
        }

        /// <summary>
        /// Creates a new news category with the given data on the server.
        /// Erstellt auf dem Server eine neue Nachrichten-Kategorie mit den 
        /// angegebenen Daten.
        /// </summary>
        /// <param name="name">Name der Kategorie.</param>
        /// <param name="number">Nummer der Kategorie.</param>
        /// <returns>Die neue Nachrichten-Kategorie.</returns>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentException">
        /// name ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        /// <remarks>Diese Methode folgt dem Factory-Muster.</remarks>
        public NewsCategory CreateCategory(string name, int number)
        {

            // Erzeuge lokal eine NewsCategory-Instanz und speichere die Daten.
            NewsCategory newsCategory = new NewsCategory();
            newsCategory.Name = name;
            newsCategory.Number = number;

            // Erstelle die neue Kategorie auf dem Server.
            CreateCategory(newsCategory);

            return newsCategory;

        }

        /// <summary>
        /// Fgt eine lokal vorhandene Nachrichten-Kategorie auf dem Server 
        /// hinzu.</summary>
        /// <param name="newsCategory">Eine Nachrichten-Kategorie, die auf dem 
        /// Server hinzugefgt werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// newsCategory ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Die Name-Eigenschaft von newsCategory ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void CreateCategory(NewsCategory newsCategory)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException(
                    Resource.CreateNewsCategoryNoUserLoggedOnException);

            // Prfe Parameter.
            if (newsCategory == null)
                throw new ArgumentNullException("newsCategory");
            if (String.IsNullOrEmpty(newsCategory.Name))
                throw new ArgumentException(
                    Resource.NullOrEmptyNewsCategoryName, "newsCategory");

            // Erstelle Nachrichten-Kategorie auf dem Server.
            cinemaBusiness.ServiceAccess.CreateNewsCategory(newsCategory);
        }

        /// <summary>
        /// ndert eine vorhandene Nachricht auf dem Server.
        /// </summary>
        /// <param name="newsItem">Die Nachricht, die auf dem Server gendert 
        /// werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// newsItem ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Die Categories-Eigenschaft von newsItem ist leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Update(NewsItem newsItem)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException(
                    Resource.UpdateNewsItemNoUserLoggedOnException);

            // Prfe Parameter.
            if (newsItem == null)
                throw new ArgumentNullException("newsItem");
            if (newsItem.Categories == null)
                throw new ArgumentException(Resource.NullNewsItemCategories,
                    "newsItem");
            if (newsItem.Categories.Length == 0)
                throw new ArgumentException(
                    Resource.EmptyArrayNewsItemCategories, "newsItem");
            if (newsItem.StartDate.Ticks == 0)
                throw new ArgumentException(
                    Resource.ZeroNewsItemStartDate, "newsItem");
            if (newsItem.EndDate.Ticks == 0)
                throw new ArgumentException(
                    Resource.ZeroNewsItemEndDate, "newsItem");

            // ndere die Nachricht auf dem Server.
            cinemaBusiness.ServiceAccess.UpdateNewsItem(newsItem);
        }

        /// <summary>
        /// ndert eine vorhandene Nachrichten-Kategorie auf dem Server.
        /// </summary>
        /// <param name="newsCategory">Eine Nachrichten-Kategorie, die auf dem 
        /// Server gendert werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// newsCategory ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Die Name-Eigenschaft von newsCategory ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void UpdateCategory(NewsCategory newsCategory)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException(
                    Resource.UpdateNewsCategoryNoUserLoggedOnException);

            // Prfe Parameter.
            if (newsCategory == null)
                throw new ArgumentNullException("newsCategory");
            if (String.IsNullOrEmpty(newsCategory.Name))
                throw new ArgumentException(
                    Resource.NullOrEmptyNewsCategoryName, "newsCategory");

            // ndere Nachrichten-Kategorie auf dem Server.
            cinemaBusiness.ServiceAccess.UpdateNewsCategory(newsCategory);
        }

        /// <summary>
        /// Lscht eine Nachricht vom Server.
        /// </summary>
        /// <param name="newsItem">Eine Nachricht, die vom Server gelscht 
        /// werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// newsItem ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Delete(NewsItem newsItem)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException(
                    Resource.DeleteNewsItemNoUserLoggedOnException);

            // Prfe Parameter.
            if (newsItem.Categories == null)
                throw new ArgumentException(Resource.NullNewsItemCategories,
                    "newsItem");

            // Lsche Nachricht vom Server.
            cinemaBusiness.ServiceAccess.DeleteNewsItem(newsItem);
        }

        /// <summary>
        /// Lscht eine Nachrichten-Kategorie vom Server.
        /// </summary>
        /// <param name="newsCategory">Eine Nachrichten-Kategorie, die vom 
        /// Server gelscht werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// newsCategory ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void DeleteCategory(NewsCategory newsCategory)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (newsCategory == null)
                throw new ArgumentNullException("newsCategory");

            // Lsche Nachrichten-Kategorie vom Server.
            cinemaBusiness.ServiceAccess.DeleteNewsCategory(newsCategory);
        }

    }

}
