using System;
using System.Collections.Generic;
using System.Text;

using AdventureWorks.Cinema.ServiceAccessLayer;
using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.SharedBusinessLogic
{
    /// <summary>
    /// Enthlt Methoden zur Verwaltung von Altersfreigaben in
    /// AdventureWorks Cinema.
    /// </summary>
    public class ParentalGuidanceBusiness
    {

        // Deklariere Variable fr CinemaBusiness.
        private CinemaBusiness cinemaBusiness;

        /// <summary>
        /// Initialisiert eine neue ParentalGuidanceBusiness-Instanz mit der 
        /// angegebenen CinemaBuiness-Instanz.
        /// </summary>
        /// <param name="cinemaBusiness">Eine CinemaBusiness-Instanz, die von 
        /// dieser CountryBusiness-Instanz verwendet werden soll.</param>
        internal ParentalGuidanceBusiness(CinemaBusiness cinemaBusiness)
        {

            // Speichere Referenz zum CinemaBusiness ab.
            this.cinemaBusiness = cinemaBusiness;

        }

        /// <summary>
        /// Rufe alle Altersfreigaben vom Server ab.
        /// </summary>
        /// <returns>Liste mit Altersfreigaben.</returns>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<ParentalGuidance> ParentalGuidances
        {
            get
            {
                return new List<ParentalGuidance>(
                    cinemaBusiness.ServiceAccess.GetParentalGuidances());
            }
        }

        /// <summary>
        /// Erstellt auf dem Server eine neue Altersfreigaben mit dem 
        /// angegebenen Namen.</summary>
        /// <param name="name">ParentalGuidance name.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentException">
        /// name ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public ParentalGuidance Create(string name)
        {

            // Erzeuge lokal eine Altersfreigabe.
            ParentalGuidance parentalGuidance = new ParentalGuidance();
            parentalGuidance.Id = Guid.NewGuid();
            parentalGuidance.Name = name;

            // Erstelle eine Altersfreigabe auf dem Server.
            Create(parentalGuidance);

            return parentalGuidance;

        }

        /// <summary>
        /// Fge eine vorhandene Altersfreigabe auf dem Server hinzu.
        /// </summary>
        /// <param name="parentalGuidance">Eine Altersfreigabe, die auf dem 
        /// Server hinzugefgt werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// parentalGuidance ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Die Name-Eigenschaft von parentalGuidance ist NULL oder leer.
        /// </exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Create(ParentalGuidance parentalGuidance)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (parentalGuidance == null)
                throw new ArgumentNullException("parentalGuidance");
            if (String.IsNullOrEmpty(parentalGuidance.Name))
                throw new ArgumentException(
                    Resource.NullOrEmptyParentalGuidanceName,
                    "parentalGuidance");

            // Erstelle Altersfreigabe auf dem Server.
            cinemaBusiness.ServiceAccess.
                CreateParentalGuidance(parentalGuidance);
        }

        /// <summary>
        /// ndert eine Altersfreigabe auf dem Server.
        /// </summary>
        /// <param name="parentalGuidance">Eine Altersfreigabe, die auf dem 
        /// Server verndert werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// parentalGuidance ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Die Name-Eigenschaft von parentalGuidance ist NULL oder leer.
        /// </exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Update(ParentalGuidance parentalGuidance)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (parentalGuidance == null)
                throw new ArgumentNullException("parentalGuidance");
            if (String.IsNullOrEmpty(parentalGuidance.Name))
                throw new ArgumentException(
                    Resource.NullOrEmptyParentalGuidanceName, 
                    "parentalGuidance");

            // ndere Altersfreigabe auf dem Server.
            cinemaBusiness.ServiceAccess.
                UpdateParentalGuidance(parentalGuidance);
        }

        /// <summary>
        /// Lscht eine Altersfreigabe vom Server.
        /// </summary>
        /// <param name="parentalGuidance">Eine Altersfreigabe, die vom Server 
        /// gelscht werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// parentalGuidance ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Delete(ParentalGuidance parentalGuidance)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (parentalGuidance == null)
                throw new ArgumentNullException("parentalGuidance");

            // Lsche Altersfreigabe vom Server.
            cinemaBusiness.ServiceAccess.
                DeleteParentalGuidance(parentalGuidance);
        }

    }

}
