using System;
using System.Collections.Generic;
using System.Data;
using System.Globalization;
using System.Text;

using AdventureWorks.Cinema.ServiceAccessLayer;
using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.SharedBusinessLogic
{
    /// <summary>
    /// Enthlt Methoden zur Verwaltung von Vorstellungen in
    /// AdventureWorks Cinema.
    /// </summary>
    public class PerformanceBusiness
    {

        // Deklariere Variable fr CinemaBusiness.
        private CinemaBusiness cinemaBusiness;
        /// <summary>
        /// Initialisiert eine neue PerformanceBusiness-Instanz mit der 
        /// angegebenen CinemaBuiness-Instanz.
        /// </summary>
        /// <param name="cinemaBusiness">Eine CinemaBusiness-Instanz, die von 
        /// dieser CountryBusiness-Instanz verwendet werden soll.</param>
        internal PerformanceBusiness(CinemaBusiness cinemaBusiness)
        {

            // Speichere Referenz zum CinemaBusiness ab.
            this.cinemaBusiness = cinemaBusiness;

        }

        /// <summary>
        /// Ruft die Anzahl freier Sitze fr die angegebene Vorstellung vom 
        /// server ab.</summary>
        /// <param name="performance">Eine Vorstellung, fr die die Anzahl 
        /// freier Sitze zurckgegeben werden soll.</param>
        /// <returns>Anzahl freier Sitze fr die angegebene Vorstellung.
        /// </returns>
        /// <exception cref="ArgumentNullException">
        /// performance ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public int GetFreeSeats(Performance performance)
        {

            // Prfe Parameter.
            if (performance == null)
                throw new ArgumentNullException("performance");

            // Rufe die Anzahl freier Sitze fr die angegebene Vorstellung vom 
            // Server ab und gib diesen Wert zurck.
            return cinemaBusiness.ServiceAccess.GetFreeSeats(performance);
        }

        /// <summary>
        /// Ruft die Anzahl freier Sitze fr die angegebene Vorstellung vom 
        /// server ab.</summary>
        /// <param name="performanceInfo">Eine Vorstellung, fr die die Anzahl 
        /// freier Sitze zurckgegeben werden soll.</param>
        /// <returns>Anzahl freier Sitze fr die angegebene Vorstellung.
        /// </returns>
        /// <exception cref="ArgumentNullException">
        /// performanceInfo ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public int GetFreeSeats(PerformanceInfo performanceInfo)
        {

            // Prfe Parameter.
            if (performanceInfo == null)
                throw new ArgumentNullException("performanceInfo");

            // Rufe die Anzahl freier Sitze fr die angegebene Vorstellung vom 
            // Server ab und gib diesen Wert zurck.
            return cinemaBusiness.ServiceAccess.GetFreeSeats(performanceInfo);
        }

        /// <summary>
        /// Ruft eine Liste aller Vorstellungen vom Server ab.
        /// </summary>
        /// <returns>Liste aller Vorstellungen.</returns>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<PerformanceInfo> Performances
        {
            get
            {
                // Rufe alle Vorstellungen vom Server ab und konvertiere
                // das Feld in eine Liste.
                return new List<PerformanceInfo>(
                    cinemaBusiness.ServiceAccess.GetPerformances(
                    DateTime.MinValue, DateTime.MaxValue));
            }
        }

        /// <summary>
        /// Ruft alle Vorstellungen zu dem angegebenen Film vom Server ab.
        /// </summary>
        /// <param name="movie">Ein Film, fr den die Vorstellungen 
        /// abgerufen werden sollen.</param>
        /// <returns>Eine Liste mit Vorstellungen zu dem angegebenen Film.
        /// </returns>
        /// <exception cref="ArgumentNullException">
        /// movie ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<PerformanceInfo> GetByMovie(Movie movie)
        {

            // Prfe Parameter.
            if (movie == null)
                throw new ArgumentNullException("movie");

            // Rufe die Vorstellungen zu dem angegebenen Film ab und 
            // konvertiere das Feld in eine Liste.
            return new List<PerformanceInfo>(
                cinemaBusiness.ServiceAccess.GetPerformances(movie));
        }

        /// <summary>
        /// Ruft alle Vorstellungen zu dem angegebenen Film vom Server ab.
        /// </summary>
        /// <param name="movieInfo">Ein Film, fr den die Vorstellungen 
        /// abgerufen werden sollen.</param>
        /// <returns>Eine Liste mit Vorstellungen zu dem angegebenen Film.
        /// </returns>
        /// <exception cref="ArgumentNullException">
        /// movieInfo ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<PerformanceInfo> GetByMovie(MovieInfo movieInfo)
        {

            // Prfe Parameter.
            if (movieInfo == null)
                throw new ArgumentNullException("movieInfo");

            // Rufe die Vorstellungen zu dem angegebenen Film ab und 
            // konvertiere das Feld in eine Liste.
            return new List<PerformanceInfo>(
                cinemaBusiness.ServiceAccess.GetPerformances(movieInfo));
        }

        /// <summary>
        /// Ruft alle Vorstellungen vom Server ab, die innerhalb eines 
        /// bestimmten Zeitraums aufgefhrt werden.</summary>
        /// <param name="startDate">Startdatum, ab Vorstellungen abgerufen 
        /// werden sollen.</param>
        /// <param name="endDate">Enddatum, bis zu dem Vorstellungen 
        /// abgerufen werden sollen.</param>
        /// <returns>Eine Liste mit Vorstellungen, die innerhalb des 
        /// angegebenen Zeitraums aufgefhrt werden.</returns>
        /// <exception cref="ArgumentNullException">
        /// Das Start- oder das Enddatum sind Null (0).</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<PerformanceInfo> GetByTimeframe(
            DateTime startDate, DateTime endDate)
        {

            // Prfe Parameter.
            if (startDate == DateTime.MinValue)
                throw new ArgumentNullException("startDate");
            if (endDate == DateTime.MinValue)
                throw new ArgumentNullException("endDate");

            // Rufe alle Vorstellungen ab, die innerhalb des angegebenen 
            // Zeitraums aufgefhrt werden und konvertiere das Feld in eine 
            // Liste.
            return new List<PerformanceInfo>(cinemaBusiness.ServiceAccess.
                GetPerformances(startDate, endDate));
        }

        /// <summary>
        /// Ruft alle Vorstellungen vom Server ab, die innerhalb eines 
        /// bestimmten Zeitraums aufgefhrt werden.</summary>
        /// <param name="room">Ein Raum, in dem die Vorstellungen aufgefhrt 
        /// werden.</param>
        /// <param name="start">Startdatum, ab Vorstellungen abgerufen 
        /// werden sollen.</param>
        /// <param name="length">Eine Zeitdauer, fr die die Vorstellungen vom 
        /// Server abgerufen werden sollen.</param>
        /// <returns>Eine Liste mit Vorstellungen, die innerhalb des 
        /// angegebenen Zeitraums aufgefhrt werden.</returns>
        /// <exception cref="ArgumentNullException">
        /// room ist NULL.
        /// - oder -
        /// start oder length ist Null (0).</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<PerformanceInfo> GetByRoomAndTimeframe(Room room,
            DateTime start, TimeSpan length)
        {

            // Prfe Parameter.
            if (room == null)
                throw new ArgumentNullException("room");
            if (start.Ticks == 0)
                throw new ArgumentException("start");
            if (length.Ticks == 0)
                throw new ArgumentException("length");

            // Rufe alle Vorstellung ab, die in dem angegebenen Raum, zur 
            // angegebenen Zeit und innerhalb der Zeitdauer in dem Raum
            // aufgefhrt werden und konvertiere das Feld in eine Liste.
            return new List<PerformanceInfo>(cinemaBusiness.ServiceAccess.
                GetPerformances(room, start, start + length));
        }

        /// <summary>
        /// Ruft vom Server fr eine angegebene PerformanceInfo-Instanz die
        /// Performance-Instanz vom Server ab.
        /// </summary>
        /// <param name="performanceInfo">Eine PerformanceInfo-Instanz, fr die 
        /// die Performance-Instanz abgerufen werden soll.</param>
        /// <returns>Eine Performance-Instanz, die der angegebenen
        /// PerformanceInfo-Instanz entspricht.</returns>
        /// <exception cref="ArgumentNullException">
        /// performanceInfo ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public Performance Get(PerformanceInfo performanceInfo)
        {

            // Prfe Parameter.
            if (performanceInfo == null)
                throw new ArgumentNullException("performanceInfo");

            // Rufe die Performance-Instanz fr die angegebene 
            // PerformanceInfo-Instanz vom Server ab.
            return cinemaBusiness.ServiceAccess.GetPerformance(performanceInfo);
        }

        /// <summary>
        /// Ruft alle Vorstellungen sortiert nach Film und Zeit vom Server ab, 
        /// die in der nchsten Woche aufgefhrt werden.</summary>
        /// <returns>Eine sortierte Liste, die alle Filme und die entsprechenden 
        /// Vorstellungen der nchsten Woche enthlt.</returns>
        public SortedList<MovieInfo, DayPerformanceInfo[]>
            GetPerformancesPerMovieOfNextWeek()
        {
            DateTime startDate = DateTime.Now;
            DateTime endDate = startDate.AddDays(6);

            return GetPerformancesPerMovie(startDate, endDate);
        }

        /// <summary>
        /// Ruft alle Vorstellungen innerhalb des angegebenen Zeitraums sortiert 
        /// nach Film und Zeit vom Server ab.</summary>
        /// <param name="startDate">Startdatum, ab dem Vorstellungen abgerufen 
        /// werden sollen. Alle Vorstellungen des angegebenen Tages werden 
        /// mitzurckgegeben.</param>
        /// <param name="endDate">Enddatum, bis zu dem Vorstellungen abgerufen 
        /// werden sollen. Alle Vorstellungen des angegebenen Tages werden 
        /// mitzurckgegeben.</param>
        /// <returns>Eine sortierte Liste, die alle Filme und die entsprechenden 
        /// Vorstellungen des angegebenen Zeitraums enthlt.</returns>
        public SortedList<MovieInfo, DayPerformanceInfo[]>
            GetPerformancesPerMovie(DateTime startDate, DateTime endDate)
        {
            MovieInfoComparer movieComparer = new MovieInfoComparer();
            SortedList<MovieInfo, DayPerformanceInfo[]> result =
                new SortedList<MovieInfo, DayPerformanceInfo[]>(movieComparer);

            // Berechne die Anzahl an Tagen im Rckgabewert.
            // (Wird fr die Lnge des Feldes bentigt.)
            startDate = new DateTime(startDate.Year, startDate.Month,
                startDate.Day);
            endDate = new DateTime(endDate.Year, endDate.Month, endDate.Day);
            int dayCount = (endDate - startDate).Days + 1;

            // Iteriere alle Vorstellungen im angegebenen Zeitraum.
            List<PerformanceInfo> performances =
                GetByTimeframe(startDate, endDate);
            foreach(PerformanceInfo performance in performances)
            {
                // Rufe die DayPerformanceInfo-Instanz des aktuellen Films ab.
                DayPerformanceInfo[] dayInfos;
                if(!result.TryGetValue(performance.MovieInfo, out dayInfos))
                {
                    // Erstelle DayPerformanceInfo-Instanzen, sofern sie
                    // nicht existieren.
                    dayInfos = new DayPerformanceInfo[dayCount];
                    for(int i = 0; i < dayInfos.Length; i++)
                    {
                        dayInfos[i] =
                            new DayPerformanceInfo(startDate.AddDays(i));
                    }

                    // Fge die DayPerformanceInfo-Instanz zum Rckgabewert
                    // hinzu.
                    result.Add(performance.MovieInfo, dayInfos);
                }

                // Rufe die richtige DayPerformanceInfo-Instanz aus dem Feld ab.
                int dayInfoIndex = (performance.Date - startDate).Days;
                DayPerformanceInfo dayInfo = dayInfos[dayInfoIndex];

                // Fge die Vorstellung an der richtigen Position (sortiert nach 
                // Datum) ein. Dies ist sehr einfach mit einer sortierten Liste
                // (SortedList). Da jedoch mehrere Vorstellungen zur gleichen 
                // Zeit stattfinden knnen, kann eine sortierte Liste nicht 
                // verwendet werden.

                // Iteriere durch alle vorhandenen Vorstellungen des aktuellen 
                // Films am aktuellen Tag.
                int index = 0;
                for(; index < dayInfo.Performances.Count; index++)
                {
                    // Beende die Iteration, wenn eine Vorstellungen gefunden 
                    // wurde, die spter stattfindet als die aktuelle.
                    if(dayInfo.Performances[index].Date > performance.Date)
                        break;
                }

                // Fge die Vorstellung an der richtigen Position ein.
                dayInfo.Performances.Insert(index, performance);
            }

            return result;
        }

        /// <summary>
        /// Erstellt eine neue Vorstellung mit den angegebenen Daten auf dem 
        /// Server.</summary>
        /// <returns>Die Vorstellung, die erstellt wurde.</returns>
        /// <remarks>Diese Methode folgt dem Factory-Muster.</remarks>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentException">
        /// movie oder room ist NULL.
        /// - oder -
        /// date ist Null (0).</exception>
        /// <exception cref="ArgumentOutOfRangeException">
        /// price ist negativ.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public Performance Create(Movie movie, DateTime date, Room room,
            decimal price)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Erstelle lokal die Vorstellung und speichere die Daten.
            Performance performance = new Performance();
            performance.Id = Guid.NewGuid();
            performance.Movie = movie;
            performance.Date = date;
            performance.Room = room;
            performance.Price = price;

            // Erstelle die Vorstellung auf dem Server.
            Create(performance);

            return performance;

        }

        /// <summary>
        /// Erstellt eine neue Vorstellung auf dem Server.
        /// </summary>
        /// <param name="performance">Die Vorstellung, die auf dem Server 
        /// erstellt werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// performance ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Die Movie-Eigenschaft oder die Room-Eigenschaft von performance 
        /// ist NULL.
        /// - oder -
        /// Die Date-Eigenschaft von performance ist Null (0).</exception>
        /// <exception cref="ArgumentOutOfRangeException">
        /// Die Price-Eigenschaft von performance ist negativ.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Create(Performance performance)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException(
                    Resource.CreatePerformanceNoUserLoggedOnException);

            // Prfe Parameter.
            if (performance == null)
                throw new ArgumentNullException("performance");
            if (performance.Movie == null)
                throw new ArgumentException(Resource.NullPerformanceMovie,
                    "performance");
            if (performance.Date.Ticks == 0)
                throw new ArgumentOutOfRangeException("performance",
                    Resource.ZeroPerformanceDate);
            if (performance.Room == null)
                throw new ArgumentException(Resource.NullPerformanceRoom,
                    "performance");
            if (performance.Price <= 0)
                throw new ArgumentOutOfRangeException("performance",
                    Resource.GreaterThanZeroPerformancePrice);

            // Erstelle die Vorstellung auf dem Server.
            cinemaBusiness.ServiceAccess.CreatePerformance(performance);
        }

        /// <summary>
        /// ndere eine vorhandene Vorstellung auf dem Server.
        /// </summary>
        /// <param name="performance">Die Vorstellung, die auf dem Server 
        /// gendert werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// performance ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Die Movie-Eigenschaft oder die Room-Eigenschaft von performance 
        /// ist NULL.
        /// - oder -
        /// Die Date-Eigenschaft von performance ist Null (0).</exception>
        /// <exception cref="ArgumentOutOfRangeException">
        /// Die Price-Eigenschaft von performance ist negativ.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Update(Performance performance)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException(
                    Resource.UpdatePerformanceNoUserLoggedOnException);

            // Prfe Parameter.
            if (performance == null)
                throw new ArgumentNullException("performance");
            if (performance.Movie == null)
                throw new ArgumentException(Resource.NullPerformanceMovie,
                    "performance");
            if (performance.Date.Ticks == 0)
                throw new ArgumentException(Resource.ZeroPerformanceDate,
                    "performance");
            if (performance.Room == null)
                throw new ArgumentException(Resource.NullPerformanceRoom,
                    "performance");
            if (performance.Price <= 0)
                throw new ArgumentOutOfRangeException("performance",
                    Resource.GreaterThanZeroPerformancePrice);

            // ndere die Vorstellung auf dem Server.
            cinemaBusiness.ServiceAccess.UpdatePerformance(performance);
        }

        /// <summary>
        /// Lscht eine Vorstellung vom Server.
        /// </summary>
        /// <param name="performanceInfo">Die Vorstellung, die vom Server 
        /// gelscht werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// performanceInfo ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Delete(PerformanceInfo performanceInfo)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (performanceInfo == null)
                throw new ArgumentNullException("performanceInfo");

            // Lsche die Vorstellung vom Server.
            cinemaBusiness.ServiceAccess.DeletePerformance(performanceInfo);
        }

        /// <summary>
        /// Lscht eine Vorstellung vom Server.
        /// </summary>
        /// <param name="performance">Die Vorstellung, die vom Server 
        /// gelscht werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// performance ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Delete(Performance performance)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (performance == null)
                throw new ArgumentNullException("performance");

            // Lsche die Vorstellung vom Server.
            cinemaBusiness.ServiceAccess.DeletePerformance(performance);
        }

    }

    /// <summary>
    /// Vergleichsklasse fr <see cref="MovieInfo"/>-Objekte.
    /// Diese Klasse wird fr die Sortierung von MovieInfo-Objekten bentigt.
    /// </summary>
    public class MovieInfoComparer : IComparer<MovieInfo>
    {
        /// <summary>
        /// Vergleicht zwei Objekte und gibt einen Wert zurck, der angibt, 
        /// ob ein Objekt kleiner, gleich oder grer ist.
        /// </summary>
        /// <param name="x">Das erste Objekt im Vergleich.</param>
        /// <param name="y">Das zweite Objekt im Vergleich.</param>
        /// <returns>
        /// Negativ wenn x kleiner y ist.
        /// - oder -
        /// Null (0), wenn x gleich y ist.
        /// - oder -
        /// Positiv, wenn x grer y ist.
        /// </returns>
        public int Compare(MovieInfo x, MovieInfo y)
        {
            // Wenn beide Referenzen NULL sind, sind sie gleich.
            if(x == null && y == null)
                return 0;

            // Wenn nur eine NULL ist, dann sind sie nicht gleich.
            if(x == null)
                return -1;
            if(y == null)
                return 1;

            // Vergleiche nur den Titel der Filme.
            return x.Title.CompareTo(y.Title);
        }
    }

    /// <summary>
    /// Container fr ein Datum und alle Vorstellungen an diesem Datum.
    /// </summary>
    public class DayPerformanceInfo
    {
        private DateTime date;
        private List<PerformanceInfo> performances;

        /// <summary>
        /// Initialisiert eine neue DayPerformanceInfo-Instanz mit dem 
        /// angegebenen Datum.
        /// </summary>
        /// <param name="date">Das Datum, fr das diese Instanz Vorstellungen 
        /// enthalten wird.</param>
        public DayPerformanceInfo(DateTime date)
        {
            this.date = new DateTime(date.Year, date.Month, date.Day);
            this.performances = new List<PerformanceInfo>();
        }

        /// <summary>
        /// Datum, fr das diese Instanz Vorstellungen enthlt.
        /// </summary>
        public DateTime Date
        {
            get
            {
                return date;
            }
        }

        /// <summary>
        /// Vorstellungen, die am angegebenen Datum stattfinden.
        /// </summary>
        public List<PerformanceInfo> Performances
        {
            get
            {
                return performances;
            }
        }
    }
}
