using System;
using System.Collections.Generic;
using System.Text;

using AdventureWorks.Cinema.ServiceAccessLayer;
using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.SharedBusinessLogic
{
    /// <summary>
    /// Enthlt Methoden zur Verwaltung von Personen in AdventureWorks Cinema.
    /// </summary>
    public class PersonBusiness
    {

        // Deklariere Variable fr CinemaBusiness.
        private CinemaBusiness cinemaBusiness;

        /// <summary>
        /// Initialisiert eine neue PersonBusiness-Instanz mit der 
        /// angegebenen CinemaBuiness-Instanz.
        /// </summary>
        /// <param name="cinemaBusiness">Eine CinemaBusiness-Instanz, die von 
        /// dieser CountryBusiness-Instanz verwendet werden soll.</param>
        internal PersonBusiness(CinemaBusiness cinemaBusiness)
        {

            // Speichere Referenz zum CinemaBusiness ab.
            this.cinemaBusiness = cinemaBusiness;

        }

        /// <summary>
        /// Ruft alle Personen vom Server ab.
        /// </summary>
        /// <returns>Liste mit Personen.</returns>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<Person> Persons
        {
            get
            {
                return new List<Person>(
                    cinemaBusiness.ServiceAccess.GetPersons());
            }
        }

        /// <summary>
        /// Erstellt eine neue Person auf dem Server.
        /// </summary>
        /// <param name="name">Name der Person.</param>
        public Person Create(string name)
        {

            // Erstelle lokal eine Person.
            Person person = new Person();
            person.Id = Guid.NewGuid();
            person.Name = name;

            // Erstelle die Person auf dem Server.
            Create(person);

            return person;

        }

        /// <summary>
        /// Fgt eine vorhandene Person auf dem Server hinzu.
        /// </summary>
        /// <param name="person">Die Person, die auf dem Server hinzugefgt 
        /// werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// person ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Die Name-Eigenschaft von person ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Create(Person person)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (person == null)
                throw new ArgumentNullException("person");
            if (String.IsNullOrEmpty(person.Name))
                throw new ArgumentException(Resource.NullOrEmptyPersonName,
                    "person");

            // Erstelle die Person auf dem Server.
            cinemaBusiness.ServiceAccess.CreatePerson(person);
        }

        /// <summary>
        /// ndert eine Person auf dem Server.
        /// </summary>
        /// <param name="person">Die Person, die gendert werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// person ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Die Name-Eigenschaft ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Update(Person person)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (person == null)
                throw new ArgumentNullException("person");
            if (String.IsNullOrEmpty(person.Name))
                throw new ArgumentException(Resource.NullOrEmptyPersonName,
                    "person");

            // ndere die Person auf dem Server.
            cinemaBusiness.ServiceAccess.UpdatePerson(person);
        }

        /// <summary>
        /// Lscht eine Person vom Server.
        /// </summary>
        /// <param name="person">Die Person, die vom Server gelscht werden 
        /// soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// person ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Delete(Person person)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (person == null)
                throw new ArgumentNullException("person");

            // Lsche die Person vom Server.
            cinemaBusiness.ServiceAccess.DeletePerson(person);
        }

    }

}