using System;
using System.Collections.Generic;
using System.Text;

using AdventureWorks.Cinema.ServiceAccessLayer;
using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.SharedBusinessLogic
{
    /// <summary>
    /// Enthlt Methoden zur Verwaltung von Rumen in AdventureWorks Cinema.
    /// </summary>
    public class RoomBusiness
    {

        // Deklariere Variable fr CinemaBusiness.
        private CinemaBusiness cinemaBusiness;

        /// <summary>
        /// Initialisiert eine neue RoomBusiness-Instanz mit der 
        /// angegebenen CinemaBuiness-Instanz.
        /// </summary>
        /// <param name="cinemaBusiness">Eine CinemaBusiness-Instanz, die von 
        /// dieser CountryBusiness-Instanz verwendet werden soll.</param>
        internal RoomBusiness(CinemaBusiness cinemaBusiness)
        {

            // Speichere Referenz zum CinemaBusiness ab.
            this.cinemaBusiness = cinemaBusiness;

        }

        /// <summary>
        /// Rufe alle Rume vom Server ab.
        /// </summary>
        /// <returns>Liste mit Rumen.</returns>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public List<Room> Rooms
        {
            get
            {

                return new List<Room>(cinemaBusiness.ServiceAccess.GetRooms());

            }
        }

        /// <summary>
        /// Erstellt einen neuen Raum auf dem Server.
        /// </summary>
        /// <param name="name">Der Name des neuen Raums.</param>
        /// <param name="seatCount">Die Anzahl der Sitzpltze im neuen Raum.
        /// </param>
        /// <param name="screenDiagonal">Die Diagonale der Leinwand im neuen 
        /// Raum.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentException">
        /// name ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public Room Create(string name, int seatCount, int screenDiagonal)
        {

            // Erstelle lokal einen Raum und speichere die Daten.
            Room room = new Room();
            room.Id = Guid.NewGuid();
            room.Name = name;
            room.SeatCount = seatCount;
            room.ScreenDiagonal = screenDiagonal;

            // Erstelle den Raum auf dem Server.
            Create(room);

            return room;

        }

        /// <summary>
        /// Fgt einen vorhandenen Raum auf dem Server hinzu.
        /// </summary>
        /// <param name="room">Der Raum, der auf dem Server hinzu gefgt werden 
        /// soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Thrown when no employee is logged on.</exception>
        /// <exception cref="ArgumentNullException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentException">
        /// Die Name-Eigenschaft von room ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Create(Room room)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException(
                    Resource.CreateRoomNoUserLoggedOnException);

            // Prfe Parameter.
            if (room == null)
                throw new ArgumentNullException("room");
            if (String.IsNullOrEmpty(room.Name))
                throw new ArgumentException(Resource.NullOrEmptyRoomName,
                    "room");
            if (room.SeatCount <= 0)
                throw new ArgumentException(
                    Resource.GreaterThanZeroRoomSeatCount, "room");

            // Erstelle den Raum auf dem Server.
            cinemaBusiness.ServiceAccess.CreateRoom(room);
        }

        /// <summary>
        /// ndert einen Raum auf dem Server.
        /// </summary>
        /// <param name="room">Der Raum, der auf dem Server verndert werden 
        /// soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// room ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// Die Name-Eigenschaft von room ist NULL oder leer.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Update(Room room)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException(
                    Resource.UpdateRoomNoUserLoggedOnException);

            // Prfe Parameter.
            if (room == null)
                throw new ArgumentNullException("room");
            if (String.IsNullOrEmpty(room.Name))
                throw new ArgumentException(Resource.NullOrEmptyRoomName,
                    "room");
            if (room.SeatCount <= 0)
                throw new ArgumentException(
                    Resource.GreaterThanZeroRoomSeatCount, "room");

            // ndere den Raum auf dem Server.
            cinemaBusiness.ServiceAccess.UpdateRoom(room);
        }

        /// <summary>
        /// Lscht einen Raum vom Server.
        /// </summary>
        /// <param name="room">Der Raum, der vom Server gelscht werden soll.
        /// </param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// room ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Delete(Room room)
        {

            // Prfe, ob ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (room == null)
                throw new ArgumentNullException("room");

            // Lsche den Raum vom Server.
            cinemaBusiness.ServiceAccess.DeleteRoom(room);
        }

    }

}
