using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

using AdventureWorks.Cinema.ServiceAccessLayer;
using AdventureWorks.Cinema.ServiceAccessLayer.WebService;

namespace AdventureWorks.Cinema.SharedBusinessLogic
{
    /// <summary>
    /// Enthlt Methoden zur Verwaltung von Tickets in AdventureWorks Cinema.
    /// </summary>
    public class TicketBusiness
    {

        // Deklariere Variable fr CinemaBusiness.
        private CinemaBusiness cinemaBusiness;

        /// <summary>
        /// Initialisiert eine neue TicketBusiness-Instanz mit der 
        /// angegebenen CinemaBuiness-Instanz.
        /// </summary>
        /// <param name="cinemaBusiness">Eine CinemaBusiness-Instanz, die von 
        /// dieser CountryBusiness-Instanz verwendet werden soll.</param>
        internal TicketBusiness(CinemaBusiness cinemaBusiness)
        {

            // Speichere Referenz zum CinemaBusiness ab.
            this.cinemaBusiness = cinemaBusiness;

        }

        /// <summary>
        /// Erstellt neue Tickets auf dem Server.
        /// </summary>
        /// <param name="performance">Eine Vorstellung, fr die Tickets 
        /// angefordert werden.</param>
        /// <param name="quantity">Die Anzahl an bentigten Tickets.</param>
        /// <param name="accountNumber">Die Kontonummer, die fr den Kauf der 
        /// Tickets verwendet wird.</param>
        /// <param name="bankNumber">Die Bankleitzahl, die fr den Kauf der 
        /// Tickets verwendet wird.</param>
        /// <returns>Ein Ticket.</returns>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Kunde oder Mitarbeiter angemeldet.</exception>
        /// <exception cref="NotEnoughFreeSeatsException">
        /// Es sind nicht ausreichend Tickets in der gewnschten Anzahl 
        /// verfgbar.</exception>
        /// <exception cref="ArgumentNullException">
        /// performance ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// quantity ist negativ.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        /// <remarks> Bankdaten werden vom Service Access Layer nicht 
        /// verarbeitet. Daher werden diese Daten hier ignoriert.</remarks>
        public Ticket Buy(Performance performance, int quantity, 
            string accountNumber, string bankNumber)
        {

            // Prfe, ob ein Kunde oder ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Customers.IsLoggedOn &&
                !cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe die Anzahl freier Sitzpltze (Anzahl verfgbarer Tickets).
            if (cinemaBusiness.Performances.GetFreeSeats(performance) <
                quantity)
                throw new NotEnoughFreeSeatsException();

            // Prfe Parameter.
            if (performance == null)
                throw new ArgumentNullException("performance");
            if (quantity <= 0)
                throw new ArgumentOutOfRangeException("quantity");

            Ticket ticket;
            string code;

            // Erstelle Tickets auf dem Server. (Kauf)
            code = cinemaBusiness.ServiceAccess.BuyTicket(
                performance, quantity);
            ticket = new Ticket();
            ticket.TicketCode = code;
            ticket.Performance = performance;
            ticket.Quantity = quantity;
            return ticket;
        }

        /// <summary>
        /// Erstellt neue Tickets auf dem Server.
        /// </summary>
        /// <param name="performance">Eine Vorstellung, fr die Tickets 
        /// angefordert werden.</param>
        /// <param name="quantity">Die Anzahl an bentigten Tickets.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Kunde oder Mitarbeiter angemeldet.</exception>
        /// <exception cref="NotEnoughFreeSeatsException">
        /// Es sind nicht ausreichend Tickets in der gewnschten Anzahl 
        /// verfgbar.</exception>
        /// <exception cref="ArgumentNullException">
        /// performance ist NULL.</exception>
        /// <exception cref="ArgumentException">
        /// quantity ist negativ.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Sell(Performance performance, int quantity)
        {

            // Prfe, ob ein Kunde oder ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Customers.IsLoggedOn &&
                !cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe die Anzahl freier Sitzpltze (Anzahl verfgbarer Tickets).
            if (cinemaBusiness.Performances.GetFreeSeats(performance) <
                quantity)
                throw new NotEnoughFreeSeatsException();

            // Prfe Parameter.
            if (performance == null)
                throw new ArgumentNullException("performance");
            if (quantity <= 0)
                throw new ArgumentOutOfRangeException("quantity");

            // Erstelle Tickets auf dem Server. (Verkauf)
            cinemaBusiness.ServiceAccess.SellTicket(performance, quantity);
        }

        /// <summary>
        /// Rufe alle nicht entwerteten Tickets mit einem bestimmten Ticket-Code 
        /// vom Server ab.
        /// </summary>
        /// <param name="ticketCode">Der Ticket-Code, dessen Tickets gesucht 
        /// werden.</param>
        /// <returns>Eine Liste mit nicht entwerteten Tickets.</returns>
        public List<Ticket> GetUndispatchedTickets(string ticketCode)
        {

            // Prfe, ob ein Kunde oder ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Customers.IsLoggedOn &&
               !cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (String.IsNullOrEmpty(ticketCode))
                throw new ArgumentException(Resource.NullOrEmptyTicketCode);

            return new List<Ticket>(
                cinemaBusiness.ServiceAccess.GetUndispatchedTickets(
                ticketCode));
        }

        /// <summary>
        /// Entwertet ein Ticket auf dem Server.
        /// </summary>
        /// <param name="ticket">Das Ticket, das entwertet werden soll.</param>
        /// <exception cref="NoUserLoggedOnException">
        /// Es ist kein Kunde oder Mitarbeiter angemeldet.</exception>
        /// <exception cref="ArgumentNullException">
        /// ticket ist NULL.</exception>
        /// <exception cref="ServiceAccessException">
        /// Der SAL kann nicht auf seinen Anbieter zugreifen.</exception>
        public void Dispatch(Ticket ticket)
        {

            // Prfe, ob ein Kunde oder ein Mitarbeiter angemeldet ist.
            if (!cinemaBusiness.Customers.IsLoggedOn &&
                !cinemaBusiness.Employees.IsLoggedOn)
                throw new NoUserLoggedOnException();

            // Prfe Parameter.
            if (ticket == null)
                throw new ArgumentNullException("ticket");

            cinemaBusiness.ServiceAccess.MarkTicketAsDispatched(ticket);
        }

    }

}